<?php

namespace CorporateTrainingBundle\Biz\Course\Service\Impl;

use AppBundle\Common\ArrayToolkit;
use Biz\Course\Service\Impl\CourseServiceImpl as BaseService;
use Codeages\Biz\Framework\Event\Event;

class CourseServiceImpl extends BaseService
{
    /**
     * @param $id
     * @param $fields
     *
     * @return mixed
     *
     * @throws \Codeages\Biz\Framework\Service\Exception\InvalidArgumentException
     *                                                                            原营销设置中增加任务规则的修改，此处复写updateCourseMarketing中过滤方法增加enableFinish字段
     */
    public function updateCourseMarketing($id, $fields)
    {
        $oldCourse = $this->tryManageCourse($id);

        $fields = ArrayToolkit::parts(
            $fields,
            array(
                'enableFinish',
                'isFree',
                'originPrice',
                'vipLevelId',
                'buyable',
                'tryLookable',
                'tryLookLength',
                'watchLimit',
                'buyExpiryTime',
                'showServices',
                'services',
                'approval',
                'coinPrice',
                'expiryMode', //days、end_date、date、forever
                'expiryDays',
                'expiryStartDate',
                'expiryEndDate',
                'taskRewardPoint',
                'rewardPoint',
            )
        );

        if ($oldCourse['status'] == 'published' || $oldCourse['status'] == 'closed') {
            unset($fields['expiryMode']);
            unset($fields['expiryDays']);
            unset($fields['expiryStartDate']);
            unset($fields['expiryEndDate']);
        }

        $requireFields = array('isFree', 'buyable');
        $courseSet = $this->getCourseSetService()->getCourseSet($oldCourse['courseSetId']);
        if ($courseSet['type'] == 'normal' && $this->isCloudStorage()) {
            array_push($requireFields, 'tryLookable');
        } else {
            $fields['tryLookable'] = 0;
        }

        if (!ArrayToolkit::requireds($fields, $requireFields)) {
            throw $this->createInvalidArgumentException('Lack of required fields');
        }

        $fields = $this->validateExpiryMode($fields);

        $fields = $this->processFields($id, $fields, $courseSet);

        $newCourse = $this->getCourseDao()->update($id, $fields);

        $this->dispatchEvent('course.update', new Event($newCourse));
        $this->dispatchEvent('course.marketing.update', array('oldCourse' => $oldCourse, 'newCourse' => $newCourse));

        return $newCourse;
    }

    /**
     * @return 训版所有课程都是免费暂无试看功能
     */
    protected function isCloudStorage()
    {
        return false;
    }

    // Refactor: 该函数方法名和逻辑表达的意思不一致,当用户为该课程的讲师，则不记入加入课程数
    public function countUserLearnCourse($userId)
    {
        return $this->getMemberService()->countMembers(array('userId' => $userId, 'role' => 'student'));
    }

    // Refactor: 该函数方法名和逻辑表达的意思不一致,当用户为该课程的讲师，则不记入学习档案详细中
    public function findUserLearnCourses($userId, $start, $limit)
    {
        return $this->getMemberService()->searchMembers(array('userId' => $userId, 'role' => 'student'), array(),
            $start, $limit);
    }

    private function processFields($id, $fields, $courseSet)
    {
        if (isset($fields['originPrice'])) {
            list($fields['price'], $fields['coinPrice']) = $this->calculateCoursePrice($id, $fields['originPrice']);
        }

        if ($fields['isFree'] == 1) {
            $fields['price'] = 0;
        }

        if ($courseSet['type'] == 'normal' && $fields['tryLookable'] == 0) {
            $fields['tryLookLength'] = 0;
        }

        if (!empty($fields['buyExpiryTime'])) {
            if (is_numeric($fields['buyExpiryTime'])) {
                $fields['buyExpiryTime'] = date('Y-m-d', $fields['buyExpiryTime']);
            }

            $fields['buyExpiryTime'] = strtotime($fields['buyExpiryTime'].' 23:59:59');
        } else {
            $fields['buyExpiryTime'] = 0;
        }

        return $fields;
    }

    public function getTopCategorySummaryDatas($userId, $num = 5)
    {
        $courses = $this->findUserLearnCourses($userId, 0, PHP_INT_MAX);
        $courses = $this->getCourseSetService()->findCourseSetsByCourseIds(ArrayToolkit::column($courses, 'courseId'));

        $coursesGroupByCategory = ArrayToolkit::group($courses, 'categoryId');
        $coursesGroupByCategoryCount = array();
        foreach ($coursesGroupByCategory as $key => $courses) {
            $coursesGroupByCategoryCount[$key] = count($courses);
        }
        arsort($coursesGroupByCategoryCount);

        $count = 1;
        $topCategoryDatas = array();
        foreach ($coursesGroupByCategoryCount as $key => $value) {
            if ($count++ > $num) {
                break;
            }
            $topCategoryDatas[$key]['courseNum'] = $value;
            $categoryIds = ArrayToolkit::column($coursesGroupByCategory[$key], 'categoryId');
            $courseIds = ArrayToolkit::column($coursesGroupByCategory[$key], 'id');
            $conditions['courseIds'] = !empty($courseIds) ? $courseIds : array(-1);
            $conditions['userId'] = $userId;
            $coursesWatchTime = $this->getTaskResultService()->sumWatchTimeByCategoryIdAndUserId($categoryIds[0],
                $userId);
            $topCategoryDatas[$key]['watchTime'] = $coursesWatchTime;
            $coursesLearnTime = $this->getTaskResultService()->sumLearnTimeByCategoryIdAndUserId($categoryIds[0],
                $userId);
            $topCategoryDatas[$key]['learnTime'] = $coursesLearnTime;
        }

        return $topCategoryDatas;
    }

    public function canManageCourse($courseId, $courseSetId = 0)
    {
        $user = $this->getCurrentUser();
        if (!$user->isLogin()) {
            return false;
        }

        $course = $this->getCourseDao()->get($courseId);

        if (empty($course)) {
            return false;
        }

        $courseSet = $this->getCourseSetDao()->get($course['courseSetId']);

        if (!in_array($courseSet['orgId'], $user['accessibleOrgIds'])) {
            return false;
        }

        if ($courseSetId > 0 && $course['courseSetId'] !== $courseSetId) {
            return false;
        }

        if ($course['parentId'] > 0) {
            $classroom = $this->getClassroomService()->getClassroomByCourseId($courseId);
            if (!empty($classroom) && $classroom['headTeacherId'] == $user['id']) {
                //班主任有权管理班级下所有课程
                return true;
            }
        }

        if (!$this->hasCourseManagerRole($courseId)) {
            return false;
        }

        return true;
    }

    public function tryManageCourse($courseId, $courseSetId = 0)
    {
        if (!$this->canManageCourse($courseId, $courseSetId)) {
            throw $this->createAccessDeniedException('Unauthorized');
        }

        return $this->getCourseDao()->get($courseId);
    }

    protected function getTaskResultService()
    {
        return $this->createService('Task:TaskResultService');
    }
}
