<?php

namespace CorporateTrainingBundle\Biz\DataStatistics\Job;

use Codeages\Biz\Framework\Scheduler\AbstractJob;
use Monolog\Handler\StreamHandler;
use Monolog\Logger;
use Symfony\Component\Yaml\Yaml;
use Topxia\Service\Common\ServiceKernel;

class BaseJob extends AbstractJob
{
    protected $logger;

    public function execute()
    {
    }

    protected function getLogger()
    {
        if ($this->logger) {
            return $this->logger;
        }
        $biz = $this->getServiceKernel()->getBiz();
        $logger = new Logger('Job');
        $logger->pushHandler(new StreamHandler("{$biz['log_directory']}/data-statistics.log", Logger::DEBUG));
        $this->logger = $logger;

        return $logger;
    }

    public function getLastExecutedTime()
    {
        $filePath = $this->getCrontabConfigYml();
        $yaml = new Yaml();

        if (!file_exists($filePath)) {
            $content = $yaml->dump(array('crontab_last_executed_time' => 0));
            $fh = fopen($filePath, 'w');
            fwrite($fh, $content);
            fclose($fh);
        }

        $fileContent = file_get_contents($filePath);
        $config = $yaml->parse($fileContent);

        return $config['crontab_last_executed_time'];
    }

    public function setLastExecutedTime($lastExecutedTime)
    {
        $filePath = $this->getCrontabConfigYml();
        $yaml = new Yaml();
        $content = $yaml->dump(array('crontab_last_executed_time' => $lastExecutedTime));
        $fh = fopen($filePath, 'w');
        fwrite($fh, $content);
        fclose($fh);
    }

    private function getCrontabConfigYml()
    {
        return $this->getServiceKernel()->getParameter('kernel.root_dir').'/../app/data/data_statistics_crontab_config.yml';
    }

    /**
     * @return \Codeages\Biz\Framework\Scheduler\Service\Impl\SchedulerServiceImpl;
     */
    protected function getScheduleService()
    {
        return $this->createService('Scheduler:SchedulerService');
    }

    protected function createService($alias)
    {
        return $this->getServiceKernel()->getBiz()->service($alias);
    }

    protected function getServiceKernel()
    {
        return ServiceKernel::instance();
    }
}
