<?php

namespace CorporateTrainingBundle\Biz\DataStatistics\Job;

use AppBundle\Common\ArrayToolkit;
use Biz\Course\Service\CourseService;
use Biz\Course\Service\MemberService;
use Biz\User\Service\UserService;
use CorporateTrainingBundle\Biz\DataStatistics\Service\UserDailyLearnRecordService;
use CorporateTrainingBundle\Biz\DataStatistics\Service\UserLearnReportService;
use CorporateTrainingBundle\Biz\OfflineCourse\Service\CategoryService;
use CorporateTrainingBundle\Biz\PostCourse\Service\PostCourseService;
use CorporateTrainingBundle\Biz\Task\Service\TaskResultService;

class InitUserLearnReportJob extends BaseJob
{
    public function execute()
    {
        $this->getLogger()->addInfo('开始初始化用户学习数据报表........');
        $lastExecutedTime = strtotime(date('Y-m-d', strtotime('-1 day')));
        $conditions = array(
            'locked' => 0,
            'noType' => 'system',
        );

        $count = $this->getUserService()->countUsers($conditions);
        $users = $this->getUserService()->searchUsers(
            $conditions,
            array('createdTime' => 'DESC'),
            0,
            $count
        );

        foreach ($users as $user) {
            $joinCourseNum = $this->getCourseService()->countUserLearnCourse($user['id']);

            $totalLearnTime = $this->getTaskResultService()->sumLearnTimeByUserId($user['id']);

            $postCourseNum = $this->getPostCourseService()->countPostCourses(array('postId' => $user['postId']));

            $finishedPostCourseNum = $this->getPostCourseService()->countFinishedPostCoursesByPostIdAndUserId($user['postId'], $user['id']);

            $postLearnTime = $this->getTaskResultService()->sumLearnTimeByPostIdAndUserId($user['postId'], $user['id']);

            $joinOfflineCourseNum = $this->getMemberService()->countMembers(array('userId' => $user['id']));

            $passedOfflineCourseNum = $this->getMemberService()->countMembers(array('userId' => $user['id'], 'passedStatus' => 'passed'));

            $fields = array(
                'userId' => $user['id'],
                'postId' => $user['postId'],
                'joinCourseNum' => empty($joinCourseNum) ? 0 : $joinCourseNum,
                'totalLearnTime' => empty($totalLearnTime) ? 0 : $totalLearnTime,
                'postCourseNum' => empty($postCourseNum) ? 0 : $postCourseNum,
                'finishedPostCourseNum' => empty($finishedPostCourseNum) ? 0 : $finishedPostCourseNum,
                'postLearnTime' => empty($postLearnTime) ? 0 : $postLearnTime,
                'joinOfflineCourseNum' => empty($joinOfflineCourseNum) ? 0 : $joinOfflineCourseNum,
                'passedOfflineCourseNum' => empty($passedOfflineCourseNum) ? 0 : $passedOfflineCourseNum,
            );
            $this->initUserLearnReport($fields);
        }
        $this->initUserDailyRecord($users, $lastExecutedTime);
        $this->getLogger()->addInfo('初始化成功');
        $this->setLastExecutedTime($lastExecutedTime);
    }

    private function initUserLearnReport($fields)
    {
        try {
            $leanReport = $this->getUserLearnReportService()->getUserLearnReportByUserId($fields['userId']);
            if (empty($leanReport)) {
                $this->getUserLearnReportService()->createLearnReport($fields);
            }
        } catch (\Exception $e) {
            $this->getLogger()->addError('保存用户学习数据错误：'.$e->getMessage(), $fields);
        }
    }

    private function initUserDailyRecord($users, $date)
    {
        $categoryGroup = $this->getCategoryService()->getGroupByCode('course');
        $conditions['groupId'] = $categoryGroup['id'];
        $categories = $this->getCategoryService()->searchCategories($conditions, array(), 0, PHP_INT_MAX);
        $categoryIds = ArrayToolkit::column($categories, 'id');

        foreach ($users as $user) {
            $members = $this->getCourseMemberService()->findStudentMemberByUserId($user['id']);
            foreach ($members as $member) {
                $course = $this->getCourseService()->getCourse($member['courseId']);
                $taskResults = $this->getTaskResultService()->searchTaskResults(
                    array('userId' => $user['id'], 'courseId' => $course['id']),
                    array('createdTime' => 'DESC'),
                    0,
                    PHP_INT_MAX
                );

                $totalLearnTime = array_sum(ArrayToolkit::column($taskResults, 'time'));
                $record['userId'] = $user['id'];
                $record['courseId'] = $course['id'];
                $record['postId'] = $user['postId'];
                $categoryId = 0;
                if (in_array($course['categoryId'], $categoryIds)) {
                    $categoryId = $course['categoryId'];
                }
                $record['categoryId'] = $categoryId;
                $record['learnTime'] = $totalLearnTime;
                $record['finishedTaskNum'] = $member['learnedNum'];
                $record['courseStatus'] = ($member['learnedNum'] >= $course['publishedTaskNum']) ? 1 : 0;
                $record['date'] = $date;

                $this->getUserDailyRecordService()->createDailyRecord($record);
            }
        }
    }

    /**
     * @return CourseService
     */
    protected function getCourseService()
    {
        return $this->createService('Course:CourseService');
    }

    /**
     * @return MemberService
     */
    protected function getCourseMemberService()
    {
        return $this->createService('Course:MemberService');
    }

    /**
     * @return PostCourseService
     */
    protected function getPostCourseService()
    {
        return $this->createService('PostCourse:PostCourseService');
    }

    /**
     * @return UserDailyLearnRecordService
     */
    protected function getUserDailyRecordService()
    {
        return $this->createService('CorporateTrainingBundle:DataStatistics:UserDailyLearnRecordService');
    }

    /**
     * @return TaskResultService
     */
    protected function getTaskResultService()
    {
        return $this->createService('CorporateTrainingBundle:Task:TaskResultService');
    }

    /**
     * @return CategoryService
     */
    protected function getCategoryService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:CategoryService');
    }

    /**
     * @return UserService
     */
    protected function getUserService()
    {
        return $this->createService('User:UserService');
    }

    /**
     * @return MemberService
     */
    protected function getMemberService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:MemberService');
    }

    /**
     * @return UserLearnReportService
     */
    protected function getUserLearnReportService()
    {
        return $this->createService('CorporateTrainingBundle:DataStatistics:UserLearnReportService');
    }
}
