<?php

namespace CorporateTrainingBundle\Biz\DataStatistics\Job;

use CorporateTrainingBundle\Biz\DataStatistics\Service\UserDailyLearnRecordService;

class PreSyncUserLearnReportJob extends BaseJob
{
    protected $size = 100;

    public function execute()
    {
        $dates = $this->getNeededSyncDates();

        if (empty($dates)) {
            return;
        }

        $this->getLogger()->addInfo('准备同步'.date('Y-m-d', $dates['startDate']).'至'.date('Y-m-d', $dates['endDate']).'号，用户学习增量数据到总报表');
        $userIds = $this->getUserDailyLearnRecordService()->findLearnUserIdsByDate($dates['startDate'], $dates['endDate']);
        if (empty($userIds)) {
            $this->getLogger()->addInfo('没有用户学习增量数据');
            $this->setLastExecutedTime($dates['endDate']);
        }

        $chunkUserIdData = array_chunk($userIds, $this->size);
        $this->getLogger()->addInfo('开始拆分同步学习增量子任务，任务有：'.count($chunkUserIdData).'个');

        foreach ($chunkUserIdData as $userIdData) {
            $this->addSyncUserLearnReportJob(array('lastExecutedTime' => $dates['endDate'], 'currentUserIdData' => $userIdData));
            $this->getLogger()->addInfo('同步学习增量子任务添加完成');
        }

        $this->addSyncUserOfflineCoursesReportJob(array('dates' => $dates));
        $this->getLogger()->addInfo('同步学习活动报表任务添加完成');
    }

    private function addSyncUserLearnReportJob($params)
    {
        try {
            $this->getLogger()->addInfo('开始添加同步学习增量子任务，参数：', $params);
            $job = array(
                'name' => 'SyncUserLearnReportJob',
                'source' => 'TRAININGMAIN',
                'expression' => time() + 60 * 60,
                'class' => 'CorporateTrainingBundle\Biz\DataStatistics\Job\SyncUserLearnReportJob',
                'args' => $params,
                'misfire_policy' => 'executing',
            );
            $this->getScheduleService()->register($job);
        } catch (\Exception $e) {
            $this->getLogger()->addError('添加同步学习增量子任务异常：'.$e->getMessage());
        }
    }

    private function addSyncUserOfflineCoursesReportJob($params)
    {
        try {
            $this->getLogger()->addInfo('开始添加同步学习活动报表任务，参数：', $params);
            $job = array(
                'name' => 'SyncUserOfflineCoursesReportJob',
                'source' => 'TRAININGMAIN',
                'expression' => time() + 60 * 60,
                'class' => 'CorporateTrainingBundle\Biz\DataStatistics\Job\SyncUserOfflineCoursesReportJob',
                'args' => $params,
                'misfire_policy' => 'executing',
            );
            $this->getScheduleService()->register($job);
        } catch (\Exception $e) {
            $this->getLogger()->addError('添加同步学习活动报表任务异常：'.$e->getMessage());
        }
    }

    private function initUserLearnReportJob()
    {
        try {
            $this->getLogger()->addInfo('上次同步时间为空，添加初始化用户学习报表任务');
            $job = array(
                'name' => 'InitUserLearnReportJob',
                'source' => 'TRAININGMAIN',
                'expression' => time(),
                'class' => 'CorporateTrainingBundle\Biz\DataStatistics\Job\InitUserLearnReportJob',
                'args' => array(),
                'misfire_policy' => 'executing',
            );
            $this->getScheduleService()->register($job);
        } catch (\Exception $e) {
            $this->getLogger()->addInfo('添加初始化用户学习报表任务异常 ：'.$e->getMessage());
        }
    }

    protected function getNeededSyncDates()
    {
        $lastExecutedTime = $this->getLastExecutedTime();

        if (empty($lastExecutedTime)) {
            $this->initUserLearnReportJob();

            return array();
        }

        $date = strtotime(date('Y-m-d', strtotime('-1 day')));
        if ($lastExecutedTime == $date) {
            $this->getLogger()->addInfo('用户学习报表任务已经同步过，上一次同步日期：'.date('Y-m-d', $lastExecutedTime));

            return array();
        }

        if ($lastExecutedTime < $date) {
            return array(
                'startDate' => $lastExecutedTime,
                'endDate' => $date,
            );
        }

        return array();
    }

    /**
     * @return UserDailyLearnRecordService
     */
    protected function getUserDailyLearnRecordService()
    {
        return $this->getServiceKernel()->createService('CorporateTrainingBundle:DataStatistics:UserDailyLearnRecordService');
    }
}
