<?php

namespace CorporateTrainingBundle\Biz\DataStatistics\Job;

use AppBundle\Common\ArrayToolkit;
use Biz\Course\Service\CourseService;
use CorporateTrainingBundle\Biz\DataStatistics\Service\UserDailyLearnRecordService;
use CorporateTrainingBundle\Biz\DataStatistics\Service\UserLearnReportService;
use CorporateTrainingBundle\Biz\PostCourse\Service\PostCourseService;
use CorporateTrainingBundle\Biz\User\Service\UserService;

class SyncUserLearnReportJob extends BaseJob
{
    public function execute()
    {
        $params = $this->args;
        $this->getLogger()->addInfo('接收到同步学习增量任务，参数：'.json_encode($params));
        foreach ($params['currentUserIdData'] as $result) {
            $user = $this->getUserService()->getUser($result['userId']);
            if (empty($user)) {
                $this->getLogger()->addInfo("用户：{$result['userId']}不存在！");
                continue;
            }
            $joinCourseNum = $this->getCourseService()->countUserLearnCourse($user['id']);

            $totalLearnTime = $this->getUserDailyLearnRecordService()->sumLearnTimeByUserId($user['id']);

            $postCourseNum = $this->getPostCourseService()->countPostCourses(array('postId' => $user['postId']));

            $finishedPostCourseNum = $this->getFinishedPostCourseNum($user['postId'], $user['id']);

            $postLearnTime = $this->sumPostLearnTime($user['id'], $user['postId']);

            $reportParams = array(
                'userId' => $user['id'],
                'postId' => $user['postId'],
                'joinCourseNum' => $joinCourseNum,
                'totalLearnTime' => $totalLearnTime,
                'postCourseNum' => $postCourseNum,
                'finishedPostCourseNum' => $finishedPostCourseNum,
                'postLearnTime' => $postLearnTime,
            );

            $this->syncLearnReport($reportParams);
            $this->getLogger()->addInfo('同步学习增量成功。');
            $this->setLastExecutedTime($params['lastExecutedTime']);
        }
    }

    private function syncLearnReport($params)
    {
        try {
            $learnReport = $this->getUserLearnReportService()->getUserLearnReportByUserId($params['userId']);
            if (empty($learnReport)) {
                $this->getUserLearnReportService()->createLearnReport($params);
            } else {
                $this->getUserLearnReportService()->updateLearnReport($learnReport['id'], $params);
            }
        } catch (\Exception $e) {
            $this->getLogger()->addError('同步学习增量失败：'.$e->getMessage(), $params);
        }
    }

    private function getFinishedPostCourseNum($postId, $userId)
    {
        $postCourses = $this->getPostCourseService()->findPostCoursesByPostId($postId);
        if (empty($postCourses)) {
            return 0;
        }
        $courseIds = ArrayToolkit::column($postCourses, 'courseId');

        return $this->getPostCourseService()->countUserLearnedPostCourses($userId, $courseIds);
    }

    private function sumPostLearnTime($userId, $postId)
    {
        return $this->getUserDailyLearnRecordService()->sumPostLearnTimeByUserIdAndPostId($userId, $postId);
    }

    /**
     * @return CourseService
     */
    protected function getCourseService()
    {
        return $this->getServiceKernel()->createService('Course:CourseService');
    }

    /**
     * @return UserDailyLearnRecordService
     */
    protected function getUserDailyLearnRecordService()
    {
        return $this->getServiceKernel()->createService('CorporateTrainingBundle:DataStatistics:UserDailyLearnRecordService');
    }

    /**
     * @return UserService
     */
    protected function getUserService()
    {
        return $this->getServiceKernel()->createService('User:UserService');
    }

    /**
     * @return PostCourseService
     */
    protected function getPostCourseService()
    {
        return $this->getServiceKernel()->createService('CorporateTrainingBundle:PostCourse:PostCourseService');
    }

    /**
     * @return UserLearnReportService
     */
    protected function getUserLearnReportService()
    {
        return $this->getServiceKernel()->createService('CorporateTrainingBundle:DataStatistics:UserLearnReportService');
    }
}
