<?php

namespace CorporateTrainingBundle\Biz\DataStatistics\Job;

class SyncUserOfflineCoursesReportJob extends BaseJob
{
    public function execute()
    {
        $params = $this->args;
        $this->getLogger()->addInfo('接收到同步学习活动报表任务，参数：'.json_encode($params));

        $results = $this->getMemberService()->findDistinctUserIdsByDate($params['dates']['startDate'], $params['dates']['endDate']);
        if (empty($results)) {
            $this->getLogger()->addInfo('暂无数据同步.....');

            return;
        }
        foreach ($results as $result) {
            try {
                $learnReport = $this->getUserLearnReportService()->getUserLearnReportByUserId($result['userId']);
                if (empty($learnReport)) {
                    $this->getLogger()->addInfo("用户:{$result['userId']}学习报表不存在！");
                    continue;
                }
                $joinOfflineCourseNum = $this->getMemberService()->countMembers(array('userId' => $result['userId']));
                $passedOfflineCourseNum = $this->getMemberService()->countMembers(array('userId' => $result['userId'], 'passedStatus' => 'passed'));

                $learnReport['joinOfflineCourseNum'] = $joinOfflineCourseNum;
                $learnReport['passedOfflineCourseNum'] = $passedOfflineCourseNum;
                $this->getUserLearnReportService()->updateLearnReport($learnReport['id'], $learnReport);
            } catch (\Exception $e) {
                $this->getLogger()->addError('同步学习活动报表失败:'.$e->getMessage());
            }
        }
        $this->getLogger()->addInfo('同步学习活动报表成功');
    }

    /**
     * @return \CorporateTrainingBundle\Biz\TrainingAssignment\Service\Impl\TrainingAssignmentMemberServiceImpl
     */
    protected function getMemberService()
    {
        return $this->getServiceKernel()->createService('CorporateTrainingBundle:OfflineCourse:MemberService');
    }

    /**
     * @return \CorporateTrainingBundle\Biz\DataStatistics\Service\Impl\UserLearnReportServiceImpl
     */
    protected function getUserLearnReportService()
    {
        return $this->getServiceKernel()->createService('CorporateTrainingBundle:DataStatistics:UserLearnReportService');
    }
}
