<?php

namespace CorporateTrainingBundle\Biz\DataStatistics\Service\Impl;

use AppBundle\Common\ArrayToolkit;
use Biz\BaseService;
use CorporateTrainingBundle\Biz\DataStatistics\Dao\UserDailyLearnRecordDao;
use CorporateTrainingBundle\Biz\DataStatistics\Service\UserDailyLearnRecordService;

class UserDailyLearnRecordServiceImpl extends BaseService implements UserDailyLearnRecordService
{
    public function getDailyRecord($id)
    {
        return $this->getUserDailyLearnRecordDao()->get($id);
    }

    public function createDailyRecord(array $dailyRecord)
    {
        if (!ArrayToolkit::requireds($dailyRecord, array('userId', 'courseId', 'date'))) {
            throw $this->createInvalidArgumentException('parameters is invalid');
        }

        $fields = $this->filterDailyLearnReportFields($dailyRecord);

        return $this->getUserDailyLearnRecordDao()->create($fields);
    }

    public function createCurrentDateDailyRecord(array $dailyRecord)
    {
        $dailyRecord['date'] = $this->getCurrentDate();

        return $this->createDailyRecord($dailyRecord);
    }

    public function updateDailyRecord($id, array $fields)
    {
        $fields = $this->filterDailyLearnReportFields($fields);

        return $this->getUserDailyLearnRecordDao()->update($id, $fields);
    }

    public function countRecords(array $conditions)
    {
        return $this->getUserDailyLearnRecordDao()->count($conditions);
    }

    public function searchRecords(array $conditions, array $orderBy, $start, $limit)
    {
        return $this->getUserDailyLearnRecordDao()->search($conditions, $orderBy, $start, $limit);
    }

    public function getRecordByUserIdAndCourseId($userId, $courseId)
    {
        return $this->getUserDailyLearnRecordDao()->getByUserIdAndCourseId($userId, $courseId);
    }

    public function getTodayRecordByUserIdAndCourseId($userId, $courseId)
    {
        $currentDate = $this->getCurrentDate();

        return $this->getUserDailyLearnRecordDao()->getByUserIdAndCourseIdAndDate($userId, $courseId, $currentDate);
    }

    public function getRecordByUserIdAndCourseIdAndCourseStatus($userId, $courseId, $status)
    {
        return $this->getUserDailyLearnRecordDao()->getByUserIdAndCourseIdAndCourseStatus($userId, $courseId, $status);
    }

    public function getRecordByUserIdAndCourseIdAndDate($userId, $courseId, $date)
    {
        return $this->getUserDailyLearnRecordDao()->getByUserIdAndCourseIdAndDate($userId, $courseId, $date);
    }

    public function waveLearnTimeById($id, $learnTime)
    {
        return $this->getUserDailyLearnRecordDao()->waveLearnTimeById($id, $learnTime);
    }

    public function waveFinishedTaskNumById($id)
    {
        return $this->getUserDailyLearnRecordDao()->waveFinishedTaskNum($id, 1);
    }

    public function sumPostLearnTimeByUserIdAndPostId($userId, $postId)
    {
        return $this->getUserDailyLearnRecordDao()->sumPostLearnTimeByUserIdAndPostId($userId, $postId);
    }

    public function statisticsUsersLearnTimeByDateGroupByUserId($date)
    {
        return $this->getUserDailyLearnRecordDao()->statisticsUsersLearnTimeByDateGroupByUserId($date);
    }

    public function statisticsLearnRecordGroupByUserId(array $conditions)
    {
        return $this->getUserDailyLearnRecordDao()->statisticsLearnRecordGroupByUserId($conditions);
    }

    public function statisticsLearnRecordGroupByOrgId(array $conditions)
    {
        return $this->getUserDailyLearnRecordDao()->statisticsLearnRecordGroupByOrgId($conditions);
    }

    public function statisticsLearnRecordGroupByPostId(array $conditions)
    {
        return $this->getUserDailyLearnRecordDao()->statisticsLearnRecordGroupByPostId($conditions);
    }

    public function statisticsLearnRecordGroupByCategoryId(array $conditions)
    {
        return $this->getUserDailyLearnRecordDao()->statisticsLearnRecordGroupByCategoryId($conditions);
    }

    public function statisticsOrgLearnTimeRankingList(array $conditions, $start = 0, $limit = 5)
    {
        return $this->getUserDailyLearnRecordDao()->statisticsOrgLearnTimeRankingList($conditions, $start, $limit);
    }

    public function statisticsPersonLearnTimeRankingList(array $conditions, $start = 0, $limit = 5)
    {
        return $this->getUserDailyLearnRecordDao()->statisticsPersonLearnTimeRankingList($conditions, $start, $limit);
    }

    public function statisticsLearnCourseRankingList(array $conditions, $start = 0, $limit = 5)
    {
        return $this->getUserDailyLearnRecordDao()->statisticsLearnCourseRankingList($conditions, $start, $limit);
    }

    public function statisticsTotalLearnTimeGroupByDate(array $conditions)
    {
        return $this->getUserDailyLearnRecordDao()->statisticsTotalLearnTimeGroupByDate($conditions);
    }

    public function statisticsLearnUsersNumGroupByDate(array $conditions)
    {
        return $this->getUserDailyLearnRecordDao()->statisticsLearnUsersNumGroupByDate($conditions);
    }

    public function statisticsFinishedCourseNumAndLearnTime(array $conditions)
    {
        return $this->getUserDailyLearnRecordDao()->statisticsFinishedCourseNumAndLearnTime($conditions);
    }

    public function sumLearnTimeByUserId($userId)
    {
        return $this->getUserDailyLearnRecordDao()->sumLearnTimeByUserId($userId);
    }

    public function findLearnUserIdsByDate($startDate, $endDate)
    {
        $conditions = array(
            'startDateTime' => $startDate,
            'endDateTime' => $endDate,
        );

        return $this->getUserDailyLearnRecordDao()->findLearnUserIds($conditions);
    }

    protected function getCurrentDate()
    {
        return  strtotime(date('Y-m-d', time()));
    }

    protected function filterDailyLearnReportFields($fields)
    {
        return ArrayToolkit::parts($fields, array(
            'userId',
            'postId',
            'courseId',
            'categoryId',
            'learnTime',
            'finishedTaskNum',
            'courseStatus',
            'classroomId',
            'date',
        ));
    }

    /**
     * @return UserDailyLearnRecordDao
     */
    protected function getUserDailyLearnRecordDao()
    {
        return $this->createDao('CorporateTrainingBundle:DataStatistics:UserDailyLearnRecordDao');
    }
}
