<?php

namespace CorporateTrainingBundle\Biz\DataStatistics\Service\Impl;

use AppBundle\Common\ArrayToolkit;
use Biz\BaseService;
use CorporateTrainingBundle\Biz\DataStatistics\Dao\UserLearnReportDao;
use CorporateTrainingBundle\Biz\DataStatistics\Service\UserLearnReportService;

class UserLearnReportServiceImpl extends BaseService implements UserLearnReportService
{
    public function getUserLearnReport($id)
    {
        return $this->getUserLearnRecordDao()->get($id);
    }

    public function createLearnReport(array $learnReport)
    {
        if (!ArrayToolkit::requireds($learnReport, array('userId'))) {
            throw $this->createInvalidArgumentException('parameters is invalid');
        }

        $fields = $this->filterLearnReportFields($learnReport);

        return $this->getUserLearnRecordDao()->create($fields);
    }

    public function updateLearnReport($id, array $fields)
    {
        $fields = $this->filterLearnReportFields($fields);

        return $this->getUserLearnRecordDao()->update($id, $fields);
    }

    public function countUserLearnReports(array $conditions)
    {
        return $this->getUserLearnRecordDao()->count($conditions);
    }

    public function searchUserLearnReports(array $conditions, array $orderBys, $start, $limit)
    {
        return $this->getUserLearnRecordDao()->search($conditions, $orderBys, $start, $limit);
    }

    public function getUserLearnReportByUserId($userId)
    {
        return $this->getUserLearnRecordDao()->getByUserId($userId);
    }

    public function hasManageUserLearnReportRole()
    {
        $currentUser = $this->getCurrentUser();
        if (!$currentUser->isLogin()) {
            return false;
        }

        return in_array('ROLE_DEPARTMENT_ADMIN', $currentUser['roles']) || in_array('ROLE_SUPER_ADMIN', $currentUser['roles']);
    }

    protected function filterLearnReportFields($fields)
    {
        return ArrayToolkit::parts($fields, array(
            'userId',
            'postId',
            'joinCourseNum',
            'totalLearnTime',
            'postCourseNum',
            'finishedPostCourseNum',
            'postLearnTime',
            'joinOfflineCourseNum',
            'passedOfflineCourseNum',
        ));
    }

    /**
     * @return UserLearnReportDao
     */
    protected function getUserLearnRecordDao()
    {
        return $this->createDao('CorporateTrainingBundle:DataStatistics:UserLearnReportDao');
    }
}
