<?php

namespace CorporateTrainingBundle\Biz\DiscoveryColumn\Service\Impl;

use Biz\DiscoveryColumn\Service\Impl\DiscoveryColumnServiceImpl as BaseService;

class DiscoveryColumnServiceImpl extends BaseService
{
    public function getDisplayData()
    {
        $columns = $this->getDiscoveryColumnDao()->findAllOrderBySeq();

        foreach ($columns as &$column) {
            if ($column['type'] == 'course' || $column['type'] == 'live') {
                $courseSets = $this->getCourseSetService()->searchCourseSets(
                    $this->determineConditions($column),
                    $this->determineSort($column),
                    0,
                    $column['showCount']
                );

                $column['data'] = $courseSets;
                $column['actualCount'] = count($courseSets);
            }

            if ($column['type'] == 'classroom') {
                $classrooms = $this->getClassroomService()->searchClassrooms(
                    $this->determineConditions($column),
                    $this->determineSort($column),
                    0,
                    $column['showCount']
                );

                $column['data'] = $classrooms;
                $column['actualCount'] = count($classrooms);
            }
        }

        return $columns;
    }

    /**
     * @todo orgCode
     */
    private function determineConditions($column)
    {
        $user = $this->getUserService()->getUser($this->getCurrentUser()->getId());
        $rootOrg = $this->getOrgService()->getOrgByOrgCode('1.');
        $orgIds = array_merge($user['orgIds'], array($rootOrg['id']));

        $conditions = array(
            'status' => 'published',
            'parentId' => 0,
            'showable' => 1,
            'orgIds' => $orgIds,
        );

        if (!empty($column['categoryId'])) {
            $childrenIds = $this->getCategoryService()->findCategoryChildrenIds($column['categoryId']);
            $conditions['categoryIds'] = array_merge(array($column['categoryId']), $childrenIds);
        }

        if ($column['type'] == 'live') {
            $conditions['type'] = 'live';
        }

        if ($column['type'] == 'course') {
            $conditions['type'] = 'normal';
        }

        if ($column['orderType'] == 'recommend') {
            $conditions['recommended'] = 1;
        }

        return $conditions;
    }

    private function determineSort($column)
    {
        $sortMap = array(
            'hot' => array('studentNum' => 'DESC'),
            'recommend' => array(
                'recommendedSeq' => 'ASC',
                'recommendedTime' => 'DESC',
            ),
        );

        if ($column['orderType'] && !empty($sortMap[$column['orderType']])) {
            return $sortMap[$column['orderType']];
        } else {
            return array(
                'createdTime' => 'DESC',
            );
        }
    }

    private function getCourseSetService()
    {
        return $this->createService('Course:CourseSetService');
    }

    private function getClassroomService()
    {
        return $this->createService('Classroom:ClassroomService');
    }

    private function getUserService()
    {
        return $this->createService('User:UserService');
    }

    private function getOrgService()
    {
        return $this->createService('Org:OrgService');
    }
}
