<?php

namespace CorporateTrainingBundle\Biz\Exporter;

use AppBundle\Common\ArrayToolkit;
use Codeages\Biz\Framework\Context\Biz;

abstract class AbstractExporter
{
    protected $biz;

    protected $startRow = 1;

    protected $objExcel;

    protected $activeSheet;

    protected $titleColNum;

    abstract public function canExport($parameters);

    abstract public function getExportFileName();

    abstract protected function getSortedHeadingRow();

    abstract protected function buildExportData($parameters);

    public function __construct(Biz $biz)
    {
        $this->biz = $biz;
    }

    public function writeToExcel($parameters)
    {
        $this->setTitleColNumber();
        $exportData = $this->buildExportData($parameters);

        $excel = new Excel();
        $this->objExcel = $excel->createPHPExcelObject();

        foreach ($exportData as $key => $sheetExportData) {
            $this->objExcel->createSheet();
            $this->activeSheet = $this->objExcel->setActiveSheetIndex($key);
            $this->writeExcelHeadingRow();
            $this->writeExportDataToSheet($sheetExportData['data']);
            if (!empty($sheetExportData['sheetName'])) {
                $this->activeSheet->setTitle($sheetExportData['sheetName']);
            }
        }
        $this->objExcel->setActiveSheetIndex(0);

        return $excel->createWriter($this->objExcel);
    }

    protected function writeExportDataToSheet($sheetExportData)
    {
        foreach ($sheetExportData as $key => $exportData) {
            foreach ($exportData as $k => $v) {
                if (array_key_exists($k, $this->titleColNum)) {
                    $colNum = $this->titleColNum[$k]['colNumber'];
                    $rowNum = $this->startRow + $key + 1;
                    $this->activeSheet->setCellValue($colNum.$rowNum, $v);
                }
            }
        }
    }

    protected function writeExcelHeadingRow()
    {
        $sortedHeadingRows = $this->getSortedHeadingRow();
        foreach ($sortedHeadingRows as $row) {
            $colNum = $this->titleColNum[$row['code']]['colNumber'];
            $this->activeSheet->setCellValue($colNum.$this->startRow, $row['title']);
        }
    }

    protected function setTitleColNumber()
    {
        $sortedHeadingRows = $this->getSortedHeadingRow();

        $titleColNum = array();

        foreach ($sortedHeadingRows as $key => $headingRow) {
            $titleColNum[] = array(
              'code' => $headingRow['code'],
              'colNumber' => \PHPExcel_Cell::stringFromColumnIndex($key),
            );
        }

        $this->titleColNum = ArrayToolkit::index($titleColNum, 'code');
    }

    protected function createService($alias)
    {
        return $this->biz->service($alias);
    }
}
