<?php

namespace CorporateTrainingBundle\Biz\Exporter;

use AppBundle\Common\ArrayToolkit;
use Biz\Org\Service\OrgService;
use Biz\User\Service\UserService;
use CorporateTrainingBundle\Biz\DataStatistics\Service\UserDailyLearnRecordService;
use CorporateTrainingBundle\Biz\DataStatistics\Service\UserLearnReportService;
use CorporateTrainingBundle\Biz\Post\Service\PostService;

abstract class BaseDataStatisticsExporter extends AbstractExporter
{
    public function canExport($parameters)
    {
        return $this->getUserLearnReportService()->hasManageUserLearnReportRole();
    }

    protected function getAvgLearnTime($userCount, $totalLearnTime)
    {
        if (empty($totalLearnTime) || empty($userCount)) {
            return 0;
        }

        return floor($totalLearnTime / $userCount);
    }

    protected function prepareSearchConditions($conditions)
    {
        if (!empty($conditions['startDateTime'])) {
            $conditions['startDateTime'] = strtotime($conditions['startDateTime']);
        }

        if (!empty($conditions['endDateTime'])) {
            $conditions['endDateTime'] = strtotime($conditions['endDateTime']);
        }

        if (empty($conditions['orgIds'])) {
            $userIds = -1;
        } else {
            $conditions['orgIds'] = explode(',', $conditions['orgIds']);
            $userIds = $this->findUserIdsByOrgIds($conditions['orgIds']);
            $userIds = empty($userIds) ? -1 : $userIds;
        }

        $keywordUserIds = array();
        if (isset($conditions['keywordType']) && !empty($conditions['keyword'])) {
            $users = $this->getUserService()->searchUsers(
                array(
                    $conditions['keywordType'] => $conditions['keyword'],
                ),
                array('id' => 'DESC'),
                0,
                PHP_INT_MAX
            );

            $keywordUserIds = empty($users) ? array(-1) : ArrayToolkit::column($users, 'id');
            unset($conditions['keywordType']);
            unset($conditions['keyword']);
        }

        if ($keywordUserIds == -1 || $userIds == -1) {
            $conditions['userIds'] = array(-1);
        } elseif (empty($keywordUserIds)) {
            $conditions['userIds'] = $userIds;
        } else {
            $conditions['userIds'] = array_intersect($userIds, $keywordUserIds);
        }

        unset($conditions['orgIds']);

        return $conditions;
    }

    protected function findUserIdsByOrgIds($orgIds)
    {
        $usersOrg = $this->getUserOrgService()->findUserOrgsByOrgIds($orgIds);

        return ArrayToolkit::column($usersOrg, 'userId');
    }

    protected function getUserOrgService()
    {
        return $this->createService('CorporateTrainingBundle:User:UserOrgService');
    }

    /**
     * @return PostService
     */
    protected function getPostService()
    {
        return $this->createService('Post:PostService');
    }

    /**
     * @return OrgService
     */
    protected function getOrgService()
    {
        return $this->createService('Org:OrgService');
    }

    /**
     * @return UserService
     */
    protected function getUserService()
    {
        return $this->createService('User:UserService');
    }

    /**
     * @return UserLearnReportService
     */
    protected function getUserLearnReportService()
    {
        return $this->createService('CorporateTrainingBundle:DataStatistics:UserLearnReportService');
    }

    /**
     * @return UserDailyLearnRecordService
     */
    protected function getUserDailyLearnRecordService()
    {
        return $this->createService('CorporateTrainingBundle:DataStatistics:UserDailyLearnRecordService');
    }
}
