<?php

namespace CorporateTrainingBundle\Biz\Exporter;

use AppBundle\Common\ArrayToolkit;
use Biz\User\Service\UserService;
use CorporateTrainingBundle\Biz\OfflineCourse\Service\MemberService;
use CorporateTrainingBundle\Biz\OfflineCourse\Service\OfflineCourseService;
use CorporateTrainingBundle\Biz\Org\Service\OrgService;
use CorporateTrainingBundle\Biz\Post\Service\PostService;

abstract class BaseOfflineCourseExporter extends AbstractExporter
{
    public function canExport($parameters)
    {
        return $this->getCourseService()->hasCourseManageRole();
    }

    protected function prepareSearchConditions($parameters)
    {
        if (isset($parameters['keywordType']) && !empty($parameters['keyword'])) {
            $users = $this->getUserService()->searchUsers(
                array($parameters['keywordType'] => $parameters['keyword']),
                array('id' => 'ASC'),
                0, PHP_INT_MAX
            );

            $parameters = array(
                'userIds' => ArrayToolkit::column($users, 'id'),
            );

            unset($parameters['keywordType']);
            unset($parameters['keyword']);
        }

        return $parameters;
    }

    /**
     * @return OfflineCourseService
     */
    protected function getCourseService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:OfflineCourseService');
    }

    /**
     * @return MemberService
     */
    protected function getMemberService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:MemberService');
    }

    /**
     * @return UserService
     */
    protected function getUserService()
    {
        return $this->createService('User:UserService');
    }

    /**
     * @return PostService
     */
    protected function getPostService()
    {
        return $this->createService('CorporateTrainingBundle:Post:PostService');
    }

    /**
     * @return OrgService
     */
    protected function getOrgService()
    {
        return $this->createService('Org:OrgService');
    }
}
