<?php

namespace CorporateTrainingBundle\Biz\Exporter;

use AppBundle\Common\ArrayToolkit;
use CorporateTrainingBundle\Biz\OfflineCourse\Service\CategoryService;
use CorporateTrainingBundle\Common\DateToolkit;

class DataStatisticCategoryExporter extends BaseDataStatisticsExporter
{
    public function getExportFileName()
    {
        return 'data_statistic_category.xls';
    }

    public function getSortedHeadingRow()
    {
        return array(
            array('code' => 'category', 'title' => '课程分类'),
            array('code' => 'learnUsers', 'title' => '学习人数'),
            array('code' => 'finishedTakNum', 'title' => '完成课程任务数'),
            array('code' => 'finishedCourseNum', 'title' => '完成课程数'),
            array('code' => 'learnTime', 'title' => '学习时长'),
        );
    }

    protected function buildExportData($parameters)
    {
        $conditions = $this->prepareSearchConditions($parameters);
        $categoryGroup = $this->getCategoryService()->getGroupByCode('course');
        $conditions['groupId'] = $categoryGroup['id'];

        $categories = $this->getCategoryService()->searchCategories(
            $conditions,
            array(),
            0,
            $this->getCategoryService()->countCategories($conditions)
        );

        $noneCategory = array(
            'id' => 0,
            'code' => 'none',
            'name' => '无分类',
        );
        array_unshift($categories, $noneCategory);

        $conditions['categoryIds'] = ArrayToolkit::column($categories, 'id');
        $learnRecords = $this->getUserDailyLearnRecordService()->statisticsLearnRecordGroupByCategoryId(
            $conditions
        );
        $learnRecords = ArrayToolkit::index($learnRecords, 'categoryId');

        $categoryData = array();
        foreach ($categories as $category) {
            $totalLearnTime = empty($learnRecords[$category['id']]) ? 0 : $learnRecords[$category['id']]['totalLearnTime'];
            $learnUsers = empty($learnRecords[$category['id']]) ? 0 : $learnRecords[$category['id']]['learnUserNum'];
            $totalFinishedTaskNum = empty($learnRecords[$category['id']]) ? 0 : $learnRecords[$category['id']]['totalFinishedTaskNum'];
            $finishedCourseNum = empty($learnRecords[$category['id']]) ? 0 : $learnRecords[$category['id']]['finishedCourseNum'];
            $categoryData[] = array(
                'category' => empty($category['name']) ? '-' : $category['name'],
                'learnUsers' => $learnUsers,
                'finishedTakNum' => $totalFinishedTaskNum,
                'finishedCourseNum' => $finishedCourseNum,
                'learnTime' => DateToolkit::timeToHour($totalLearnTime),
            );
        }

        $exportData[] = array(
            'sheetName' => '课程分类报表汇总',
            'data' => $categoryData,
        );

        return $exportData;
    }

    /**
     /**
     * @return CategoryService
     */
    public function getCategoryService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:CategoryService');
    }
}
