<?php

namespace CorporateTrainingBundle\Biz\Exporter;

use AppBundle\Common\ArrayToolkit;
use CorporateTrainingBundle\Common\DateToolkit;

class DataStatisticDepartmentExporter extends BaseDataStatisticsExporter
{
    public function getExportFileName()
    {
        return 'data_statistic_department.xls';
    }

    public function getSortedHeadingRow()
    {
        return array(
            array('code' => 'orgName', 'title' => '部门'),
            array('code' => 'totalUsers', 'title' => '总人数'),
            array('code' => 'learnUsers', 'title' => '学习人数'),
            array('code' => 'finishedTaskNum', 'title' => '完成课程任务数'),
            array('code' => 'finishedNum', 'title' => '完成课程数'),
            array('code' => 'totalLearnTime', 'title' => '学习总时长'),
            array('code' => 'avgLearnTime', 'title' => '人均时长'),
        );
    }

    protected function buildExportData($parameters)
    {
        $orgIds = empty($parameters['orgIds']) ? array(-1) : explode(',', $parameters['orgIds']);
        $orgs = $this->getOrgService()->searchOrgs(
            array('orgIds' => $orgIds),
            array(),
            0,
            PHP_INT_MAX
        );

        $records = $this->getUserDailyLearnRecordService()->searchRecords(
            $this->prepareDepartmentLearnConditions($parameters),
            array('id' => 'DESC'),
            0,
            PHP_INT_MAX
        );

        $recordIds = ArrayToolkit::column($records, 'id');

        $learnRecords = $this->getUserDailyLearnRecordService()->statisticsLearnRecordGroupByOrgId(array(
            'orgIds' => empty($orgIds) ? array(-1) : $orgIds,
            'recordIds' => empty($recordIds) ? array(-1) : $recordIds,
        ));
        $learnRecords = ArrayToolkit::index($learnRecords, 'orgId');

        $orgUserNums = $this->getUserService()->statisticsOrgUserNumGroupByOrgId();
        $orgUserNums = ArrayToolkit::index($orgUserNums, 'orgId');

        $departmentData = array();

        foreach ($orgs as $org) {
            $totalUsers = empty($orgUserNums[$org['id']]['count']) ? 0 : $orgUserNums[$org['id']]['count'];
            $totalLearnTime = empty($learnRecords[$org['id']]) ? 0 : $learnRecords[$org['id']]['totalLearnTime'];
            $departmentData[] = array(
                'orgName' => empty($org['name']) ? '-' : $org['name'],
                'totalUsers' => $totalUsers,
                'finishedTaskNum' => empty($learnRecords[$org['id']]['totalFinishedTaskNum']) ? '-' : $learnRecords[$org['id']]['totalFinishedTaskNum'],
                'finishedNum' => empty($learnRecords[$org['id']]['finishedCourseNum']) ? '-' : $learnRecords[$org['id']]['finishedCourseNum'],
                'learnUsers' => empty($learnRecords[$org['id']]['learnUserNum']) ? '-' : $learnRecords[$org['id']]['learnUserNum'],
                'totalLearnTime' => DateToolkit::timeToHour($totalLearnTime),
                'avgLearnTime' => DateToolkit::timeToHour($this->getAvgLearnTime($totalUsers, $totalLearnTime)),
            );
        }

        $exportData[] = array(
            'sheetName' => '部门报表汇总',
            'data' => $departmentData,
        );

        return $exportData;
    }

    protected function prepareDepartmentLearnConditions($fields)
    {
        $conditions = array();

        if (!empty($fields['startDateTime'])) {
            $conditions['startDateTime'] = strtotime($fields['startDateTime']);
        }

        if (!empty($fields['endDateTime'])) {
            $conditions['endDateTime'] = strtotime($fields['endDateTime']);
        }

        if (!empty($fields['postId'])) {
            $conditions['postId'] = $fields['postId'];
        }

        if (!empty($fields['categoryId'])) {
            $conditions['categoryId'] = $fields['categoryId'];
        }

        return $conditions;
    }
}
