<?php

namespace CorporateTrainingBundle\Biz\Exporter;

use AppBundle\Common\ArrayToolkit;
use CorporateTrainingBundle\Common\DateToolkit;

class DataStatisticPersonExporter extends BaseDataStatisticsExporter
{
    public function getExportFileName()
    {
        return 'data_statistic_person.xls';
    }

    protected function getSortedHeadingRow()
    {
        return array(
            array('code' => 'truename', 'title' => '姓名'),
            array('code' => 'username', 'title' => '用户名'),
            array('code' => 'post', 'title' => '岗位'),
            array('code' => 'finishedTaskNum', 'title' => '完成课程任务数'),
            array('code' => 'finishedCourseNum', 'title' => '完成课程数'),
            array('code' => 'totalLearnTime', 'title' => '学习时长'),
        );
    }

    protected function buildExportData($parameters)
    {
        $conditions = $this->prepareSearchConditions($parameters);

        $userSearchConditions = $conditions;
        $userSearchConditions['locked'] = 0;
        $userSearchConditions['noType'] = 'system';

        $users = $this->getUserService()->searchUsers(
            $userSearchConditions,
            array(),
            0,
            PHP_INT_MAX
        );

        $userIds = ArrayToolkit::column($users, 'id');
        $conditions['userIds'] = $userIds;
        $records = $this->getUserDailyLearnRecordService()->statisticsLearnRecordGroupByUserId($conditions);
        $records = ArrayToolkit::index($records, 'userId');

        $profiles = $this->getUserService()->findUserProfilesByIds($userIds);
        $profiles = ArrayToolkit::index($profiles, 'id');
        $posts = $this->getPostService()->findPostsByIds(ArrayToolkit::column($users, 'postId'));
        $posts = ArrayToolkit::index($posts, 'id');

        $recordData = array();

        foreach ($users as $user) {
            $recordData[] = array(
                'truename' => empty($profiles[$user['id']]['truename']) ? '-' : $profiles[$user['id']]['truename'],
                'username' => empty($user['nickname']) ? '-' : $user['nickname'],
                'post' => empty($posts[$user['postId']]['name']) ? '-' : $posts[$user['postId']]['name'],
                'finishedTaskNum' => empty($records[$user['id']]['totalFinishedTaskNum']) ? '-' : $records[$user['id']]['totalFinishedTaskNum'],
                'finishedCourseNum' => empty($records[$user['id']]['finishedCourseNum']) ? '-' : $records[$user['id']]['finishedCourseNum'],
                'totalLearnTime' => empty($records[$user['id']]['totalLearnTime']) ? 0 : DateToolkit::timeToHour($records[$user['id']]['totalLearnTime']),
            );
        }

        $exportData[] = array(
            'data' => $recordData,
        );

        return $exportData;
    }
}
