<?php

namespace CorporateTrainingBundle\Biz\Exporter;

use AppBundle\Common\ArrayToolkit;
use CorporateTrainingBundle\Common\DateToolkit;

class DataStatisticPostExporter extends BaseDataStatisticsExporter
{
    public function getExportFileName()
    {
        return 'data_statistic_post.xls';
    }

    public function getSortedHeadingRow()
    {
        return array(
            array('code' => 'postName', 'title' => '岗位'),
            array('code' => 'totalUsers', 'title' => '总人数'),
            array('code' => 'learnUsers', 'title' => '学习人数'),
            array('code' => 'finishedTaskNum', 'title' => '完成课程任务数'),
            array('code' => 'finishedNum', 'title' => '完成课程数'),
            array('code' => 'totalLearnTime', 'title' => '总学习时长'),
            array('code' => 'avgLearnTime', 'title' => '人均时长'),
        );
    }

    protected function buildExportData($parameters)
    {
        $conditions = $this->prepareSearchConditions($parameters);
        $posts = $this->getPostService()->searchPosts(
            $conditions,
            array(),
            0,
            $this->getPostService()->countPosts($conditions)
        );

        $nonePost = array(
            'id' => 0,
            'code' => 'none',
            'name' => '无岗位',
        );
        array_unshift($posts, $nonePost);

        $conditions['postIds'] = ArrayToolkit::column($posts, 'id');
        $learnRecords = $this->getUserDailyLearnRecordService()->statisticsLearnRecordGroupByPostId(
            $conditions
        );
        $learnRecords = ArrayToolkit::index($learnRecords, 'postId');

        $postUserNums = $this->getUserService()->statisticsPostUserNumGroupByPostId();
        $postUserNums = ArrayToolkit::index($postUserNums, 'postId');

        $postData = array();

        foreach ($posts as $post) {
            $totalUsers = empty($postUserNums[$post['id']]['count']) ? 0 : $postUserNums[$post['id']]['count'];
            $totalLearnTime = empty($learnRecords[$post['id']]) ? 0 : $learnRecords[$post['id']]['totalLearnTime'];
            $postData[] = array(
                'postName' => empty($post['name']) ? '-' : $post['name'],
                'totalUsers' => $totalUsers,
                'learnUsers' => empty($learnRecords[$post['id']]['learnUserNum']) ? 0 : $learnRecords[$post['id']]['learnUserNum'],
                'finishedTaskNum' => empty($learnRecords[$post['id']]['totalFinishedTaskNum']) ? 0 : $learnRecords[$post['id']]['totalFinishedTaskNum'],
                'finishedNum' => empty($learnRecords[$post['id']]['finishedCourseNum']) ? 0 : $learnRecords[$post['id']]['finishedCourseNum'],
                'totalLearnTime' => DateToolkit::timeToHour($totalLearnTime),
                'avgLearnTime' => DateToolkit::timeToHour($this->getAvgLearnTime($totalUsers, $totalLearnTime)),
            );
        }

        $exportData[] = array(
            'sheetName' => '岗位报表汇总',
            'data' => $postData,
        );

        return $exportData;
    }
}
