<?php

namespace CorporateTrainingBundle\Biz\Exporter;

use AppBundle\Common\ArrayToolkit;
use CorporateTrainingBundle\Common\DateToolkit;
use CorporateTrainingBundle\Common\OrgToolkit;

class DataUserLearnExporter extends BaseDataStatisticsExporter
{
    public function getExportFileName()
    {
        return 'data_user_learn.xls';
    }

    public function getSortedHeadingRow()
    {
        return array(
            array('code' => 'truename', 'title' => '姓名'),
            array('code' => 'nickname', 'title' => '用户名'),
            array('code' => 'post', 'title' => '岗位'),
            array('code' => 'org', 'title' => '部门'),
            array('code' => 'postCourseProgress', 'title' => '岗位课程进度'),
            array('code' => 'postCourseLearnTime', 'title' => '岗位课程学习时长'),
            array('code' => 'joinCourseNum', 'title' => '加入总课程数'),
            array('code' => 'totalLearnTime', 'title' => '总学习时长'),
            array('code' => 'joinOfflineCourseNum', 'title' => '参加活动'),
            array('code' => 'passedOfflineCourseNum', 'title' => '通过考核活动数'),
        );
    }

    protected function buildExportData($parameters)
    {
        $users = null;
        $orgs = null;
        $posts = null;
        $conditions = $this->prepareSearchConditions($parameters);
        $conditions = array(
            'noType' => 'system',
            'locked' => 0,
        );
        $count = $this->getUserService()->countUsers($conditions);
        $users = $this->getUserService()->searchUsers(
            $conditions,
            array('id' => 'DESC'),
            0,
            $count
        );

        $userIds = ArrayToolkit::column($users, 'id');

        $results = $this->getUserLearnReportService()->searchUserLearnReports(array('ids' => $userIds), array(), 0, $count);
        $results = ArrayToolkit::index($results, 'userId');

        $userProfiles = $this->getUserService()->findUserProfilesByIds($userIds);
        $userProfiles = ArrayToolkit::index($userProfiles, 'id');

        $orgIds = array();

        foreach ($users as $user) {
            $orgIds = array_merge($orgIds, $user['orgIds']);
        }
        $orgIds = array_values(array_unique($orgIds));
        $orgs = $this->getOrgService()->findOrgsByIds($orgIds);
        $orgs = ArrayToolkit::index($orgs, 'id');

        if (!empty($results)) {
            $postIds = ArrayToolkit::column($results, 'postId');
            $posts = $this->getPostService()->findPostsByIds($postIds);
            $posts = ArrayToolkit::index($posts, 'id');
        }

        $resultData = array();

        foreach ($users as $user) {
            $resultData[] = array(
                'truename' => $userProfiles[$user['id']]['truename'],
                'nickname' => $user['nickname'],
                'post' => empty($results[$user['id']]) ? '-' : $this->getPostName($posts, $results[$user['id']]),
                'org' => OrgToolkit::buildOrgsNames($user['orgIds'], $orgs),
                'postCourseProgress' => empty($results[$user['id']]) ? '0/0' : $results[$user['id']]['finishedPostCourseNum'].'/'.$results[$user['id']]['postCourseNum'],
                'postCourseLearnTime' => empty($results[$user['id']]) ? '0' : DateToolkit::timeToHour($results[$user['id']]['postLearnTime']),
                'joinCourseNum' => empty($results[$user['id']]) ? '0' : $results[$user['id']]['joinCourseNum'],
                'totalLearnTime' => empty($results[$user['id']]) ? '0' : DateToolkit::timeToHour($results[$user['id']]['totalLearnTime']),
                'joinOfflineCourseNum' => empty($results[$user['id']]) ? '0' : $results[$user['id']]['joinOfflineCourseNum'],
                'passedOfflineCourseNum' => empty($results[$user['id']]) ? '0' : $results[$user['id']]['passedOfflineCourseNum'],
            );
        }

        $exportData[] = array(
            'sheetName' => '员工名册报表',
            'data' => $resultData,
        );

        return $exportData;
    }

    protected function getUserIdsByKeyWord($fields)
    {
        $conditions = array();
        if (!empty($fields['keywordType'])) {
            $conditions['keywordType'] = $fields['keywordType'];
        }
        if (!empty($fields['keyword'])) {
            $conditions['keyword'] = $fields['keyword'];
        }
        if (empty($conditions)) {
            return;
        }
        $users = $this->getUserService()->searchUsers($conditions, array(), 0, PHP_INT_MAX);
        if (empty($users)) {
            return;
        }

        return ArrayToolkit::column($users, 'id');
    }

    private function getUserName($users, $userId)
    {
        return empty($users[$userId]) ? '-' : $users[$userId]['nickname'];
    }

    private function getOrgName($orgs, $orgId)
    {
        return empty($orgs[$orgId]) ?: $orgs[$orgId]['name'];
    }

    private function getPostName($posts, $result)
    {
        $postName = empty($posts[$result['postId']]) ? '-' : $posts[$result['postId']]['name'];

        return $postName;
    }

    private function getUserTrueName($users, $userId)
    {
        return empty($users[$userId]['truename']) ? '--' : $users[$userId]['truename'];
    }
}
