<?php

namespace CorporateTrainingBundle\Biz\Exporter;

use AppBundle\Common\ArrayToolkit;
use CorporateTrainingBundle\Common\OrgToolkit;

class OfflineCourseBatchGradeExampleExporter extends BaseOfflineCourseExporter
{
    public function getExportFileName()
    {
        return 'batch_grade_example.xls';
    }

    protected function getSortedHeadingRow()
    {
        return array(
            array('code' => 'truename', 'title' => '姓名'),
            array('code' => 'nickname', 'title' => '用户名'),
            array('code' => 'org', 'title' => '部门'),
            array('code' => 'post', 'title' => '岗位'),
            array('code' => 'attendStatus', 'title' => '出勤情况'),
            array('code' => 'score', 'title' => '成绩'),
            array('code' => 'evaluate', 'title' => '表现评价'),
            array('code' => 'passStatus', 'title' => '考核结果'),
        );
    }

    protected function buildExportData($parameters)
    {
        $conditions = $this->prepareSearchConditions($parameters);
        $members = $this->getMemberService()->searchMembers(
            $conditions,
            array('createdTime' => 'DESC'),
            0,
            PHP_INT_MAX
        );

        $userIds = ArrayToolkit::column($members, 'userId');
        $users = $this->getUserService()->findUsersByIds($userIds);
        $profiles = $this->getUserService()->findUserProfilesByIds($userIds);
        $posts = $this->getPostService()->findPostsByIds(ArrayToolkit::column($users, 'postId'));
        $posts = ArrayToolkit::index($posts, 'id');

        $orgIds = array();

        foreach ($users as $user) {
            $orgIds = array_merge($orgIds, $user['orgIds']);
        }
        $orgIds = array_values(array_unique($orgIds));
        $orgs = $this->getOrgService()->findOrgsByIds($orgIds);
        $orgs = ArrayToolkit::index($orgs, 'id');

        $memberData = array();

        foreach ($members as $member) {
            $memberData[] = array(
                $user = $users[$member['userId']],
                'truename' => empty($profiles[$member['userId']]) ? '-' : $profiles[$member['userId']]['truename'],
                'nickname' => empty($users[$member['userId']]) ? '-' : $users[$member['userId']]['nickname'],
                'org' => OrgToolkit::buildOrgsNames($user['orgIds'], $orgs),
                'post' => empty($posts[$users[$member['userId']]['postId']]) ? '-' : $posts[$users[$member['userId']]['postId']]['name'],
                'attendStatus' => '',
                'score' => '',
                'evaluate' => '',
                'passStatus' => '',
            );
        }

        $course = $this->getCourseService()->getOfflineCourse($parameters['courseId']);

        $exportData[] = array(
            'sheetName' => $course['title'],
            'data' => $memberData,
        );

        return $exportData;
    }
}
