<?php

namespace CorporateTrainingBundle\Biz\OfflineCourse\Dao\Impl;

use Codeages\Biz\Framework\Dao\GeneralDaoImpl;
use CorporateTrainingBundle\Biz\OfflineCourse\Dao\MemberDao;

class MemberDaoImpl extends GeneralDaoImpl implements MemberDao
{
    protected $table = 'offline_course_member';

    public function findByCourseId($courseId)
    {
        return $this->findByFields(array(
            'offlineCourseId' => $courseId,
        ));
    }

    public function getByCourseIdAndUserId($courseId, $userId)
    {
        return $this->getByFields(
            array(
                'offlineCourseId' => $courseId,
                'userId' => $userId,
            )
        );
    }

    public function findDistinctUserIdsByCreatedTime($startTime, $endTime)
    {
        $sql = "SELECT DISTINCT userId FROM {$this->table} WHERE createdTime >= ? AND createdTime <= ? ";

        return $this->db()->fetchAll($sql, array($startTime, $endTime));
    }

    public function statisticAttendStatusByCourseId($courseId)
    {
        $sql = "SELECT attendedStatus, COUNT(id) AS count FROM  {$this->table}  WHERE offlineCourseId = ? GROUP BY attendedStatus";

        return $this->db()->fetchAll($sql, array($courseId));
    }

    public function statisticPassStatusByCourseId($courseId)
    {
        $sql = "SELECT passedStatus, COUNT(id) AS count FROM  {$this->table}  WHERE offlineCourseId = ? GROUP BY passedStatus";

        return $this->db()->fetchAll($sql, array($courseId));
    }

    public function statisticScoreByCourseId($courseId)
    {
        $sql = "SELECT sum(CASE WHEN score <= 10 AND score >= 0 THEN 1 ELSE 0 END) AS '0-10' ,
                       sum(CASE WHEN score <= 20 AND score > 10 THEN 1 ELSE 0 END) AS '11-20', 
                       sum(CASE WHEN score <= 30 AND score > 20 THEN 1 ELSE 0 END) AS '21-30',
                       sum(CASE WHEN score <= 40 AND score > 30 THEN 1 ELSE 0 END) AS '31-40',
                       sum(CASE WHEN score <= 50 AND score > 40 THEN 1 ELSE 0 END) AS '41-50',
                       sum(CASE WHEN score <= 60 AND score > 50 THEN 1 ELSE 0 END) AS '51-60',
                       sum(CASE WHEN score <= 70 AND score > 60 THEN 1 ELSE 0 END) AS '61-70',
                       sum(CASE WHEN score <= 80 AND score > 70 THEN 1 ELSE 0 END) AS '71-80',
                       sum(CASE WHEN score <= 90 AND score > 80 THEN 1 ELSE 0 END) AS '81-90',
                       sum(CASE WHEN score <= 100 AND score > 90  THEN 1 ELSE 0 END) AS '91-100'       
                      FROM {$this->table} WHERE offlineCourseId = ? AND passedStatus IN ('passed', 'unpassed')";

        return $this->db()->fetchAll($sql, array($courseId));
    }

    public function declares()
    {
        return array(
            'timestamps' => array('createdTime', 'updatedTime'),
            'orderbys' => array(
                'id',
                'score',
                'createdTime',
                'updatedTime',
            ),
            'conditions' => array(
                'userId = :userId',
                'joinStatus = :joinStatus',
                'offlineCourseId = :courseId',
                'passedStatus = :passedStatus',
                'userId IN ( :userIds)',
                'offlineCourseId IN ( :offlineCourseIds)',
            ),
        );
    }
}
