<?php

namespace CorporateTrainingBundle\Biz\OfflineCourse\Event;

use Biz\Sms\Service\SmsService;
use Biz\User\Service\NotificationService;
use Codeages\Biz\Framework\Event\Event;
use Codeages\PluginBundle\Event\EventSubscriber;
use CorporateTrainingBundle\Biz\OfflineCourse\Service\EnrollmentRecordService;
use CorporateTrainingBundle\Biz\OfflineCourse\Service\OfflineCourseService;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class OfflineCourseMemberEventSubscriber extends EventSubscriber implements EventSubscriberInterface
{
    public static function getSubscribedEvents()
    {
        return array(
            'offline.become.member' => 'onBecomeMember',
            'offline.member.delete' => 'onMemberDelete',
            'offline.reject.apply' => 'onRejectApply',
        );
    }

    public function onBecomeMember(Event $event)
    {
        $member = $event->getSubject();
        $offlineCourse = $this->getOfflineCourseService()->getOfflineCourse($member['offlineCourseId']);

        if (empty($member) || empty($offlineCourse)) {
            return;
        }

        $this->smsBecomeMember($member['userId'], $offlineCourse);

        $this->notifyBecomeMember($member['userId'], $offlineCourse);
    }

    protected function smsBecomeMember($userId, $offlineCourse)
    {
        $smsType = 'sms_activity_signup';

        $isOpen = $this->getSmsService()->isOpen($smsType);
        if (!$isOpen) {
            return;
        }

        $parameters = array(
            'campaign' => $offlineCourse['title'],
            'date' => date('Y-m-d H:i', $offlineCourse['startDate']),
            'address' => $offlineCourse['address'],
        );

        $this->getSmsService()->smsSend($smsType, array($userId), array(), $parameters);
    }

    protected function notifyBecomeMember($userId, $offlineCourse)
    {
        $this->getNotificationService()->notify($userId, 'offline_course_become_member', $offlineCourse);
    }

    public function onRejectApply(Event $event)
    {
        $record = $event->getSubject();
        $offlineCourse = $this->getOfflineCourseService()->getOfflineCourse($record['offlineCourseId']);

        if (empty($offlineCourse) || empty($record)) {
            return;
        }

        $this->smsRejectApply($record['userId'], $offlineCourse, $record);

        $this->notifyRejectApply($record['userId'], $offlineCourse, $record);
    }

    protected function smsRejectApply($userId, $offlineCourse, $record)
    {
        $smsType = 'sms_activity_signup_fail';

        $isOpen = $this->getSmsService()->isOpen($smsType);
        if (!$isOpen) {
            return;
        }

        $parameters = array(
            'campaign' => $offlineCourse['title'],
            'reason' => $record['rejectedReason'],
        );

        $this->getSmsService()->smsSend($smsType, array($userId), array(), $parameters);
    }

    protected function notifyRejectApply($userId, $offlineCourse, $record)
    {
        $content['offlineCourse'] = $offlineCourse;
        $content['record'] = $record;
        $this->getNotificationService()->notify($userId, 'offline_course_enroll_rejected', $content);
    }

    public function onMemberDelete(Event $event)
    {
        $member = $event->getSubject();
        $this->getEnrollmentRecordService()->deleteEnrollmentRecordByCourseIdAndUserId($member['offlineCourseId'], $member['userId']);
        $this->getOfflineCourseService()->refreshMemberCount($member['offlineCourseId']);
    }

    /**
     * @return SmsService
     */
    protected function getSmsService()
    {
        return $this->getBiz()->service('Sms:SmsService');
    }

    /**
     * @return OfflineCourseService
     */
    protected function getOfflineCourseService()
    {
        return $this->getBiz()->service('CorporateTrainingBundle:OfflineCourse:OfflineCourseService');
    }

    /**
     * @return EnrollmentRecordService
     */
    protected function getEnrollmentRecordService()
    {
        return $this->getBiz()->service('CorporateTrainingBundle:OfflineCourse:EnrollmentRecordService');
    }

    /**
     * @return NotificationService
     */
    protected function getNotificationService()
    {
        return $this->getBiz()->service('User:NotificationService');
    }
}
