<?php

namespace CorporateTrainingBundle\Biz\OfflineCourse\Service\Impl;

use AppBundle\Common\ArrayToolkit;
use Biz\BaseService;
use Codeages\Biz\Framework\Event\Event;
use CorporateTrainingBundle\Biz\OfflineCourse\Dao\EnrollmentRecordDao;
use CorporateTrainingBundle\Biz\OfflineCourse\Service\EnrollmentRecordService;
use CorporateTrainingBundle\Biz\OfflineCourse\Service\MemberService;
use CorporateTrainingBundle\Biz\OfflineCourse\Service\OfflineCourseService;

class EnrollmentRecordServiceImpl extends BaseService implements EnrollmentRecordService
{
    public function createEnrollmentRecord($record)
    {
        if (!ArrayToolkit::requireds($record, array('userId', 'offlineCourseId'))) {
            throw $this->createServiceException('parameter is invalid!');
        }

        $record = $this->filterFields($record);

        return $this->getEnrollmentRecordDao()->create($record);
    }

    public function updateEnrollmentRecord($id, $fields)
    {
        $fields = $this->filterFields($fields);

        return $this->getEnrollmentRecordDao()->update($id, $fields);
    }

    public function getEnrollmentRecord($id)
    {
        return $this->getEnrollmentRecordDao()->get($id);
    }

    public function countEnrollmentRecords(array $conditions)
    {
        return $this->getEnrollmentRecordDao()->count($conditions);
    }

    public function searchEnrollmentRecords(array $conditions, array $orderBys, $start, $limit)
    {
        return $this->getEnrollmentRecordDao()->search($conditions, $orderBys, $start, $limit);
    }

    public function getLatestEnrollmentRecordByCourseIdAndUserId($courseId, $userId)
    {
        return $this->getEnrollmentRecordDao()->getLatestByCourseIdAndUserId($courseId, $userId);
    }

    public function passOfflineCourseApply($recordId)
    {
        if (!$this->canAuditApply($recordId)) {
            throw $this->createServiceException('can not audit this apply');
        }

        $record = $this->getEnrollmentRecord($recordId);
        if ('submitted' != $record['status']) {
            throw $this->createServiceException('This apply cant not be approved');
        }

        $record = $this->updateEnrollmentRecord($recordId, array(
            'status' => 'approved',
            'approvedTime' => time(),
        ));

        $this->getMemberService()->becomeMember($record['offlineCourseId'], $record['userId']);

        return $record;
    }

    public function rejectOfflineCourseApply($recordId, $info = array())
    {
        if (!$this->canAuditApply($recordId)) {
            throw $this->createServiceException('can not audit this apply');
        }

        $record = $this->getEnrollmentRecord($recordId);
        if ('submitted' != $record['status']) {
            throw $this->createServiceException('This apply can not be rejected');
        }

        $fields['status'] = 'rejected';
        if (!empty($info['rejectedReason'])) {
            $fields['rejectedReason'] = $info['rejectedReason'];
        }

        $result = $this->updateEnrollmentRecord($recordId, $fields);
        $this->dispatchEvent('offline.reject.apply',
            new Event($result));

        return $result;
    }

    public function batchPass($recordIds)
    {
        try {
            $this->biz['db']->beginTransaction();

            foreach ($recordIds as $recordId) {
                $this->passOfflineCourseApply($recordId);
            }

            $this->biz['db']->commit();
        } catch (\Exception $e) {
            $this->biz['db']->rollback();
            throw $e;
        }
    }

    public function batchReject($recordIds)
    {
        try {
            $this->biz['db']->beginTransaction();

            foreach ($recordIds as $recordId) {
                $this->rejectOfflineCourseApply($recordId);
            }

            $this->biz['db']->commit();
        } catch (\Exception $e) {
            $this->biz['db']->rollback();
            throw $e;
        }
    }

    public function statisticsOfflineCourseSubmittedStudentNum($courseId)
    {
        return $this->getEnrollmentRecordDao()->statisticsSubmittedStudentNum($courseId);
    }

    public function findEnrollmentRecordsByCourseId($courseId)
    {
        return $this->getEnrollmentRecordDao()->findByCourseId($courseId);
    }

    public function deleteEnrollmentRecordByCourseIdAndUserId($courseId, $userId)
    {
        return $this->getEnrollmentRecordDao()->deleteByCourseIdAndUserId($courseId, $userId);
    }

    protected function canAuditApply($recordId)
    {
        if (!$this->getCourseService()->hasCourseManageRole()) {
            return false;
        }

        $record = $this->getEnrollmentRecord($recordId);
        if (empty($record)) {
            return false;
        }

        return true;
    }

    protected function filterFields($fields)
    {
        return ArrayToolkit::parts(
            $fields,
            array(
                'userId',
                'offlineCourseId',
                'submittedTime',
                'status',
                'approvedTime',
                'rejectedReason',
            )
        );
    }

    /**
     * @return OfflineCourseService
     */
    protected function getCourseService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:OfflineCourseService');
    }

    /**
     * @return MemberService
     */
    protected function getMemberService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:MemberService');
    }

    /**
     * @return EnrollmentRecordDao
     */
    protected function getEnrollmentRecordDao()
    {
        return $this->createDao('CorporateTrainingBundle:OfflineCourse:EnrollmentRecordDao');
    }
}
