<?php

namespace CorporateTrainingBundle\Biz\Task\Dao\Impl;

use Biz\Task\Dao\Impl\TaskResultDaoImpl as BaseDaoImpl;
use CorporateTrainingBundle\Biz\Task\Dao\TaskResultDao;

class TaskResultDaoImpl extends BaseDaoImpl implements TaskResultDao
{
    public function sumLearnTimeByCourseIdAndUserId($courseId, $userId)
    {
        $sql = 'SELECT sum(TIME) FROM `course_task_result` WHERE `courseId` = ? AND `userId`= ?';

        return $this->db()->fetchColumn($sql, array($courseId, $userId));
    }

    public function sumLearnTimeByPostIdAndUserId($postId, $userId)
    {
        $sql = "SELECT sum(time) FROM {$this->table()} JOIN `post_course` ON course_task_result.courseId = post_course.courseId WHERE `postId` = ? AND `userId` = ? ";

        return $this->db()->fetchColumn($sql, array($postId, $userId));
    }

    public function sumLearnTimeByUserId($userId)
    {
        $sql = 'SELECT sum(TIME) FROM `course_task_result` WHERE `userId`= ?';

        return $this->db()->fetchColumn($sql, array($userId));
    }

    public function sumLearnTimeByTaskIdAndUserId($taskId, $userId)
    {
        $sql = 'SELECT sum(time) FROM `course_task_result` WHERE `courseTaskId` = ? AND `userId`= ?';

        return $this->db()->fetchColumn($sql, array($taskId, $userId));
    }

    public function sumLearnTimeByCategoryIdAndUserId($categoryId, $userId)
    {
        $sql = "SELECT sum(time) FROM {$this->table()} JOIN `course_set_v8` ON course_task_result.courseId = course_set_v8.id WHERE `categoryId` = ?AND `userId` = ?";

        return $this->db()->fetchColumn($sql, array($categoryId, $userId));
    }

    public function sumWatchTimeByCourseIdAndUserId($courseId, $userId)
    {
        $sql = 'SELECT sum(watchTime) FROM `course_task_result` WHERE`courseId` = ? AND `userId`= ?';

        return $this->db()->fetchColumn($sql, array($courseId, $userId));
    }

    public function sumWatchTimeByPostIdAndUserId($postId, $userId)
    {
        $sql = "SELECT sum(watchTime) FROM {$this->table()} JOIN `post_course` ON course_task_result.courseId = post_course.courseId WHERE `postId` = ? AND `userId` = ? ";

        return $this->db()->fetchColumn($sql, array($postId, $userId));
    }

    public function sumWatchTimeByTaskIdAndUserId($taskId, $userId)
    {
        $sql = 'SELECT sum(watchTime) FROM `course_task_result` WHERE`courseTaskId` = ? AND `userId`= ?';

        return $this->db()->fetchColumn($sql, array($taskId, $userId));
    }

    public function sumWatchTimeByCategoryIdAndUserId($categoryId, $userId)
    {
        $sql = "SELECT sum(watchTime) FROM {$this->table()} JOIN `course_set_v8` ON course_task_result.courseId = course_set_v8.id WHERE `categoryId` = ?AND `userId` = ?";

        return $this->db()->fetchColumn($sql, array($categoryId, $userId));
    }

    public function findFinishedTimeGroupByUserId(array $courseIds)
    {
        $sql = 'SELECT userId, MAX(finishedTime) as finishedTime  FROM `course_task_result` WHERE `status` = ? AND `courseId` IN (?) group BY userId ';

        return $this->db()->fetchAll($sql, array('finish', implode($courseIds))) ?: array();
    }

    public function sumLearnTimeByCourseIdsAndUserIdsGroupByUserId(array $courseIds, array $userIds)
    {
        $builder = $this->createQueryBuilder(array('courseIds' => $courseIds, 'userIds' => $userIds))
            ->select('sum(time) as totalLearnTime, userId')
            ->groupBy('userId');

        return $builder->execute()->fetchAll();
    }

    public function sumLearnTimeByCourseId($courseId)
    {
        $sql = "SELECT SUM(time) AS totalLearnTime FROM {$this->table} WHERE courseId = ?";

        return $this->db()->fetchColumn($sql, array($courseId));
    }

    public function declares()
    {
        $declares = parent::declares();
        array_push($declares['conditions'], 'updatedTime >= :updatedTime_GE');
        array_push($declares['conditions'], 'userId IN (:userIds)');

        return $declares;
    }
}
