<?php

namespace CorporateTrainingBundle\Biz\TrainingAssignment\Event;

use AppBundle\Common\ArrayToolkit;
use Biz\User\Service\NotificationService;
use Codeages\Biz\Framework\Event\Event;
use Codeages\Biz\Framework\Event\EventSubscriber;
use CorporateTrainingBundle\Biz\TrainingAssignment\Service\TrainingAssignmentMemberService;
use CorporateTrainingBundle\Biz\TrainingAssignment\Service\TrainingAssignmentItemService;
use CorporateTrainingBundle\Biz\TrainingAssignment\Service\TrainingAssignmentService;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class MemberEventSubscriber extends EventSubscriber implements EventSubscriberInterface
{
    public static function getSubscribedEvents()
    {
        return array(
            'become_training_assignment_member' => 'onBecomeTrainingAssignmentMember',
            'course.task.finish' => array('onCourseTaskFinish', -100),
        );
    }

    public function onBecomeTrainingAssignmentMember(Event $event)
    {
        $member = $event->getSubject();

        $assignment = $this->getTrainingAssignmentService()->getTrainingAssignment($member['assignmentId']);

        if (empty($member) || empty($assignment) || $assignment['status'] != 'published') {
            return;
        }

        $this->getNotificationService()->notify($member['userId'], 'assignment_become_member', $assignment);
    }

    public function onCourseTaskFinish(Event $event)
    {
        $taskResult = $event->getSubject();
        $user = $event->getArgument('user');

        $assignmentItems = $this->getTrainingAssignmentItemService()->findTrainingAssignmentItemsByTargetIdAndTargetType($taskResult['courseId'], 'course');

        $assignmentIds = ArrayToolkit::column($assignmentItems, 'assignmentId');

        foreach ($assignmentIds as &$assignmentId) {
            $member = $this->getTrainingAssignmentMemberService()->getTrainingAssignmentMemberByUserIdAndAssignmentId($user['id'], $assignmentId);

            $result = $this->createAssignmentStrategy($assignmentId)->isTrainingAssignmentFinished($assignmentId, $user['id']);

            if (empty($result)) {
                $this->getTrainingAssignmentMemberService()->updateTrainingAssignmentMember($member['id'], array('status' => 'ongoing'));
            } else {
                $this->getTrainingAssignmentMemberService()->updateTrainingAssignmentMember($member['id'], array('finishedTime' => time(), 'status' => 'finished'));
            }
        }
    }

    protected function createAssignmentStrategy()
    {
        return $this->getBiz()->offsetGet('assignment_strategy_context')->createStrategy();
    }

    /**
     * @return TrainingAssignmentService
     */
    protected function getTrainingAssignmentService()
    {
        return $this->getBiz()->service('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentService');
    }

    /**
     * @return TrainingAssignmentMemberService
     */
    protected function getTrainingAssignmentMemberService()
    {
        return $this->getBiz()->service('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentMemberService');
    }

    /**
     * @return TrainingAssignmentItemService
     */
    protected function getTrainingAssignmentItemService()
    {
        return $this->getBiz()->service('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentItemService');
    }

    /**
     * @return NotificationService
     */
    protected function getNotificationService()
    {
        return $this->getBiz()->service('User:NotificationService');
    }
}
