<?php

namespace CorporateTrainingBundle\Biz\TrainingAssignment\Strategy\Impl;

use AppBundle\Common\ArrayToolkit;
use CorporateTrainingBundle\Biz\TrainingAssignment\Strategy\AssignmentStrategy;
use CorporateTrainingBundle\Biz\TrainingAssignment\Strategy\BaseAssignmentStrategy;

class CourseAssignmentStrategyImpl extends BaseAssignmentStrategy implements AssignmentStrategy
{
    public function findAssignmentItemIds($assignment)
    {
        $assignmentItems = $this->getTrainingAssignmentItemService()->findAssignmentItemsByAssignmentIdAndTargetType($assignment['id'], 'course');
        $courseIds = ArrayToolkit::column($assignmentItems, 'targetId');
        if (empty($courseIds)) {
            return array();
        }

        $courses = $this->getCourseService()->searchCourses(
            array('courseIds' => $courseIds, 'status' => 'published'),
            array(),
            0,
            PHP_INT_MAX
        );

        return ArrayToolkit::column($courses, 'id');
    }

    public function appendProgress($assignment, $members)
    {
        $courseIds = $this->findAssignmentItemIds($assignment);

        foreach ($members as &$member) {
            if (empty($courseIds)) {
                $progress['percent'] = 0;
            } else {
                $progress = $this->getLearningDataAnalysisService()->getUserLearningProgressByCourseIds($courseIds, $member['userId']);
            }

            $member['progress'] = $progress['percent'];
        }

        return $members;
    }

    public function calculateMemberLearnProgress($assignment, $userId)
    {
        $courseIds = $this->findAssignmentItemIds($assignment);

        if (empty($courseIds)) {
            $progress['percent'] = 0;
        } else {
            $progress = $this->getLearningDataAnalysisService()->getUserLearningProgressByCourseIds($courseIds, $userId);
        }

        return $progress['percent'];
    }

    public function calculateMembersTotalLearnTime($assignment, array $userIds)
    {
        $courseIds = $this->findAssignmentItemIds($assignment);
        $courseIds = empty($courseIds) ? array(-1) : $courseIds;
        $usersTotalLearnTime = $this->getTaskResultService()->sumLearnTimeByCourseIdsAndUserIdsGroupByUserId($courseIds, ArrayToolkit::column($userIds, 'userId'));

        $usersTotalLearnTime = ArrayToolkit::index($usersTotalLearnTime, 'userId');
        $usersLearnTime = array();
        foreach ($userIds as $userId) {
            $totalLearnTime = empty($usersTotalLearnTime[$userId]) ? 0 : $usersTotalLearnTime[$userId]['totalLearnTime'];
            $usersLearnTime[$userId] = $totalLearnTime;
        }

        return $usersLearnTime;
    }

    public function getUserAssignmentScoreResults($assignment, $userId)
    {
        $courseIds = $this->findAssignmentItemIds($assignment);
        $conditions = array(
            'courseIds' => $courseIds,
            'userId' => $userId,
            'types' => array('homework', 'testpaper'),
        );

        return  $this->getTestPaperService()->searchTestpaperResults(
            $conditions,
            array('id' => 'ASC'),
            0,
            PHP_INT_MAX
        );
    }

    public function isTrainingAssignmentFinished($assignmentId, $userId)
    {
        $assignment = $this->getTrainingAssignmentService()->getTrainingAssignment($assignmentId);
        $memberAssignmentProgress = $this->calculateMemberLearnProgress($assignment, $userId);

        $isFinished = false;
        if (100 === (int) $memberAssignmentProgress) {
            $isFinished = true;
        }

        return $isFinished;
    }

    /**
     * @return \Biz\Course\Service\Impl\CourseServiceImpl
     */
    protected function getCourseService()
    {
        return $this->createService('Course:CourseService');
    }

    /**
     * @return \Biz\Course\Service\Impl\CourseSetServiceImpl
     */
    protected function getCourseSetService()
    {
        return $this->createService('Course:CourseSetService');
    }

    /**
     * @return \Biz\Task\Service\Impl\TaskServiceImpl
     */
    protected function getTaskService()
    {
        return $this->createService('Task:TaskService');
    }

    /**
     * @return \CorporateTrainingBundle\Biz\Task\Service\Impl\TaskResultServiceImpl
     */
    protected function getTaskResultService()
    {
        return $this->createService('CorporateTrainingBundle:Task:TaskResultService');
    }

    /**
     * @return \Biz\Activity\Service\Impl\ActivityServiceImpl
     */
    protected function getActivityService()
    {
        return $this->createService('Activity:ActivityService');
    }

    /**
     * @return \Biz\Testpaper\Service\Impl\TestpaperServiceImpl
     */
    protected function getTestPaperService()
    {
        return $this->createService('Testpaper:TestpaperService');
    }

    /**
     * @return \Biz\Course\Service\Impl\LearningDataAnalysisServiceImpl
     */
    protected function getLearningDataAnalysisService()
    {
        return $this->createService('Course:LearningDataAnalysisService');
    }
}
