<?php

namespace CorporateTrainingBundle\Component\EIMClient;

use InvalidArgumentException;

class DepartmentFactory
{
    /**
     * 创建EIM Department Client实例.
     *
     * @param string $type   Client的类型
     * @param array  $config 必需包含key, secret两个参数
     *
     * @return EIM Department Client
     */
    public static function create($config)
    {
        if (!array_key_exists('key', $config) || !array_key_exists('secret', $config)) {
            throw new InvalidArgumentException('参数中必需包含key, secret两个为key的值');
        }

        $departments = self::departments();

        if (!array_key_exists($config['type'], $departments)) {
            throw new InvalidArgumentException(array('参数不正确%type%', array('%type%' => $config['type'])));
        }

        $class = $departments[$config['type']]['class'];

        return new $class($config);
    }

    public static function departments()
    {
        $departments = array(
            'dingtalk' => array(
                'class' => 'CorporateTrainingBundle\Component\EIMClient\DingTalk\Department',
            ),
        );

        return $departments;
    }
}
