<?php

namespace CorporateTrainingBundle\Controller\Admin;

use AppBundle\Common\ArrayToolkit;
use Symfony\Component\HttpFoundation\Request;
use AppBundle\Common\CurlToolkit;
use AppBundle\Controller\Admin\AnalysisController as BaseAnalysisController;

class AnalysisController extends BaseAnalysisController
{
    public function indexAction(Request $request)
    {
        $data = array();
        $count = 0;

        $condition = $request->query->all();
        $condition['analysisDateType'] = 'login';
        $timeRange = $this->getTimeRange($condition);

        if (!$timeRange) {
            $this->setFlashMessage('danger', '输入的日期有误!');

            return $this->redirect($this->generateUrl('admin'));
        }

        $loginData = $this->getLogService()->analysisLoginDataByTime($timeRange['startTime'], $timeRange['endTime']);

        $data = $this->fillAnalysisData($condition, $loginData);
        foreach ($loginData as $key => $value) {
            $count += $value['count'];
        }

        $loginStartData = $this->getLogService()->searchLogs(array('action' => 'login_success'), 'createdByAsc', 0, 1);

        if ($loginStartData) {
            $loginStartDate = date('Y-m-d', $loginStartData[0]['createdTime']);
        }

        $dataInfo = $this->getDataInfo($condition, $timeRange);

        return $this->render('default/corporate-training-admin-index.html.twig', array(
            'data' => $data,
            'count' => $count,
            'loginStartDate' => $loginStartDate,
            'dataInfo' => $dataInfo,
        ));
    }

    public function versionAction(Request $request)
    {
        $apps = $this->getAppService()->checkAppUpgrades();

        if (!is_null($apps)) {
            $indexApps = ArrayToolkit::index($apps, 'code');
        }
        $mainAppUpgrade = empty($indexApps['TRAININGMAIN']) ? array() : $indexApps['TRAININGMAIN'];

        $android = $this->getMobileVersion('android');
        $ios = $this->getMobileVersion('iphone');

        return $this->render('default/version.html.twig', array(
            'android' => $android,
            'ios' => $ios,
            'mainAppUpgrade' => $mainAppUpgrade,
        ));
    }

    public function userStatisticsAction(Request $request)
    {
        $userCount = $this->getUserService()->countUsers(array('noType' => 'system', 'locked' => 0));

        $conditions = array(
            'loginStartTime' => strtotime(date('Y-m-d')),
            'loginEndTime' => strtotime(date('Y-m-d')) + 86400,
        );

        $loginCount = $this->getUserService()->countUsers($conditions);

        $conditions = array(
            'updatedTime_GE' => strtotime(date('Y-m-d')),
            'finishedTime_LT' => strtotime(date('Y-m-d')) + 86400,
        );

        $todayLearned = $this->getTaskResultService()->searchTaskResults($conditions, array('finishedTime' => 'desc'), '0', PHP_INT_MAX);

        $users = ArrayToolkit::column($todayLearned, 'userId');
        $taskNum = count(array_unique($users));

        $userCountInfo = $this->getUserService()->countUsersByLockedStatus();

        return $this->render('default/user-statistics.html.twig', array(
            'userCount' => $userCount,
            'userCountInfo' => $userCountInfo,
            'loginCount' => $loginCount,
            'learnedCount' => $taskNum,
        ));
    }

    public function userActiveChartAction(Request $request)
    {
        $data = array();
        $count = 0;

        $condition = $request->request->all();

        $condition['analysisDateType'] = 'login';
        $timeRange = $this->getTimeRange($condition);

        if (!$timeRange) {
            $this->setFlashMessage('danger', '输入的日期有误!');

            return $this->redirect($this->generateUrl('admin'));
        }

        $loginData = $this->getLogService()->analysisLoginDataByTime($timeRange['startTime'], $timeRange['endTime']);

        $data = $this->fillAnalysisData($condition, $loginData);
        foreach ($loginData as $key => $value) {
            $count += $value['count'];
        }

        $loginStartData = $this->getLogService()->searchLogs(array('action' => 'login_success'), 'createdByAsc', 0, 1);

        if ($loginStartData) {
            $loginStartDate = date('Y-m-d', $loginStartData[0]['createdTime']);
        }

        $dataInfo = $this->getDataInfo($condition, $timeRange);

        return $this->render('default/user-active-chart.html.twig', array(
            'data' => $data,
            'count' => $count,
            'loginStartDate' => empty($loginStartDate) ? array() : $loginStartDate,
            'dataInfo' => $dataInfo,
        ));
    }

    public function validateSyncOrgAction(Request $request)
    {
        $inspectList = array(
            $this->addInspectRole('host', $this->inspectSyncOrg($request)),
        );
        $inspectList = array_filter($inspectList);

        return $this->render('admin/default/domain.html.twig', array(
            'inspectList' => $inspectList,
        ));
    }

    private function addInspectRole($name, $value)
    {
        if (empty($value) || $value['status'] == 'ok') {
            return array();
        }

        return array('name' => $name, 'value' => $value);
    }

    private function inspectSyncOrg($request)
    {
        $sync = $this->getSettingService()->get('sync_department_setting', array());
        $siteSetting = $this->getSettingService()->get('site');

        if ($sync && $sync['enable'] && $sync['times'] <= 0) {
            return array(
                'status' => 'warning',
                'errorMessage' => '钉钉企业同步开启后，请在【组织机构管理】中对组织机构完成【初始化同步】',
                'except' => $siteSetting['url'],
                'actually' => $request->server->get('HTTP_HOST'),
                'settingUrl' => $this->generateUrl('admin_org'),
            );
        }

        return array();
    }

    protected function getDataInfo($condition, $timeRange)
    {
        $startTime = strtotime(date('Y-m', $timeRange['startTime']).'-01 00:00:01');
        $endTime = strtotime(date('Y-m', $timeRange['endTime']).'-01 00:00:01');

        return array(
            'startTime' => date('Y-m-d', $timeRange['startTime']),
            'endTime' => date('Y-m-d', $timeRange['endTime'] - 24 * 3600),
            'currentMonthStart' => date('Y-m-d', strtotime(date('Y-m', time()))),
            'currentMonthEnd' => date('Y-m-d', strtotime(date('Y-m-d', time()))),
            'lastMonthStart' => date('Y-m-d', strtotime(date('Y-m', strtotime('-1 month', $startTime)))),
            'lastMonthEnd' => date('Y-m-d', strtotime(date('Y-m', $endTime)) - 24 * 3600),
            'lastThreeMonthsStart' => date('Y-m-d', strtotime(date('Y-m', strtotime('-2 month', $startTime)))),
            'lastThreeMonthsEnd' => date('Y-m-d', strtotime(date('Y-m-d', time()))),
            'analysisDateType' => $condition['analysisDateType'], );
    }

    private function getMobileVersion($system)
    {
        $mobileVersion = CurlToolkit::request('GET', "http://www.edusoho.com/version/edusoho-training-{$system}", array());
        if (empty($mobileVersion)) {
            return;
        }

        return $mobileVersion;
    }

    protected function getAppService()
    {
        return $this->createService('CloudPlatform:AppService');
    }

    protected function getSettingService()
    {
        return $this->createService('System:SettingService');
    }
}
