<?php

namespace CorporateTrainingBundle\Controller\Admin;

use AppBundle\Common\ArrayToolkit;
use AppBundle\Controller\Admin\BaseController;
use Symfony\Component\HttpFoundation\Request;
use AppBundle\Common\Paginator;

class OfflineCourseCategoryController extends BaseController
{
    public function categoryAction(Request $request)
    {
        $conditions = $request->query->all();

        $group = $this->getCategoryService()->getGroupByCode('offlinecourse');

        if (empty($group)) {
            throw $this->createNotFoundException();
        }
        $conditions['groupId'] = $group['id'];

        $offlineCategoriesCount = $this->getCategoryService()->countCategories($conditions);

        $paginator = new Paginator($request, $offlineCategoriesCount, 10);

        $offlineCategories = $this->getCategoryService()->searchCategories(
            $conditions,
            array('weight' => 'ASC'),
            $paginator->getOffsetCount(),
            $paginator->getPerPageCount()
        );
        $categoryIds = ArrayToolkit::column($offlineCategories, 'id');
        $conditions['status'] = 'published';
        $conditions['categoryIds'] = $categoryIds;
        $allOfflineCourse = $this->getOfflineCourseService()->searchOfflineCourses(
            $conditions,
            array(),
            0,
            PHP_INT_MAX
        );
        $groupCourses = ArrayToolkit::group($allOfflineCourse, 'categoryId');
        foreach ($offlineCategories as &$category) {
            if (!empty($groupCourses[$category['id']])) {
                $category = $this->buildCategoryData($groupCourses[$category['id']], $category);
            } else {
                $category['count'] = array('handing' => 0, 'end' => 0);
                $category['mem'] = 0;
                $category['pass'] = 0;
                $category['join'] = 0;
            }
        }

        return $this->render('admin/offline-course/category/category.html.twig', array(
            'offlineCategorysCount' => $offlineCategoriesCount,
            'paginator' => $paginator,
            'group' => $group,
            'categories' => $offlineCategories,
        ));
    }

    public function createAction(Request $request)
    {
        if ($request->getMethod() == 'POST') {
            $category = $this->getCategoryService()->createCategory($request->request->all());

            return $this->renderTbody($category['groupId']);
        }

        $category = array(
            'id' => 0,
            'name' => '',
            'code' => '',
            'description' => '',
            'groupId' => (int) $request->query->get('groupId'),
            'parentId' => (int) $request->query->get('parentId', 0),
            'weight' => 0,
            'icon' => '',
        );

        return $this->render('admin/offline-course/category/modal.html.twig', array(
            'category' => $category,
        ));
    }

    public function editAction(Request $request, $id)
    {
        $category = $this->getCategoryService()->getCategory($id);

        if (empty($category)) {
            throw $this->createNotFoundException();
        }

        if ($request->getMethod() == 'POST') {
            $category = $this->getCategoryService()->updateCategory($id, $request->request->all());

            return $this->renderTbody($category['groupId']);
        }

        return $this->render('admin/offline-course/category/modal.html.twig', array(
            'category' => $category,
        ));
    }

    public function deleteAction(Request $request, $id)
    {
        $category = $this->getCategoryService()->getCategory($id);

        if (empty($category)) {
            throw $this->createNotFoundException();
        }

        $course = $this->getOfflineCourseService()->searchOfflineCourses(
            array('categoryId' => $id),
            array(),
            0,
            1
        );
        if (!empty($course)) {
            return $this->createJsonResponse(array('result' => false, 'message' => '该类型下已创建活动，不可直接删除！'));
        }

        $this->getCategoryService()->deleteCategory($id);

        return $this->createJsonResponse(true);
    }

    protected function renderTbody($groupId)
    {
        $group = $this->getCategoryService()->getGroup($groupId);
        $categories = $this->getCategoryService()->getCategoryStructureTree($groupId);

        return $this->render('admin/offline-course/category/tbody.html.twig', array(
            'categories' => $categories,
            'group' => $group,
        ));
    }

    private function buildCategoryData(array $courses, $category)
    {
        $handing = 0;
        $end = 0;
        $memjoin = 0;
        $mempass = 0;
        $courseIds = ArrayToolkit::column($courses, 'id');
        foreach ($courses as $value) {
            if ($value['endDate'] > time()) {
                ++$handing;
            } elseif ($value['endDate'] < time()) {
                ++$end;
            }
        }

        $conditions = array('offlineCourseIds' => $courseIds);
        $mems = $this->getMemberService()->searchMembers(
            $conditions,
            array(),
            0,
            PHP_INT_MAX
        );
        if (!empty($mems)) {
            foreach ($mems as $mem) {
                if ($mem['passedStatus'] == 'passed') {
                    ++$mempass;
                }
                if ($mem['attendedStatus'] == 'attended') {
                    ++$memjoin;
                }
            }
        }
        $category['mem'] = count($mems);
        $category['pass'] = $mempass;
        $category['join'] = $memjoin;
        $category['count']['handing'] = $handing;
        $category['count']['end'] = $end;

        return $category;
    }

    protected function getCategoryService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:CategoryService');
    }

    protected function getOfflineCourseService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:OfflineCourseService');
    }

    protected function getMemberService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:MemberService');
    }
}
