<?php

namespace CorporateTrainingBundle\Controller\Admin;

use AppBundle\Common\TreeToolkit;
use AppBundle\Common\ArrayToolkit;
use Symfony\Component\HttpFoundation\Request;
use AppBundle\Controller\Admin\OrgManageController as BaseController;

class OrgManageController extends BaseController
{
    public function indexAction(Request $request)
    {
        $orgs = $this->getOrgService()->searchOrgs(array(), array(), 0, PHP_INT_MAX);

        $treeOrgs = TreeToolkit::makeTree($orgs, 'seq');
        $userIds = ArrayToolkit::column($orgs, 'createdUserId');
        $createdUsers = $this->getUserService()->findUsersByIds($userIds);
        $syncSetting = $this->getSettingService()->get('sync_department_setting', array());

        return $this->render(
            'admin/org-manage/index.html.twig',
            array(
                'orgs' => $treeOrgs,
                'createdUsers' => $createdUsers,
                'syncSetting' => $syncSetting,
            )
        );
    }

    public function syncDepartmentAction(Request $request)
    {
        return $this->render('admin/org-manage/sync-department-modal.html.twig');
    }

    public function syncDepartmentFromAction(Request $request)
    {
        if ($request->getMethod() == 'POST') {
            $result = $this->getOrgSyncService()->syncFrom();

            if ($result['success']) {
                $result = array('success' => $result['success'], 'type' => 'success', 'message' => '同步成功');
            } else {
                if ($result['code'] == 2) {
                    $type = 'warning';
                    $message = '同步失败，请先开启同步开关';
                } elseif ($result['code'] == 3) {
                    $type = 'warning';
                    $message = '同步失败，请先删除系统中非钉钉下的组织机构';
                } else {
                    $type = 'danger';
                    $message = '同步失败';
                }
                $result = array('success' => $result['success'], 'type' => $type, 'message' => $message);
            }

            return $this->createJsonResponse($result);
        }
    }

    public function syncDepartmentToAction(Request $request)
    {
        if ($request->getMethod() == 'POST') {
            $result = $this->getOrgSyncService()->syncTo();

            if ($result['success']) {
                $result = array('success' => $result['success'], 'type' => 'success', 'message' => '同步成功');
            } else {
                if ($result['code'] == 2) {
                    $type = 'warning';
                    $message = '同步失败，请先开启同步开关';
                } elseif ($result['code'] == 3) {
                    $type = 'warning';
                    $message = '同步失败，请先清空钉钉下组织机构';
                } else {
                    $type = 'danger';
                    $message = '同步失败';
                }
                $result = array('success' => $result['success'], 'type' => $type, 'message' => $message);
            }

            return $this->createJsonResponse($result);
        }
    }

    protected function getSettingService()
    {
        return $this->createService('System:SettingService');
    }

    protected function getOrgSyncService()
    {
        return $this->createService('OrgSync:OrgSyncService');
    }
}
