<?php

namespace CorporateTrainingBundle\Controller\Admin;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\JsonResponse;
use AppBundle\Controller\Admin\BaseController;

class PostController extends BaseController
{
    public function indexAction(Request $request)
    {
        $postGroups = $this->getPostService()->searchPostGroups(array(), array('seq' => 'ASC'), 0, PHP_INT_MAX);
        $postGroups = $this->getPostService()->getPostStructureTree($postGroups);

        return $this->render('admin/post/index.html.twig', array(
            'postGroups' => $postGroups,
        ));
    }

    public function createAction(Request $request, $groupId)
    {
        if ($request->getMethod() == 'POST') {
            $post = $request->request->all();
            $post['groupId'] = $groupId;
            $user = $this->getCurrentUser();
            $post['createdUserId'] = $user['id'];
            $post = $this->getPostService()->createPost($post);

            return $this->redirect($this->generateUrl('admin_post_manage'));
        }

        $post = array('id' => 0, 'name' => '', 'groupId' => $groupId);
        $postGroup = $this->getPostService()->getPostGroup($groupId);

        return $this->render('admin/post/post-modal.html.twig', array(
            'post' => $post,
            'postGroup' => $postGroup,
        ));
    }

    public function editAction(Request $request, $id)
    {
        if ($request->getMethod() == 'POST') {
            $fields = $request->request->all();
            $post = $this->getPostService()->updatePost($id, $fields);

            return $this->redirect($this->generateUrl('admin_post_manage'));
        }

        $post = $this->getPostService()->getPost($id);
        $postGroup = $this->getPostService()->getPostGroup($post['groupId']);

        return $this->render('admin/post/post-modal.html.twig', array(
            'post' => $post,
            'postGroup' => $postGroup,
        ));
    }

    public function deleteAction(Request $request, $id)
    {
        $result = $this->getPostService()->checkPostCanDelete($id);

        if (!$result) {
            return $this->createJsonResponse(array('success' => $result, 'message' => '岗位已绑定用户，请先解除绑定，再进行删除操作！'));
        }

        $result = $this->getPostService()->deletePost($id);

        if ($result) {
            $result = array('success' => $result, 'message' => '删除岗位成功！');
        } else {
            $result = array('success' => $result, 'message' => '删除岗位失败，请重试！');
        }

        return $this->createJsonResponse($result);
    }

    public function sortAction(Request $request)
    {
        $ids = $request->request->get('ids');

        if (!empty($ids)) {
            $this->getPostService()->sortPosts($ids);
        }

        return $this->createJsonResponse(true);
    }

    public function matchAction(Request $request)
    {
        $data = array();
        $queryString = $request->query->get('q');

        $likeName = '%'.$queryString.'%';

        $posts = $this->getPostService()->searchPosts(array('likeName' => $likeName), array(), 0, 100);

        foreach ($posts as $post) {
            $data[] = array('id' => $post['id'], 'name' => $post['name']);
        }

        return new JsonResponse($data);
    }

    public function createGroupAction(Request $request)
    {
        if ($request->getMethod() == 'POST') {
            $postGroup = $request->request->all();
            $user = $this->getCurrentUser();
            $postGroup['createdUserId'] = $user['id'];
            $postGroup = $this->getPostService()->createPostGroup($postGroup);

            return $this->redirect($this->generateUrl('admin_post_manage'));
        }

        return $this->render('admin/post/post-group-modal.html.twig');
    }

    public function editGroupAction(Request $request, $id)
    {
        if ($request->getMethod() == 'POST') {
            $fields = $request->request->all();
            $postGroup = $this->getPostService()->updatePostGroup($id, $fields);

            return $this->redirect($this->generateUrl('admin_post_manage'));
        }

        $postGroup = $this->getPostService()->getPostGroup($id);

        return $this->render('admin/post/post-group-modal.html.twig', array(
            'postGroup' => $postGroup,
        ));
    }

    public function deleteGroupAction(Request $request, $id)
    {
        $result = $this->getPostService()->checkPostGroupCanDelete($id);

        if (!$result) {
            return $this->createJsonResponse(array('success' => false, 'message' => '不能删除含有岗位的分组！'));
        }

        $result = $this->getPostService()->deletePostGroup($id);

        if ($result) {
            $result = array('success' => $result, 'message' => '删除岗位分组成功！');
        } else {
            $result = array('success' => $result, 'message' => '删除岗位失败！');
        }

        return $this->createJsonResponse($result);
    }

    public function sortGroupAction(Request $request)
    {
        $ids = $request->request->get('ids');

        if (!empty($ids)) {
            $this->getPostService()->sortPostGroups($ids);
        }

        return $this->createJsonResponse(true);
    }

    public function checkNameAction(Request $request)
    {
        $name = $request->query->get('value');
        $exclude = $request->query->get('exclude');
        $available = $this->getPostService()->isPostNameAvailable($name, $exclude);

        if ($available) {
            $result = array(
                'success' => true,
                'message' => '',
            );
        } else {
            $result = array(
                'success' => false,
                'message' => '岗位已存在',
            );
        }

        return $this->createJsonResponse($result);
    }

    public function checkGroupNameAction(Request $request)
    {
        $name = $request->query->get('value');
        $exclude = $request->query->get('exclude');
        $available = $this->getPostService()->isPostGroupNameAvailable($name, $exclude);

        if ($available) {
            $result = array(
                'success' => true,
                'message' => '',
            );
        } else {
            $result = array(
                'success' => false,
                'message' => '岗位分组已存在',
            );
        }

        return $this->createJsonResponse($result);
    }

    public function checkCodeAction(Request $request)
    {
        $code = $request->query->get('value');
        $exclude = $request->query->get('exclude');
        $isAvailable = $this->getPostService()->isPostCodeAvailable($code, $exclude);

        if ($isAvailable) {
            $response = array('success' => true, 'message' => '');
        } else {
            $response = array('success' => false, 'message' => '编码已被占用,请换一个');
        }

        return $this->createJsonResponse($response);
    }

    protected function getPostService()
    {
        return $this->createService('CorporateTrainingBundle:Post:PostService');
    }
}
