<?php

namespace CorporateTrainingBundle\Controller\Admin;

use AppBundle\Common\Paginator;
use AppBundle\Common\ArrayToolkit;
use Symfony\Component\HttpFoundation\Request;
use AppBundle\Controller\Admin\BaseController;

class PostCourseController extends BaseController
{
    public function indexAction(Request $request, $postId)
    {
        $post = $this->getPostService()->getPost($postId);
        $count = $this->getPostCourseService()->countPostCourses(array('postId' => $postId));

        if ($count > 0) {
            $postCourses = $this->getPostCourseService()->searchPostCourses(
                array('postId' => $postId),
                array('seq' => 'ASC'),
                0,
                PHP_INT_MAX
            );

            $courseSets = $this->getCourseSetService()->findCourseSetsByIds(ArrayToolkit::column($postCourses,
                'courseSetId'));
            $categories = $this->getCategoryService()->findCategoriesByIds(ArrayToolkit::column($courseSets,
                'categoryId'));
        }

        return $this->render(
            'admin/post-course/post-course.html.twig',
            array(
                'post' => $post,
                'orgs' => empty($orgs) ? array() : $orgs,
                'courseSets' => empty($courseSets) ? array() : $courseSets,
                'categories' => empty($categories) ? array() : $categories,
                'postCourses' => empty($postCourses) ? array() : $postCourses,
            )
        );
    }

    public function chooseCoursesAction(Request $request, $postId)
    {
        $post = $this->getPostService()->getPost($postId);
        $postCourses = $this->getPostCourseService()->findPostCoursesByPostId($postId);
        $courseSetIds = ArrayToolkit::column($postCourses, 'courseSetId');
        $user = $this->getCurrentUser();

        $conditions = array(
            'excludeIds' => $courseSetIds,
            'likeOrgCode' => $user->getSelectOrgCode(),
            'status' => 'published',
            'parentId' => 0,
        );

        $courseSetNum = $this->getCourseSetService()->countCourseSets($conditions);

        if ($courseSetNum > 0) {
            $paginator = new Paginator(
                $request,
                $courseSetNum,
                10
            );

            $courseSets = $this->getCourseSetService()->searchCourseSets(
                $conditions,
                array('createdTime' => 'DESC'),
                $paginator->getOffsetCount(),
                $paginator->getPerPageCount()
            );

            $courseSets = $this->combineCoursesByCourseSets($courseSets);
            $orgs = $this->getOrgService()->findOrgsByIds(ArrayToolkit::column($courseSets, 'orgId'));
            $orgs = ArrayToolkit::index($orgs, 'id');
        }

        return $this->render('admin/post-course/assign-course-modal.html.twig', array(
            'post' => $post,
            'orgs' => empty($orgs) ? array() : $orgs,
            'courseSets' => empty($courseSets) ? array() : $courseSets,
            'paginator' => empty($paginator) ? array() : $paginator,
        ));
    }

    private function combineCoursesByCourseSets($courseSets)
    {
        $courseSets = ArrayToolkit::index($courseSets, 'id');
        $courses = $this->getCourseService()->findCoursesByCourseSetIds(array_keys($courseSets));

        if (!empty($courses)) {
            foreach ($courses as $course) {
                if ($course['status'] != 'published') {
                    continue;
                }

                if (empty($courseSets[$course['courseSetId']]['courses'])) {
                    $courseSets[$course['courseSetId']]['courses'] = array($course);
                } else {
                    $courseSets[$course['courseSetId']]['courses'][] = $course;
                }
            }
        }

        return $courseSets;
    }

    public function assignCoursesAction(Request $request, $postId)
    {
        if ($request->getMethod() == 'POST') {
            $courseIds = $request->request->get('courseIds');

            $this->getPostCourseService()->batchCreatePostCourses($postId, explode(',', $courseIds));

            return $this->createJsonResponse(array('success' => true));
        }
    }

    public function searchCoursesAction(Request $request, $postId)
    {
        $user = $this->getCurrentUser();
        $fields = $this->prepareConditions($request->request->all());
        $postCourses = $this->getPostCourseService()->findPostCoursesByPostId($postId);
        $conditions = array(
            'excludeIds' => ArrayToolkit::column($postCourses, 'courseSetId'),
            'parentId' => 0,
            'likeOrgCode' => $user->getSelectOrgCode(),
            'status' => 'published',
        );

        $conditions = array_merge($conditions, $fields);

        if (!$this->isSubOrgCode($conditions['likeOrgCode'], $user['orgCodes'])) {
            return $this->createMessageResponse('error', '没有权限查看当前组织机构下信息');
        }

        $courseSetNum = $this->getCourseSetService()->countCourseSets($conditions);

        if ($courseSetNum > 0) {
            $paginator = new Paginator(
                $request,
                $courseSetNum,
                10
            );

            $courseSets = $this->getCourseSetService()->searchCourseSets(
                $conditions,
                array('createdTime' => 'DESC'),
                $paginator->getOffsetCount(),
                $paginator->getPerPageCount()
            );

            $courseSets = $this->combineCoursesByCourseSets($courseSets);
            $orgs = $this->getOrgService()->findOrgsByIds(ArrayToolkit::column($courseSets, 'orgId'));
            $orgs = ArrayToolkit::index($orgs, 'id');
        }

        return $this->render('admin/post-course/course-list.html.twig', array(
            'type' => 'ajax_pagination',
            'orgs' => empty($orgs) ? array() : $orgs,
            'courseSets' => empty($courseSets) ? array() : $courseSets,
            'paginator' => empty($paginator) ? array() : $paginator,
        ));
    }

    public function deleteAction(Request $request, $id)
    {
        if ($request->getMethod() == 'POST') {
            $result = $this->getPostCourseService()->deletePostCourse($id);

            return $this->createJsonResponse(array('success' => $result));
        }
    }

    public function sortAction(Request $request)
    {
        $ids = $request->request->get('ids');

        if (!empty($ids)) {
            $this->getPostCourseService()->sortPostCourses($ids);
        }

        return $this->createJsonResponse(array('success' => true, 'message' => '更新课程顺序成功'));
    }

    private function prepareConditions($conditions)
    {
        if (isset($conditions['keywordType']) && !empty($conditions['keyword'])) {
            if ($conditions['keywordType'] == 'ids') {
                $conditions[$conditions['keywordType']] = array($conditions['keyword']);
            } else {
                $conditions[$conditions['keywordType']] = $conditions['keyword'];
            }
            unset($conditions['keywordType']);
            unset($conditions['keyword']);
        }

        if (isset($conditions['orgCode'])) {
            $conditions['likeOrgCode'] = $conditions['orgCode'];
            unset($conditions['orgCode']);
        }

        return $conditions;
    }

    private function isSubOrgCode($childOrgCode, $parentOrgCodes)
    {
        $isSubOrgCode = false;
        foreach ($parentOrgCodes as $parentOrgCode) {
            if (substr($childOrgCode, 0, strlen($parentOrgCode)) == $parentOrgCode) {
                $isSubOrgCode = true;
                break;
            }
        }

        return $isSubOrgCode;
    }

    protected function getPostService()
    {
        return $this->createService('CorporateTrainingBundle:Post:PostService');
    }

    protected function getPostCourseService()
    {
        return $this->createService('CorporateTrainingBundle:PostCourse:PostCourseService');
    }

    protected function getCourseService()
    {
        return $this->createService('Course:CourseService');
    }

    protected function getCourseSetService()
    {
        return $this->createService('Course:CourseSetService');
    }

    protected function getCategoryService()
    {
        return $this->createService('Taxonomy:CategoryService');
    }

    protected function getOrgService()
    {
        return $this->createService('Org:OrgService');
    }
}
