<?php

namespace CorporateTrainingBundle\Controller\Admin;

use AppBundle\Common\ArrayToolkit;
use Symfony\Component\HttpFoundation\Request;
use AppBundle\Controller\Admin\BaseController;
use Biz\Org\Service\OrgService;
use Biz\Course\Service\CourseSetService;
use CorporateTrainingBundle\Biz\TrainingAssignment\Service\TrainingAssignmentItemService;
use CorporateTrainingBundle\Biz\TrainingAssignment\Service\TrainingAssignmentMemberService;
use CorporateTrainingBundle\Biz\TrainingAssignment\Service\TrainingAssignmentService;
use AppBundle\Common\Paginator;

class TrainingAssignmentController extends BaseController
{
    public function indexAction(Request $request)
    {
        $conditions = $request->query->all();
        $conditions = $this->prepareManageListSearchConditions($conditions);

        $assignmentCount = $this->getTrainingAssignmentService()->countTrainingAssignments($conditions);
        $paginator = new Paginator(
            $request,
            $assignmentCount,
            20
        );

        $assignments = $this->getTrainingAssignmentService()->searchTrainingAssignments(
            $conditions,
            array('createdTime' => 'DESC'),
            $paginator->getOffsetCount(),
            $paginator->getPerPageCount()
        );

        foreach ($assignments as &$assignment) {
            $assignment['memberNum'] = $this->getAssignmentMemberNum($assignment['id']);
            $assignment['itemNum'] = $this->getAssignmentCourseNum($assignment['id']);
        }

        return $this->render('admin/training-assignment/index.html.twig',
        array(
                'assignments' => $assignments,
                'paginator' => $paginator,
            )
        );
    }

    protected function prepareManageListSearchConditions($conditions)
    {
        if ($this->setting('magic.enable_org')) {
            if (!isset($conditions['orgCode'])) {
                $conditions['likeOrgCode'] = $this->getCurrentUser()->getSelectOrgCode();
            } else {
                $conditions['likeOrgCode'] = $conditions['orgCode'];
                unset($conditions['orgCode']);
            }
        } else {
            if (isset($conditions['orgCode'])) {
                unset($conditions['orgCode']);
            }
        }

        if (!empty($conditions['status']) && 'all' === $conditions['status']) {
            unset($conditions['status']);
        }

        return $conditions;
    }

    private function getAssignmentMemberNum($assignmentId)
    {
        $assignmentMembers = $this->getTrainingAssignmentMemberService()->searchTrainingAssignmentMembers(
            array('assignmentId' => $assignmentId),
            array(),
            0,
            PHP_INT_MAX
        );

        if (empty($assignmentMembers)) {
            return 0;
        }

        $userIds = ArrayToolkit::column($assignmentMembers, 'userId');

        return count($userIds);
    }

    private function getAssignmentCourseNum($assignmentId)
    {
        $courseItems = $this->getTrainingAssignmentItemService()->findAssignmentItemsByAssignmentIdAndTargetType($assignmentId, 'course');

        if (empty($courseItems)) {
            return 0;
        }

        $courseIds = ArrayToolkit::column($courseItems, 'targetId');

        return count($courseIds);
    }

    /**
     * @return TrainingAssignmentService
     */
    protected function getTrainingAssignmentService()
    {
        return $this->createService('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentService');
    }

    /**
     * @return TrainingAssignmentItemService
     */
    protected function getTrainingAssignmentItemService()
    {
        return $this->createService('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentItemService');
    }

    /**
     * @return TrainingAssignmentMemberService
     */
    protected function getTrainingAssignmentMemberService()
    {
        return $this->createService('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentMemberService');
    }

    /**
     * @return OrgService
     */
    protected function getOrgService()
    {
        return $this->createService('Org:OrgService');
    }

    /**
     * @return CourseSetService
     */
    protected function getCourseSetService()
    {
        return $this->createService('Course:CourseSetService');
    }
}
