<?php

namespace CorporateTrainingBundle\Controller\Admin;

use CorporateTrainingBundle\Common\OrgToolkit;
use Symfony\Component\HttpFoundation\Request;
use AppBundle\Common\Paginator;
use AppBundle\Common\ArrayToolkit;
use AppBundle\Controller\Admin\UserController as BaseController;

class UserController extends BaseController
{
    public function indexAction(Request $request)
    {
        $fields = $request->query->all();

        $conditions = array(
            'roles' => '',
            'keywordType' => '',
            'keyword' => '',
            'keywordUserType' => '',
            'noType' => 'system',
        );

        $conditions = array_merge($conditions, $fields);
        $conditions = $this->fillOrgCode($conditions);

        $userCount = $this->getUserService()->countUsers($conditions);
        $paginator = new Paginator(
            $this->get('request'),
            $userCount,
            20
        );

        $users = $this->getUserService()->searchUsers(
            $conditions,
            array('createdTime' => 'DESC'),
            $paginator->getOffsetCount(),
            $paginator->getPerPageCount()
        );

        $app = $this->getAppService()->findInstallApp('UserImporter');

        $showUserExport = false;

        if (!empty($app) && array_key_exists('version', $app)) {
            $showUserExport = version_compare($app['version'], '1.0.2', '>=');
        }

        $userIds = ArrayToolkit::column($users, 'id');
        $profiles = $this->getUserService()->findUserProfilesByIds($userIds);

        $allRoles = $this->getAllRoles();
        if (isset($allRoles['ROLE_ADMIN'])) {
            unset($allRoles['ROLE_ADMIN']);
        }
        $userCountInfo = $this->getUserService()->countUsersByLockedStatus();
        $conditions['locked'] = 1;
        $userLockedCount = $this->getUserService()->countUsers($conditions);

        return $this->render('admin/user/index.html.twig', array(
            'users' => $users,
            'allRoles' => $allRoles,
            'userCount' => $userCount,
            'userLockedCount' => $userLockedCount,
            'userCountInfo' => $userCountInfo,
            'paginator' => $paginator,
            'profiles' => $profiles,
            'showUserExport' => $showUserExport,
        ));
    }

    public function createAction(Request $request)
    {
        if ($request->getMethod() == 'POST') {
            $formData = $request->request->all();
            $formData['type'] = 'import';
            $registration = $this->getRegisterData($formData, $request->getClientIp());
            try {
                $user = $this->getAuthService()->register($registration);
            } catch (\Exception $e) {
                return $this->createJsonResponse(array('error' => $e->getMessage()));
            }
            $this->get('session')->set('registed_email', $user['email']);

            if (isset($formData['roles'])) {
                $roles = $formData['roles'];
                $roles[] = 'ROLE_USER';
                if (in_array('ROLE_TRAINING_ADMIN', $roles) && !in_array('ROLE_TEACHER', $roles)) {
                    $roles[] = 'ROLE_TEACHER';
                }

                $this->getUserService()->changeUserRoles($user['id'], $roles);
            }

            if (!empty($formData['postId'])) {
                $this->getUserService()->changeUserPost($user['id'], $formData['postId']);
            }

            if (!empty($formData['truename'])) {
                $this->getUserService()->updateUserProfile($user['id'], array('truename' => $formData['truename']));
            }

            $this->getLogService()->info('user', 'add', "管理员添加新用户 {$user['nickname']} ({$user['id']})");

            return $this->redirect($this->generateUrl('admin_user'));
        }
        $user = $this->getCurrentUser();

        return $this->render('admin/user/create-modal.html.twig', array('user' => $user));
    }

    public function editAction(Request $request, $id)
    {
        $user = $this->getUserService()->getUser($id);

        $profile = $this->getUserService()->getUserProfile($user['id']);
        $profile['title'] = $user['title'];

        if ($request->getMethod() == 'POST') {
            $profile = $request->request->all();

            if (!((strlen($user['verifiedMobile']) > 0) && isset($profile['mobile']))) {
                $profile = $this->getUserService()->updateUserProfile($user['id'], $profile);
                $this->getLogService()->info('user', 'edit', "管理员编辑用户资料 {$user['nickname']} (#{$user['id']})", $profile);
            } else {
                $this->setFlashMessage('danger', '用户已绑定的手机不能修改。');
            }

            return $this->render('admin/user/user-table-tr.html.twig', array(
                'user' => $user,
                'profile' => $this->getUserService()->getUserProfile($id),
            ));
        }

        $fields = $this->getFields();

        return $this->render('admin/user/edit-modal.html.twig', array(
            'user' => $user,
            'profile' => $profile,
            'fields' => $fields,
        ));
    }

    public function showAction(Request $request, $id)
    {
        $user = $this->getUserService()->getUser($id);
        $profile = $this->getUserService()->getUserProfile($id);
        $profile['title'] = $user['title'];
        $fields = $this->getFields();
        $orgs = $this->getOrgService()->findOrgsByIds($user['orgIds']);
        $orgNames = OrgToolkit::buildOrgsNames($user['orgIds'], $orgs);

        return $this->render('admin/user/show-modal.html.twig', array(
            'user' => $user,
            'profile' => $profile,
            'fields' => $fields,
            'orgNames' => $orgNames,
        ));
    }

    public function rolesAction(Request $request, $id)
    {
        $user = $this->getUserService()->getUser($id);
        $currentUser = $this->getCurrentUser();

        if ($request->getMethod() == 'POST') {
            $roles = $request->request->get('roles');
            if (in_array('ROLE_TRAINING_ADMIN', $roles) && !in_array('ROLE_TEACHER', $roles)) {
                $roles[] = 'ROLE_TEACHER';
            }

            $this->getUserService()->changeUserRoles($user['id'], $roles);

            if (!empty($roles)) {
                $roleSet = $this->getRoleService()->searchRoles(array(), 'created', 0, 9999);
                $rolesByIndexCode = ArrayToolkit::index($roleSet, 'code');
                $roleNames = $this->getRoleNames($roles, $rolesByIndexCode);

                $message = array(
                    'userId' => $currentUser['id'],
                    'userName' => $currentUser['nickname'],
                    'role' => implode(',', $roleNames),
                );

                $this->getNotifiactionService()->notify($user['id'], 'role', $message);
            }
            $user = $this->getUserService()->getUser($id);

            return $this->render('admin/user/user-table-tr.html.twig', array(
                'user' => $user,
                'profile' => $this->getUserService()->getUserProfile($id),
            ));
        }

        return $this->render('admin/user/roles-modal.html.twig', array(
            'user' => $user,
        ));
    }

    public function changePostAction(Request $request, $id)
    {
        $user = $this->getUserService()->getUser($id);

        if ($request->getMethod() == 'POST') {
            $postId = $request->request->get('newPostId');

            $this->getUserService()->changeUserPost($user['id'], $postId);

            return $this->render('admin/user/user-table-tr.html.twig', array(
                'user' => $this->getUserService()->getUser($id),
                'profile' => $this->getUserService()->getUserProfile($id),
            ));
        }

        return $this->render('admin/user/change-post-modal.html.twig', array(
            'user' => $user,
        ));
    }

    public function changeNicknameAction(Request $request, $id)
    {
        $user = $this->getUserService()->getUser($id);

        if ($request->getMethod() == 'POST') {
            $nickname = $request->request->get('newNickname');

            $this->getUserService()->changeNickname($user['id'], $nickname);

            return $this->render('admin/user/user-table-tr.html.twig', array(
                'user' => $this->getUserService()->getUser($id),
                'profile' => $this->getUserService()->getUserProfile($id),
            ));
        }

        return $this->render('admin/user/change-nickname-modal.html.twig', array(
            'user' => $user,
        ));
    }

    public function batchUpdatePostAction(Request $request)
    {
        if ($request->getMethod() == 'POST') {
            $ids = $request->request->get('ids');
            $postId = $request->request->get('batchNewPostId');
            $this->getUserService()->batchUpdatePost($ids, $postId);

            return $this->createJsonResponse(true);
        }

        return $this->render('admin/user/batch-update-post-modal.html.twig');
    }

    public function batchUpdateOrgsAction(Request $request)
    {
        if ($request->getMethod() == 'POST') {
            $ids = $request->request->get('ids');
            $orgCodes = $request->request->get('orgCodes');
            $this->getUserService()->batchUpdateOrgs($ids, $orgCodes);

            return $this->createJsonResponse(true);
        }

        return $this->render('org/batch-update-orgs-modal.html.twig');
    }

    public function lockAction($id)
    {
        $this->getUserService()->lockUser($id);
        $this->kickUserLogout($id);

        return $this->render('admin/user/user-table-tr.html.twig', array(
            'user' => $this->getUserService()->getUser($id),
            'profile' => $this->getUserService()->getUserProfile($id),
        ));
    }

    public function unlockAction($id)
    {
        try {
            $this->getUserService()->unlockUser($id);

            return $this->render('admin/user/user-table-tr.html.twig', array(
                'user' => $this->getUserService()->getUser($id),
                'profile' => $this->getUserService()->getUserProfile($id),
            ));
        } catch (\Exception $e) {
            return $this->createJsonResponse(array('error' => $e->getMessage()));
        }
    }

    public function orgUpdateAction(Request $request, $id)
    {
        $user = $this->getUserService()->getUser($id);

        if ($request->isMethod('POST')) {
            $orgCodes = $request->request->get('orgCodes', $user['orgCodes']);
            $this->getUserService()->changeUserOrgs($user['id'], $orgCodes);
        }

        $orgs = $this->getOrgService()->findOrgsByIds($user['orgIds']);

        return $this->render('admin/user/update-org-modal.html.twig', array(
            'user' => $user,
            'orgCodes' => ArrayToolkit::column($orgs, 'orgCode'),
        ));
    }

    protected function getRegisterData($formData, $clientIp)
    {
        $userData = parent::getRegisterData($formData, $clientIp);
        if (isset($formData['postId'])) {
            $userData['postId'] = $formData['postId'];
        }

        if (isset($formData['orgCodes'])) {
            $formData['orgCodes'] = implode('|', $formData['orgCodes']);
            $userData['orgCodes'] = $formData['orgCodes'];
        }

        return $userData;
    }

    protected function getUserService()
    {
        return $this->createService('CorporateTrainingBundle:User:UserService');
    }

    protected function getUserOrgService()
    {
        return $this->createService('CorporateTrainingBundle:User:UserOrgService');
    }

    protected function getPostService()
    {
        return $this->createService('CorporateTrainingBundle:Post:PostService');
    }
}
