<?php

namespace CorporateTrainingBundle\Controller\Admin;

use AppBundle\Common\ArrayToolkit;
use Symfony\Component\HttpFoundation\Request;
use AppBundle\Component\OAuthClient\OAuthClientFactory;
use AppBundle\Controller\Admin\UserSettingController as BaseController;

class UserSettingController extends BaseController
{
    public function loginConnectAction(Request $request)
    {
        $clients = OAuthClientFactory::clients();
        $default = $this->getDefaultLoginConnect($clients);
        $loginConnect = $this->getSettingService()->get('login_bind', array());
        $loginConnect = array_merge($default, $loginConnect);
        $syncSetting = $this->getSettingService()->get('sync_department_setting', array());

        if ($request->isMethod('POST')) {
            $loginConnect = $request->request->all();

            if (isset($loginConnect['dingtalksync_key']) && isset($loginConnect['dingtalksync_secret'])) {
                $setting = array(
                    'enable' => ($loginConnect['dingtalkMode'] == 'sync') ? 1 : 0,
                    'type' => 'dingtalk',
                    'key' => $loginConnect['dingtalksync_key'],
                    'secret' => $loginConnect['dingtalksync_secret'],
                    'syncTime' => (empty($syncSetting['syncTime'])) ? 0 : $syncSetting['syncTime'],
                    'times' => (empty($syncSetting['times'])) ? 0 : $syncSetting['times'],
                );

                $this->getSettingService()->set('sync_department_setting', $setting);
                unset($loginConnect['dingtalksync_secret']);
                unset($loginConnect['dingtalksync_key']);
                if ($loginConnect['dingtalkMode'] == 'sync') {
                    $loginConnect['dingtalksync_enabled'] = 1;
                }
            }

            if ($loginConnect['dingtalkMode'] != 'close' && isset($loginConnect['dingtalkweb_key']) && isset($loginConnect['dingtalkweb_secret'])) {
                $loginConnect['dingtalkweb_enabled'] = 1;
            }

            $loginConnect = ArrayToolkit::trim($loginConnect);
            $loginConnect = $this->decideEnabledLoginConnect($loginConnect);

            $this->getSettingService()->set('login_bind', $loginConnect);
            $this->getLogService()->info('system', 'update_settings', '更新登录设置', $loginConnect);
            $this->updateWeixinMpFile($loginConnect['weixinmob_mp_secret']);
        }

        return $this->render('admin/system/login-connect.html.twig', array(
            'loginConnect' => $loginConnect,
            'clients' => $clients,
            'syncSetting' => $syncSetting,
        ));
    }

    private function getDefaultLoginConnect($clients)
    {
        $default = array(
            'login_limit' => 0,
            'enabled' => 0,
            'verify_code' => '',
            'captcha_enabled' => 0,
            'temporary_lock_enabled' => 0,
            'temporary_lock_allowed_times' => 5,
            'ip_temporary_lock_allowed_times' => 20,
            'temporary_lock_minutes' => 20,
        );

        foreach ($clients as $type => $client) {
            $default["{$type}_enabled"] = 0;
            $default["{$type}_key"] = '';
            $default["{$type}_secret"] = '';
            $default["{$type}_set_fill_account"] = 0;
            if ($type == 'weixinmob') {
                $default['weixinmob_mp_secret'] = '';
            }
        }

        return $default;
    }

    private function decideEnabledLoginConnect($loginConnect)
    {
        if ($loginConnect['enabled'] == 0) {
            $loginConnect['weibo_enabled'] = 0;
            $loginConnect['qq_enabled'] = 0;
            $loginConnect['renren_enabled'] = 0;
            $loginConnect['weixinweb_enabled'] = 0;
            $loginConnect['weixinmob_enabled'] = 0;
            $loginConnect['dingtalkweb_enabled'] = 0;
            $loginConnect['dingtalksync_enabled'] = 0;
        }
        //新增第三方登陆方式，加入下列列表计算，以便判断是否关闭第三方登陆功能
        $loginConnects = ArrayToolkit::parts($loginConnect, array('weibo_enabled', 'qq_enabled', 'renren_enabled', 'weixinweb_enabled', 'weixinmob_enabled', 'dingtalkweb_enabled', 'dingtalksync_enabled'));
        $sum = 0;
        foreach ($loginConnects as $value) {
            $sum += $value;
        }

        if ($sum < 1) {
            if ($loginConnect['enabled'] == 1) {
                $this->setFlashMessage('danger', '请至少开启一种您需要的第三方登录方式！');
            }
            if ($loginConnect['enabled'] == 0) {
                $this->setFlashMessage('success', '登录设置已保存！');
            }
            $loginConnect['enabled'] = 0;
        } else {
            $loginConnect['enabled'] = 1;
            $this->setFlashMessage('success', '登录设置已保存！');
        }

        if (isset($loginConnect['dingtalkweb_enabled'])) {
            $loginConnect['dingtalkmob_enabled'] = $loginConnect['dingtalkweb_enabled'];
        }

        if (isset($loginConnect['dingtalkweb_key'])) {
            $loginConnect['dingtalkmob_key'] = $loginConnect['dingtalkweb_key'];
        }

        if (isset($loginConnect['dingtalkweb_secret'])) {
            $loginConnect['dingtalkmob_secret'] = $loginConnect['dingtalkweb_secret'];
        }

        return $loginConnect;
    }
}
