<?php

namespace CorporateTrainingBundle\Controller\Course;

use AppBundle\Common\ArrayToolkit;
use Biz\Org\Service\OrgService;
use Symfony\Component\HttpFoundation\Request;
use AppBundle\Controller\Course\CourseSetManageController as BaseCourseSetManageController;

class CourseSetManageController extends BaseCourseSetManageController
{
    public function createAction(Request $request)
    {
        if ($request->isMethod('POST')) {
            $data = $request->request->all();

            if (!isset($data['type'])) {
                throw $this->createNotFoundException('未设置课程类型');
            }

            $data = $this->buildOrgId($data);
            $courseSet = $this->getCourseSetService()->createCourseSet($data);

            return $this->redirect(
                $this->generateUrl(
                    'course_set_manage_base',
                    array(
                        'id' => $courseSet['id'],
                    )
                )
            );
        }

        if (!$this->getCourseSetService()->hasCourseSetManageRole()) {
            $this->createAccessDeniedException('Unauthorized');
        }
        $user = $this->getUser();
        $userProfile = $this->getUserService()->getUserProfile($user->getId());
        $user = $this->getUserService()->getUser($user->getId());

        return $this->render(
            'courseset-manage/create.html.twig',
            array(
                'user' => $user,
                'userProfile' => $userProfile,
            )
        );
    }

    public function baseAction(Request $request, $id)
    {
        $courseSet = $this->getCourseSetService()->tryManageCourseSet($id);
        $course = $this->getCourseService()->getDefaultCourseByCourseSetId($courseSet['id']);

        if ($request->isMethod('POST')) {
            $data = $request->request->all();
            $data = $this->buildOrgId($data);
            $this->getCourseSetService()->updateCourseSet($id, $data);

            if ($courseSet['type'] == 'live') {
                $this->getCourseService()->updateCourse($course['id'], array(
                    'title' => $course['title'],
                    'courseSetId' => $course['courseSetId'],
                    'maxStudentNum' => $data['maxStudentNum'],
                ));
            }

            $this->setFlashMessage('success', '课程基本信息已保存！');

            return $this->redirect($this->generateUrl('course_set_manage_base', array('id' => $id)));
        }

        if ($courseSet['locked']) {
            return $this->redirectToRoute(
                'course_set_manage_sync',
                array(
                    'id' => $id,
                    'sideNav' => 'base',
                )
            );
        }

        $tags = $this->getTagService()->findTagsByOwner(array(
            'ownerType' => 'course-set',
            'ownerId' => $id,
        ));

        return $this->render(
            'courseset-manage/base.html.twig',
            array(
                'courseSet' => $courseSet,
                'course' => $course,
                'tags' => ArrayToolkit::column($tags, 'name'),
            )
        );
    }

    protected function buildOrgId($fields)
    {
        if (!empty($fields['orgCode']) && empty($fields['orgId'])) {
            $org = $this->getOrgService()->getOrgByOrgCode($fields['orgCode']);
            $fields['orgId'] = empty($org) ? 1 : $org['id'];
        }

        return $fields;
    }

    /**
     * @return OrgService
     */
    protected function getOrgService()
    {
        return $this->createService('Org:OrgService');
    }
}
