<?php

namespace CorporateTrainingBundle\Controller\DataReport;

use AppBundle\Common\ArrayToolkit;
use AppBundle\Common\Paginator;
use AppBundle\Controller\Admin\BaseController;
use Biz\Course\Service\CourseService;
use Biz\Course\Service\MemberService;
use Biz\Task\Service\TaskResultService;
use Biz\Task\Service\TaskService;
use CorporateTrainingBundle\Biz\DataStatistics\Service\UserDailyLearnRecordService;
use CorporateTrainingBundle\Biz\DataStatistics\Service\UserLearnReportService;
use CorporateTrainingBundle\Biz\OfflineCourse\Service\CategoryService;
use CorporateTrainingBundle\Biz\OfflineCourse\Service\OfflineCourseService;
use CorporateTrainingBundle\Biz\Org\Service\OrgService;
use CorporateTrainingBundle\Biz\Post\Service\PostService;
use CorporateTrainingBundle\Biz\PostCourse\Service\PostCourseService;
use CorporateTrainingBundle\Biz\User\Service\UserOrgService;
use CorporateTrainingBundle\Biz\User\Service\UserService;
use CorporateTrainingBundle\Common\DateToolkit;
use CorporateTrainingBundle\Common\OrgToolkit;
use Symfony\Component\HttpFoundation\Request;

class DataReportController extends BaseController
{
    public function departmentLearnDataStatisticAction(Request $request)
    {
        if (!$this->getUserLearnRecordService()->hasManageUserLearnReportRole()) {
            return $this->createMessageResponse('error', '没有查看数据统计的权限');
        }

        $fields = $request->query->all();
        list($startDateTime, $endDateTime) = DateToolkit::generateStartDateAndEndDate('year', 0, 'date');

        $fields['startDateTime'] = $startDateTime;
        $fields['endDateTime'] = $endDateTime;
        $orgIds = $this->prepareOrgIds($fields);

        $totalDataConditions = $this->prepareSearchConditions($fields);
        $totalData = $this->getUserDailyLearnRecordService()->statisticsFinishedCourseNumAndLearnTime($totalDataConditions);

        list($startDateTime, $endDateTime) = DateToolkit::generateStartDateAndEndDate('week', 0, 'date');
        $fields['startDateTime'] = $startDateTime;
        $fields['endDateTime'] = $endDateTime;
        $currentDataConditions = $this->prepareSearchConditions($fields);
        $currentData = $this->getUserDailyLearnRecordService()->statisticsFinishedCourseNumAndLearnTime(
            $currentDataConditions
        );

        $userIds = $this->findUserIdsByOrgIds($orgIds);
        $userCount = $this->getUserService()->countUsers(array(
            'userIds' => empty($userIds) ? array(-1) : $userIds,
            'noType' => 'system',
            'locked' => 0,
        ));

        return $this->render(
            'data-report/department-learn-data.html.twig',
            array(
                'userCount' => $userCount,
                'totalData' => $totalData,
                'currentData' => $currentData,
                'orgIds' => implode(',', $orgIds),
            )
        );
    }

    public function ajaxGetLearnTimeStatisticAction(Request $request)
    {
        if (!$this->getUserLearnRecordService()->hasManageUserLearnReportRole()) {
            return $this->createMessageResponse('error', '没有查看数据统计的权限');
        }

        $cursor = $request->query->get('cursor', 0);
        list($startDate, $endDate) = DateToolkit::generateStartDateAndEndDate('month', $cursor, 'date');

        $conditions = array(
            'startDateTime' => $startDate,
            'endDateTime' => $endDate,
            'orgIds' => $request->query->get('orgIds', ''),
        );

        $conditions = $this->prepareSearchConditions($conditions);
        $record = $this->getUserDailyLearnRecordService()->statisticsTotalLearnTimeGroupByDate($conditions);
        $record = ArrayToolkit::index($record, 'date');
        $record = $this->buildAjaxStatisticData($cursor, $record, 'totalLearnTime');

        foreach ($record['data'] as &$datum) {
            $datum = DateToolkit::convertSecondToHour($datum);
        }

        $chartData = array(
            'xAxis' => array('data' => $record['date']),
            'series' => array(
                'name' => '学习时长',
                'type' => 'line',
                'stack' => '总量',
                'symbolSize' => 6,
                'data' => $record['data'],
            ),
            'yAxis' => array(
                'minInterval' => 0,
            ),
        );

        return $this->createJsonResponse($chartData);
    }

    public function ajaxGetLearnNumStatisticAction(Request $request)
    {
        if (!$this->getUserLearnRecordService()->hasManageUserLearnReportRole()) {
            return $this->createMessageResponse('error', '没有查看数据统计的权限');
        }

        $cursor = $request->query->get('cursor', 0);
        list($startDate, $endDate) = DateToolkit::generateStartDateAndEndDate('month', $cursor, 'date');

        $conditions = array(
            'startDateTime' => $startDate,
            'endDateTime' => $endDate,
            'orgIds' => $request->query->get('orgIds', ''),
        );

        $conditions = $this->prepareSearchConditions($conditions);
        $record = $this->getUserDailyLearnRecordService()->statisticsLearnUsersNumGroupByDate($conditions);

        $record = ArrayToolkit::index($record, 'date');
        $record = $this->buildAjaxStatisticData($cursor, $record, 'learnedUserNum');

        $chartData = array(
            'xAxis' => array('data' => $record['date']),
            'series' => array(
                'name' => '学习人数',
                'type' => 'line',
                'stack' => '总量',
                'symbolSize' => 6,
                'data' => $record['data'],
            ),
            'yAxis' => array(
                'minInterval' => 1,
            ),
        );

        return $this->createJsonResponse($chartData);
    }

    protected function buildAjaxStatisticData($cursor, $record, $dataFields)
    {
        list($startDate, $endDate) = DateToolkit::generateStartDateAndEndDate('month', $cursor, 'timestamp');
        $buildData = array();
        $totalDays = ($endDate - $startDate) / 86400 + 1;
        for ($i = 0; $i < $totalDays; ++$i) {
            $date = $startDate + 86400 * $i;
            $buildData['date'][$i] = date('Y-m-d', $date);
            $buildData['data'][$i] = empty($record[$date]) ? 0 : $record[$date][$dataFields];
        }
        $buildData['date'] = array_values($buildData['date']);
        $buildData['data'] = array_values($buildData['data']);

        return $buildData;
    }

    public function userLearnDataStatisticAction(Request $request)
    {
        if (!$this->getUserLearnRecordService()->hasManageUserLearnReportRole()) {
            return $this->createMessageResponse('error', '没有查看数据统计的权限');
        }

        $fields = $request->query->all();

        $orgIds = $this->prepareOrgIds($fields);

        $conditions = array(
            'noType' => 'system',
            'locked' => 0,
        );

        if (isset($fields['orgIds']) && empty($fields['orgIds'])) {
            $conditions['userIds'] = array(-1);
        } else {
            $userIds = $this->findUserIdsByOrgIds($orgIds);
            $conditions['userIds'] = empty($userIds) ? array(-1) : $userIds;
        }

        if (isset($fields['keywordType']) && !empty($fields['keyword'])) {
            $conditions[$fields['keywordType']] = $fields['keyword'];
        }

        if (isset($fields['postId']) && !empty($fields['postId'])) {
            $conditions['postId'] = $fields['postId'];
        }

        $count = $this->getUserService()->countUsers($conditions);

        $paginator = new Paginator(
            $this->get('request'),
            $count,
            20
        );

        $users = $this->getUserService()->searchUsers(
            $conditions,
            array('id' => 'DESC'),
            $paginator->getOffsetCount(),
            $paginator->getPerPageCount()
        );

        $userLearns = $this->getUserLearnRecordService()->searchUserLearnReports(
            array('userIds' => !empty($users) ? ArrayToolkit::column($users, 'id') : array(-1)),
            array('createdTime' => 'DESC', 'id' => 'DESC'),
            0,
            PHP_INT_MAX
        );

        $userLearns = ArrayToolkit::index($userLearns, 'userId');

        $posts = $this->getPostService()->findPostsByIds(ArrayToolkit::column($users, 'postId'));
        $userProfiles = $this->getUserService()->findUserProfilesByIds(!empty($users) ? ArrayToolkit::column($users, 'id') : array(-1));

        return $this->render(
            'data-report/user-learn-data.html.twig',
            array(
                'paginator' => $paginator,
                'userLearns' => $userLearns,
                'count' => $count,
                'orgIds' => implode(',', $orgIds),
                'posts' => ArrayToolkit::index($posts, 'id'),
                'users' => ArrayToolkit::index($users, 'id'),
                'userProfiles' => ArrayToolkit::index($userProfiles, 'id'),
            )
        );
    }

    public function courseLearnDataStatisticAction(Request $request)
    {
        if (!$this->getUserLearnRecordService()->hasManageUserLearnReportRole()) {
            return $this->createMessageResponse('error', '没有查看数据统计的权限');
        }

        $fields = $request->query->all();
        if (empty($fields['startDateTime']) && empty($fields['endDateTime'])) {
            list($startDateTime, $endDateTime) = DateToolkit::generateStartDateAndEndDate('month');
            $fields['startDateTime'] = $startDateTime;
            $fields['endDateTime'] = $endDateTime;
        }
        $orgIds = $this->prepareOrgIds($fields);
        $conditions = $this->prepareSearchConditions($fields);

        $userSearchConditions = $conditions;
        $userSearchConditions['locked'] = 0;
        $userSearchConditions['noType'] = 'system';

        $count = $this->getUserService()->countUsers($userSearchConditions);
        $paginator = new Paginator(
            $request,
            $count,
            20
        );

        $users = $this->getUserService()->searchUsers(
            $userSearchConditions,
            array(),
            $paginator->getOffsetCount(),
            $paginator->getPerPageCount()
        );

        $conditions['userIds'] = ArrayToolkit::column($users, 'id');
        $records = $this->getUserDailyLearnRecordService()->statisticsLearnRecordGroupByUserId($conditions);
        $records = ArrayToolkit::index($records, 'userId');

        $posts = $this->getPostService()->findPostsByIds(ArrayToolkit::column($users, 'postId'));
        $userProfiles = $this->getUserService()->findUserProfilesByIds(ArrayToolkit::column($users, 'id'));

        return $this->render(
            'data-report/course-learn-data.html.twig',
            array(
                'users' => $users,
                'records' => $records,
                'userNum' => $count,
                'paginator' => $paginator,
                'orgIds' => implode(',', $orgIds),
                'posts' => ArrayToolkit::index($posts, 'id'),
                'userProfiles' => ArrayToolkit::index($userProfiles, 'id'),
                'startDateTime' => $fields['startDateTime'],
                'endDateTime' => $fields['endDateTime'],
            )
        );
    }

    public function courseLearnDataStatisticDepartmentAction(Request $request)
    {
        if (!$this->getUserLearnRecordService()->hasManageUserLearnReportRole()) {
            return $this->createMessageResponse('error', '没有查看数据统计的权限');
        }

        $fields = $request->query->all();
        $orgIds = $this->prepareOrgIds($fields);

        if (empty($fields['startDateTime']) && empty($fields['endDateTime'])) {
            list($startDateTime, $endDateTime) = DateToolkit::generateStartDateAndEndDate('month');
            $fields['startDateTime'] = $startDateTime;
            $fields['endDateTime'] = $endDateTime;
        }
        $count = $this->getOrgService()->countOrgs(
            array('orgIds' => $orgIds)
        );
        $paginator = new Paginator(
            $this->get('request'),
            $count,
            20
        );

        $orgs = $this->getOrgService()->searchOrgs(
            array('orgIds' => $orgIds),
            array(),
            $paginator->getOffsetCount(),
            $paginator->getPerPageCount()
        );

        $records = $this->getUserDailyLearnRecordService()->searchRecords(
            $this->prepareDepartmentLearnConditions($fields),
            array('id' => 'DESC'),
            0,
            PHP_INT_MAX
        );

        $recordIds = ArrayToolkit::column($records, 'id');

        $learnRecords = $this->getUserDailyLearnRecordService()->statisticsLearnRecordGroupByOrgId(array(
            'orgIds' => empty($orgIds) ? array(-1) : $orgIds,
            'recordIds' => empty($recordIds) ? array(-1) : $recordIds,
        ));
        $learnRecords = ArrayToolkit::index($learnRecords, 'orgId');

        $orgUserNums = $this->getUserService()->statisticsOrgUserNumGroupByOrgId();
        $orgUserNums = ArrayToolkit::index($orgUserNums, 'orgId');

        return $this->render(
            'data-report/course-learn-data-department.html.twig',
            array(
                'paginator' => $paginator,
                'orgs' => $orgs,
                'userNum' => $count,
                'learnRecords' => $learnRecords,
                'startDateTime' => $fields['startDateTime'],
                'endDateTime' => $fields['endDateTime'],
                'orgIds' => implode(',', $orgIds),
                'orgUserNums' => $orgUserNums,
            )
        );
    }

    protected function prepareDepartmentLearnConditions($fields)
    {
        $conditions = array();

        if (!empty($fields['startDateTime'])) {
            $conditions['startDateTime'] = strtotime($fields['startDateTime']);
        }

        if (!empty($fields['endDateTime'])) {
            $conditions['endDateTime'] = strtotime($fields['endDateTime']);
        }

        if (!empty($fields['postId'])) {
            $conditions['postId'] = $fields['postId'];
        }

        if (!empty($fields['categoryId'])) {
            $conditions['categoryId'] = $fields['categoryId'];
        }

        return $conditions;
    }

    public function courseLearnDataStatisticPostAction(Request $request)
    {
        if (!$this->getUserLearnRecordService()->hasManageUserLearnReportRole()) {
            return $this->createMessageResponse('error', '没有查看数据统计的权限');
        }

        $fields = $request->query->all();
        if (empty($fields['startDateTime']) && empty($fields['endDateTime'])) {
            list($startDateTime, $endDateTime) = DateToolkit::generateStartDateAndEndDate('month');
            $fields['startDateTime'] = $startDateTime;
            $fields['endDateTime'] = $endDateTime;
        }

        $orgIds = $this->prepareOrgIds($fields);
        $conditions = $this->prepareSearchConditions($fields);
        $count = $this->getPostService()->countPosts(
            array()
        );
        $paginator = new Paginator(
            $this->get('request'),
            $count,
            20
        );

        $posts = $this->getPostService()->searchPosts(
            array(),
            array(),
            $paginator->getOffsetCount(),
            $paginator->getPerPageCount()
        );

        if ($paginator->getCurrentPage() == 1) {
            $nonePost = array(
                'id' => 0,
                'code' => 'none',
                'name' => '无岗位',
            );
            array_unshift($posts, $nonePost);
        }

        $conditions['postIds'] = ArrayToolkit::column($posts, 'id');
        $learnRecords = $this->getUserDailyLearnRecordService()->statisticsLearnRecordGroupByPostId(
            $conditions
        );
        $learnRecords = ArrayToolkit::index($learnRecords, 'postId');

        $postUserNums = $this->getUserService()->statisticsPostUserNumGroupByPostId();
        $postUserNums = ArrayToolkit::index($postUserNums, 'postId');

        return $this->render(
            'data-report/course-learn-data-post.html.twig',
            array(
                'paginator' => $paginator,
                'posts' => $posts,
                'userNum' => $count,
                'learnRecords' => $learnRecords,
                'startDateTime' => $fields['startDateTime'],
                'endDateTime' => $fields['endDateTime'],
                'postUserNums' => $postUserNums,
                'orgIds' => implode(',', $orgIds),
            )
        );
    }

    public function courseLearnDataStatisticCategoryAction(Request $request)
    {
        if (!$this->getUserLearnRecordService()->hasManageUserLearnReportRole()) {
            return $this->createMessageResponse('error', '没有查看数据统计的权限');
        }

        $fields = $request->query->all();
        if (empty($fields['startDateTime']) && empty($fields['endDateTime'])) {
            list($startDateTime, $endDateTime) = DateToolkit::generateStartDateAndEndDate('month');
            $fields['startDateTime'] = $startDateTime;
            $fields['endDateTime'] = $endDateTime;
        }
        $orgIds = $this->prepareOrgIds($fields);
        $conditions = $this->prepareSearchConditions($fields);
        $categoryGroup = $this->getCategoryService()->getGroupByCode('course');
        $conditions['groupId'] = $categoryGroup['id'];
        $count = $this->getCategoryService()->countCategories($conditions);

        $paginator = new Paginator(
            $this->get('request'),
            $count,
            20
        );

        $categories = $this->getCategoryService()->searchCategories(
            $conditions,
            array(),
            $paginator->getOffsetCount(),
            $paginator->getPerPageCount()
        );

        if ($paginator->getCurrentPage() == 1) {
            $noneCategory = array(
                'id' => 0,
                'code' => 'none',
                'name' => '无分类',
            );
            array_unshift($categories, $noneCategory);
        }

        $conditions['categoryIds'] = ArrayToolkit::column($categories, 'id');
        $learnRecords = $this->getUserDailyLearnRecordService()->statisticsLearnRecordGroupByCategoryId(
            $conditions
        );

        $learnRecords = ArrayToolkit::index($learnRecords, 'categoryId');

        return $this->render(
            'data-report/course-learn-data-category.html.twig',
            array(
                'paginator' => $paginator,
                'categories' => $categories,
                'userNum' => $count,
                'learnRecords' => $learnRecords,
                'startDateTime' => $fields['startDateTime'],
                'endDateTime' => $fields['endDateTime'],
                'orgIds' => implode(',', $orgIds),
            )
        );
    }

    public function hotCoursesAction(Request $request, $orgIds = '')
    {
        if (!$this->getUserLearnRecordService()->hasManageUserLearnReportRole()) {
            return $this->createMessageResponse('error', '没有查看数据统计的权限');
        }

        $fields = $request->query->all();
        if (empty($fields['orgIds'])) {
            $fields['orgIds'] = $orgIds;
        }
        $fields['type'] = empty($fields['type']) ? 'week' : $fields['type'];
        list($startDateTime, $endDateTime) = DateToolkit::generateStartDateAndEndDate($fields['type']);
        $fields['startDateTime'] = $startDateTime;
        $fields['endDateTime'] = $endDateTime;
        $conditions = $this->prepareSearchConditions($fields);

        $ranks = $this->getUserDailyLearnRecordService()->statisticsLearnCourseRankingList($conditions);

        $courseIds = ArrayToolkit::column($ranks, 'courseId');
        $courses = $this->getCourseService()->findCoursesByIds($courseIds);
        $courses = ArrayToolkit::index($courses, 'id');

        return $this->render('data-report/widgets/hot-courses-list.html.twig', array(
            'studyRanks' => $ranks,
            'courses' => $courses,
        ));
    }

    public function userLearnRankListAction(Request $request, $orgIds = '')
    {
        $fields = $request->query->all();
        if (empty($fields['orgIds'])) {
            $fields['orgIds'] = $orgIds;
        }
        $fields['type'] = empty($fields['type']) ? 'week' : $fields['type'];
        list($startDateTime, $endDateTime) = DateToolkit::generateStartDateAndEndDate($fields['type']);
        $fields['startDateTime'] = $startDateTime;
        $fields['endDateTime'] = $endDateTime;

        if ($request->query->get('sourceFrom')) {
            $conditions = array(
                'startDateTime' => strtotime($fields['startDateTime']),
                'endDateTime' => strtotime($fields['endDateTime']),
            );
        } else {
            $conditions = $this->prepareSearchConditions($fields);
        }

        $ranks = $this->getUserDailyLearnRecordService()->statisticsPersonLearnTimeRankingList($conditions);

        return $this->render('data-report/widgets/user-learn-rank-list.html.twig', array(
            'studyRanks' => array('personStudyRanks' => $ranks, 'sourceFrom' => $request->query->get('sourceFrom')),
        ));
    }

    public function orgLearnRankListAction(Request $request)
    {
        $fields = $request->query->all();
        $fields['type'] = empty($fields['type']) ? 'week' : $fields['type'];
        list($startDateTime, $endDateTime) = DateToolkit::generateStartDateAndEndDate($fields['type']);
        $fields['startDateTime'] = $startDateTime;
        $fields['endDateTime'] = $endDateTime;
        $conditions = $this->prepareSearchConditions($fields);

        $ranks = $this->getUserDailyLearnRecordService()->statisticsOrgLearnTimeRankingList($conditions);

        $orgUsers = $this->getUserService()->statisticsOrgUserNumGroupByOrgId();
        $orgUsers = ArrayToolkit::index($orgUsers, 'orgId');

        return $this->render('data-report/widgets/org-learn-rank-list.html.twig', array(
            'studyRanks' => array('orgStudyRanks' => $ranks, 'orgUsers' => $orgUsers),
        ));
    }

    public function userLearnStudyRecordAction(Request $request, $userId)
    {
        $user = $this->getUserService()->getUser($userId);
        $user['profile'] = $this->getUserService()->getUserProfile($user['id']);
        $post = $this->getPostService()->getPost($user['postId']);
        $orgs = $this->getOrgService()->findOrgsByIds($user['orgIds']);

        $orgNames = OrgToolkit::buildOrgsNames($user['orgIds'], $orgs);

        $userReport = $this->getUserLearnRecordService()->getUserLearnReportByUserId($userId);

        $result = $this->calculatePostCoursesDatas($user['postId'], $user['id']);

        return $this->render('data-report/user-learn-record.html.twig', array(
            'postCourseCount' => $result['postCourseCount'],
            'totalLearnTime' => $result['totalLearnTime'],
            'learnedCoursesNum' => $result['learnedCoursesNum'],
            'courses' => $result['courses'],
            'recentStartTask' => $result['recentStartTask'],
            'userId' => $userId,
            'user' => $user,
            'post' => $post,
            'orgNames' => $orgNames,
            'userReport' => $userReport,
            'tab_types' => $request->get('tab_types', 'assignmentRecord'),
            'tab_type' => $request->get('tab_type', 'assignmentRecord'),
        ));
    }

    public function calculatePostCoursesDatas($postId, $userId)
    {
        $postCourses = $this->getPostCourseService()->findPostCoursesByPostId($postId);

        if (!empty($postCourses)) {
            $courseIds = ArrayToolkit::column($postCourses, 'courseId');
            $courses = $this->getCourseService()->findCoursesByIds($courseIds);
            list($courses, $totalLearnTime) = $this->calculateFinishedTaskNumAndLearnTime($userId, $courses);
            $learnedCoursesNum = $this->getPostCourseService()->countUserLearnedPostCourses($userId, $courseIds);
            $recentStartTask = $this->findRecentStartTask($userId, $courseIds);
        }

        $datas = array(
            'postCourseCount' => empty($postCourses) ? 0 : count($postCourses),
            'totalLearnTime' => empty($totalLearnTime) ? 0 : $totalLearnTime,
            'learnedCoursesNum' => empty($learnedCoursesNum) ? 0 : $learnedCoursesNum,
            'courses' => empty($courses) ? array() : $courses,
            'recentStartTask' => empty($recentStartTask) ? array() : $recentStartTask,
        );

        return $datas;
    }

    private function calculateFinishedTaskNumAndLearnTime($userId, $courses)
    {
        foreach ($courses as $key => &$course) {
            $course['finishedTaskNum'] = $this->getTaskResultService()->countTaskResults(array(
                'courseId' => $key,
                'userId' => $userId,
                'status' => 'finish',
            ));

            $course['learnTime'] = $this->getTaskResultService()->sumLearnTimeByCourseIdAndUserId($key, $userId);
        }
        $learnTime = ArrayToolkit::column($courses, 'learnTime');

        return array($courses, array_sum($learnTime));
    }

    private function findRecentStartTask($userId, $courseIds)
    {
        $taskResults = $this->getTaskResultService()->searchTaskResults(
            array('userId' => $userId, 'courseIds' => $courseIds),
            array('createdTime' => 'DESC'),
            0, 1
        );

        $taskResult = array_shift($taskResults);

        if (empty($taskResult)) {
            return array();
        }

        $task = $this->getTaskService()->getTask($taskResult['courseTaskId']);

        return $task;
    }

    protected function prepareOrgIds($conditions)
    {
        if (!isset($conditions['orgIds'])) {
            $orgIds = $this->getUserManageOrgs();

            return ArrayToolkit::column($orgIds, 'id');
        }

        return explode(',', $conditions['orgIds']);
    }

    protected function prepareSearchConditions($conditions)
    {
        if (!empty($conditions['startDateTime'])) {
            $conditions['startDateTime'] = strtotime($conditions['startDateTime']);
        }

        if (!empty($conditions['endDateTime'])) {
            $conditions['endDateTime'] = strtotime($conditions['endDateTime']);
        }

        if (isset($conditions['orgIds']) && empty($conditions['orgIds'])) {
            $userIds = -1;
        } else {
            $orgIds = $this->prepareOrgIds($conditions);
            $userIds = $this->findUserIdsByOrgIds($orgIds);
            $userIds = empty($userIds) ? -1 : $userIds;
        }

        $keywordUserIds = array();
        if (isset($conditions['keywordType']) && !empty($conditions['keyword'])) {
            $users = $this->getUserService()->searchUsers(
                array(
                    $conditions['keywordType'] => $conditions['keyword'],
                ),
                array('id' => 'DESC'),
                0,
                PHP_INT_MAX
            );

            $keywordUserIds = empty($users) ? array(-1) : ArrayToolkit::column($users, 'id');
            unset($conditions['keywordType']);
            unset($conditions['keyword']);
        }

        if ($keywordUserIds == -1 || $userIds == -1) {
            $conditions['userIds'] = array(-1);
        } elseif (empty($keywordUserIds)) {
            $conditions['userIds'] = $userIds;
        } else {
            $conditions['userIds'] = array_intersect($userIds, $keywordUserIds);
        }

        unset($conditions['orgIds']);

        return $conditions;
    }

    protected function findUserIdsByOrgIds($orgIds)
    {
        $usersOrg = $this->getUserOrgService()->findUserOrgsByOrgIds($orgIds);

        return ArrayToolkit::column($usersOrg, 'userId');
    }

    /**
     * @return UserDailyLearnRecordService
     */
    protected function getUserDailyLearnRecordService()
    {
        return $this->createService('CorporateTrainingBundle:DataStatistics:UserDailyLearnRecordService');
    }

    /**
     * @return TaskService
     */
    protected function getTaskService()
    {
        return $this->createService('Task:TaskService');
    }

    /**
     * @return UserService
     */
    protected function getUserService()
    {
        return $this->createService('CorporateTrainingBundle:User:UserService');
    }

    /**
     * @return UserLearnReportService
     */
    protected function getUserLearnRecordService()
    {
        return $this->createService('CorporateTrainingBundle:DataStatistics:UserLearnReportService');
    }

    /**
     * @return CategoryService
     */
    protected function getCategoryService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:CategoryService');
    }

    /**
     * @return PostService
     */
    protected function getPostService()
    {
        return $this->createService('Post:PostService');
    }

    /**
     * @return OrgService
     */
    protected function getOrgService()
    {
        return $this->createService('CorporateTrainingBundle:Org:OrgService');
    }

    /**
     * @return CourseService
     */
    protected function getCourseService()
    {
        return $this->createService('Course:CourseService');
    }

    /**
     * @return MemberService
     */
    protected function getOfflineCourseMemberService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:MemberService');
    }

    /**
     * @return UserOrgService
     */
    protected function getUserOrgService()
    {
        return $this->createService('CorporateTrainingBundle:User:UserOrgService');
    }

    /**
     * @return OfflineCourseService
     */
    protected function getOfflineCourseService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:OfflineCourseService');
    }

    /**
     * @return PostCourseService
     */
    protected function getPostCourseService()
    {
        return $this->createService('CorporateTrainingBundle:PostCourse:PostCourseService');
    }

    /**
     * @return TaskResultService
     */
    protected function getTaskResultService()
    {
        return $this->createService('Task:TaskResultService');
    }
}
