<?php

namespace CorporateTrainingBundle\Controller;

use AppBundle\Common\ArrayToolkit;
use Biz\Group\Service\Impl\ThreadServiceImpl;
use CorporateTrainingBundle\Biz\OfflineCourse\Service\OfflineCourseService;
use CorporateTrainingBundle\Biz\TrainingAssignment\Service\TrainingAssignmentMemberService;
use CorporateTrainingBundle\Biz\TrainingAssignment\Service\TrainingAssignmentService;
use Symfony\Component\HttpFoundation\Request;
use AppBundle\Controller\BaseController;

class HomeDefaultController extends BaseController
{
    public function indexAction(Request $request)
    {
        $user = $this->getCurrentUser();
        if (!empty($user)) {
            $user['courseLearningCount'] = $this->getCourseService()->countUserLearningCourses($user['id']);
            $user['courseCount'] = $this->getCourseService()->countUserLearnCourse($user['id']);
            $user['threadCount'] = $this->getThreadNum($user['id']);
            $user['learnTime'] = $this->getTaskResultService()->sumLearnTimeByUserId($user['id']);
            $postCourses = $this->getPostCourseService()->findPostCoursesByPostId($user['postId']);

            $this->getBatchNotificationService()->checkoutBatchNotification($user['id']);
        }
        $user['taskCount'] = $this->getTaskCount();

        //部门课程模块获取课程与组织机构
        $user = $this->getCurrentUser();

        $conditions = $request->query->all();

        $courseSetting = $this->getSettingService()->get('course', array());

        if (!isset($courseSetting['explore_default_orderBy'])) {
            $courseSetting['explore_default_orderBy'] = 'latest';
        }

        $orderBy = $courseSetting['explore_default_orderBy'];
        $orderBy = empty($conditions['orderBy']) ? $orderBy : $conditions['orderBy'];
        unset($conditions['orderBy']);

        $conditions['parentId'] = 0;
        $conditions['status'] = 'published';
        $conditions['orgCodes'] = $this->getOrgCodes($user['orgCodes']);
        $courseSets = $this->getCourseSetService()->searchCourseSets(
            $conditions,
            $orderBy,
            null,
            null
        );
        $courseSets = array_slice($courseSets, 0, 4);

        //小组模块获取小组
        $activeGroup = $this->getGroupService()->searchGroups(array('status' => 'open'), array('memberNum' => 'DESC'),
            0, 8);

        $offlineCourseConditions = array(
            'status' => 'published',
            'enrollmentEndDate_GE' => time(),
        );
        $offlineCourses = $this->getOfflineCourseService()->searchOfflineCourses(
            $offlineCourseConditions,
            array('enrollmentEndDate' => 'ASC'),
            0,
            12
        );

        return $this->render(
            'default/index.html.twig',
            array(
                'time' => time(),
                'user' => $user,
                'courseSets' => $courseSets,
                'activeGroup' => $activeGroup,
                'offlineCourses' => $offlineCourses,
            )
        );
    }

    protected function getTaskCount()
    {
        $user = $this->getCurrentUser();

        $allPostCourseCount = $this->getPostCourseService()->countPostCourses(
            array('postId' => $user['postId'])
        );
        $finishedPostCourseNum = $this->getPostCourseService()->countFinishedPostCoursesByPostIdAndUserId($user['postId'], $user['id']);
        $postCourseCount = $allPostCourseCount - $finishedPostCourseNum;

        $assignments = $this->getTrainingAssignmentMemberService()->findMembersByUserId($user['id']);
        $assignmentIds = ArrayToolkit::column($assignments, 'assignmentId');

        $assignmentTrainings = $this->getTrainingAssignmentService()->searchTrainingAssignments(
            array('ids' => $assignmentIds, 'status' => 'published'),
            array(),
            0,
            PHP_INT_MAX
        );

        $publishAssignmentIds = ArrayToolkit::column($assignmentTrainings, 'id');

        $assignmentTrainingCount = $this->getTrainingAssignmentMemberService()->countTrainingAssignmentMembers(
            array('assignmentIds' => $publishAssignmentIds, 'statuses' => array('ongoing', 'notStart'), 'userId' => $user['id'])
        );

        $surveyCount = 0;
        if ($this->isPluginInstalled('Survey')) {
            $surveyCount = $this->getSurveyMemberService()->getUserUnfinishedSurveyNum($user['id']);
        }

        $examCount = 0;
        if ($this->isPluginInstalled('Exam')) {
            $examCount = $this->getExamService()->getUserUnfinishedExamNum($user['id']);
        }

        return $postCourseCount + $assignmentTrainingCount + $surveyCount + $examCount;
    }

    protected function getOrgCodes($orgCodes)
    {
        if (count($orgCodes) > 1 && in_array('1.', $orgCodes)) {
            foreach ($orgCodes as $key => $orgCode) {
                if ($orgCode == '1.') {
                    unset($orgCodes[$key]);
                    break;
                }
            }
        }

        return $orgCodes;
    }

    /**
     * @return TrainingAssignmentService
     */
    protected function getTrainingAssignmentService()
    {
        return $this->createService('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentService');
    }

    /**
     * @return TrainingAssignmentMemberService
     */
    protected function getTrainingAssignmentMemberService()
    {
        return $this->createService('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentMemberService');
    }

    /**
     * @return BatchNotificationService
     */
    protected function getBatchNotificationService()
    {
        return $this->getBiz()->service('User:BatchNotificationService');
    }

    protected function getThreadNum($userId)
    {
        $threadNum = $this->getThreadService()->countThreads(array('userId' => $userId));
        $threadPostNum = $this->getThreadService()->searchThreadPostsCount(array('userId' => $userId));

        return (int) $threadNum + (int) $threadPostNum[0]['count'];
    }

    protected function getCourseService()
    {
        return $this->createService('Course:CourseService');
    }

    /**
     * @return ThreadServiceImpl
     */
    protected function getThreadService()
    {
        return $this->createService('Course:ThreadService');
    }

    protected function getTaskResultService()
    {
        return $this->createService('CorporateTrainingBundle:Task:TaskResultService');
    }

    protected function getPostCourseService()
    {
        return $this->createService('CorporateTrainingBundle:PostCourse:PostCourseService');
    }

    protected function getOrgService()
    {
        return $this->createService('Org:OrgService');
    }

    protected function getSettingService()
    {
        return $this->createService('System:SettingService');
    }

    protected function getCourseSetService()
    {
        return $this->createService('Course:CourseSetService');
    }

    protected function getGroupService()
    {
        return $this->createService('Group:GroupService');
    }

    /**
     * @return OfflineCourseService
     */
    protected function getOfflineCourseService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:OfflineCourseService');
    }

    protected function getSurveyMemberService()
    {
        return $this->createService('SurveyPlugin:Survey:SurveyMemberService');
    }

    protected function getExamService()
    {
        return $this->createService('ExamPlugin:Exam:ExamService');
    }
}
