<?php

namespace CorporateTrainingBundle\Controller;

use AppBundle\Common\SimpleValidator;
use Biz\User\CurrentUser;
use CorporateTrainingBundle\Component\EIMClient\DingTalk\User;
use Symfony\Component\HttpFoundation\Request;
use AppBundle\Controller\LoginBindController as BaseLoginBindController;
use CorporateTrainingBundle\Component\EIMClient\UserFactory;
use AppBundle\Component\OAuthClient\OAuthClientFactory;

class LoginBindController extends BaseLoginBindController
{
    public function callbackAction(Request $request, $type)
    {
        $code = $request->query->get('code');
        $inviteCode = $request->query->get('inviteCode');
        $token = $request->query->get('token', '');

        $this->validateToken($request, $type);

        $callbackUrl = $this->generateUrl('login_bind_callback', array('type' => $type, 'token' => $token), true);
        $token = $this->createOAuthClient($type)->getAccessToken($code, $callbackUrl);

        $bind = $this->getUserService()->getUserBindByTypeAndFromId($type, $token['userId']);

        $request->getSession()->set('oauth_token', $token);

        if ($bind) {
            $user = $this->getUserService()->getUser($bind['toId']);

            if (empty($user) || $user['locked'] == 1) {
                $this->setFlashMessage('danger', '绑定的用户不存在！');

                return $this->redirect($this->generateUrl('login'));
            }

            $this->authenticateUser($user);

            if ($this->getAuthService()->hasPartnerAuth()) {
                return $this->redirect($this->generateUrl('partner_login', array('goto' => $this->getTargetPath($request))));
            } else {
                $goto = $this->getTargetPath($request);

                return $this->redirect($goto);
            }
        } else {
            $setting = $this->getSettingService()->get('sync_department_setting', array());

            if ($setting && $setting['enable']) {
                $oauthclient = $this->createOAuthClient($type);
                $oauthUser = $oauthclient->getUserInfo($token);
                $userClient = UserFactory::create($setting);

                if ($type == 'dingtalkweb') {
                    $userId = $userClient->getUserIdByUnionId($oauthUser['id']);

                    if (!$userId) {
                        return $this->createMessageResponse('error', '您的钉钉账号还未加入该企业钉钉！');
                    }

                    $user = $userClient->get($userId);
                }

                if ($user && !$this->isDepartmentMember($user)) {
                    return $this->createMessageResponse('error', '用户所属的部门不存在，请联系管理员同步部门！');
                }

                if (!$this->isExistedUserByMobile($user['mobile'])) {
                    return $this->redirect($this->generateUrl('login_bind_choose', array('type' => $type, 'inviteCode' => $inviteCode)));
                }
            }

            if (!empty($user) && $this->isExistedUserByMobile($user['mobile'])) {
                $isExist = array(
                    'isExisted' => true,
                    'mobile' => $user['mobile'],
                );
                $isExist = json_encode($isExist);

                return $this->redirect($this->generateUrl('login_bind_change', array('type' => $type, 'inviteCode' => $inviteCode, 'isExist' => base64_encode($isExist))));
            } else {
                return $this->redirect($this->generateUrl('login_bind_change', array('type' => $type, 'inviteCode' => $inviteCode)));
            }
        }
    }

    public function chooseAction(Request $request, $type)
    {
        $token = $request->getSession()->get('oauth_token');
        $inviteCode = $request->query->get('inviteCode', '');
        $inviteUser = $inviteCode ? $inviteUser = $this->getUserService()->getUserByInviteCode($inviteCode) : array();

        $client = $this->createOAuthClient($type);
        $clientMetas = OAuthClientFactory::clients();
        $clientMeta = $clientMetas[$type];

        try {
            $oauthUser = $client->getUserInfo($token);
            $oauthUser['name'] = preg_replace('/[^\x{4e00}-\x{9fa5}a-zA-z0-9_.]+/u', '', $oauthUser['name']);
            $oauthUser['name'] = str_replace(array('-'), array('_'), $oauthUser['name']);
        } catch (\Exception $e) {
            $message = $e->getMessage();

            if ($message == 'unaudited') {
                $this->setFlashMessage('danger', $this->get('translator')->trans('user.bind.unaudited', array('%name%' => $clientMeta['name'])));
            } elseif ($message == 'unAuthorize') {
                return $this->redirect($this->generateUrl('login'));
            } else {
                $this->setFlashMessage('danger', $this->get('translator')->trans('user.bind.error', array('%message%' => $message)));
            }

            return $this->redirect($this->generateUrl('login'));
        }

        $name = $this->mateName($type);
        $syncSetting = $this->getSettingService()->get('sync_department_setting', array());
        if ($syncSetting && $syncSetting['enable'] && $syncSetting['type'] == 'dingtalk') {
            $userClient = UserFactory::create($syncSetting);
            $userId = $userClient->getUserIdByUnionId($oauthUser['id']);

            if (!$userId) {
                return $this->createMessageResponse('error', '您的钉钉账号还未加入该企业钉钉！');
            }

            $userInfo = $userClient->get($userId);
        }

        return $this->render('login/bind-choose.html.twig', array(
            'inviteUser' => $inviteUser,
            'oauthUser' => $oauthUser,
            'type' => $type,
            'name' => $name,
            'hasPartnerAuth' => $this->getAuthService()->hasPartnerAuth(),
            'userInfo' => $userInfo,
        ));
    }

    public function changeToExistAction(Request $request, $type)
    {
        $token = $request->getSession()->get('oauth_token');
        $isExist = $request->query->get('isExist', false);
        $isExist = json_decode(base64_decode($isExist), true);

        $user = $this->getUserService()->getUserByVerifiedMobile($isExist['mobile']);

        if (empty($token)) {
            return $this->createMessageResponse('error', '页面已过期，请重新登录。');
        }

        $client = $this->createOAuthClient($type);
        $oauthUser = $client->getUserInfo($token);
        $name = $this->mateExistName($type);

        return $this->render('login/bind-choose-exist.html.twig', array(
            'oauthUser' => $oauthUser,
            'type' => $type,
            'name' => $name,
            'hasPartnerAuth' => $this->getAuthService()->hasPartnerAuth(),
            'isExisted' => $isExist['isExisted'],
            'user' => empty($user) ? null : $user,
        ));
    }

    public function newSetAction(Request $request, $type)
    {
        $setData = $request->request->all();
        $syncSetting = $this->getSettingService()->get('sync_department_setting', array());

        if (isset($setData['set_bind_emailOrMobile']) && !empty($setData['set_bind_emailOrMobile'])) {
            if (SimpleValidator::email($setData['set_bind_emailOrMobile'])) {
                $setData['email'] = $setData['set_bind_emailOrMobile'];
            } elseif (SimpleValidator::mobile($setData['set_bind_emailOrMobile'])) {
                $setData['mobile'] = $setData['set_bind_emailOrMobile'];
            }

            unset($setData['set_bind_emailOrMobile']);
        }

        $token = $request->getSession()->get('oauth_token');

        if (empty($token)) {
            $response = array('success' => false, 'message' => '页面已过期，请重新登录。');
            goto response;
        }

        $client = $this->createOAuthClient($type);
        $oauthUser = $client->getUserInfo($token);
        $oauthUser['createdIp'] = $request->getClientIp();

        if (empty($oauthUser['id'])) {
            $response = array('success' => false, 'message' => '网络超时，获取用户信息失败，请重试。');
            goto response;
        }

        if (!(!empty($syncSetting) && $syncSetting['enable'] && $syncSetting['type'] == 'dingtalk')) {
            if (!$this->getAuthService()->isRegisterEnabled()) {
                $response = array('success' => false, 'message' => '注册功能未开启，请联系管理员！');
                goto response;
            }
        }

        $user = $this->generateUser($type, $token, $oauthUser, $setData);

        if (empty($user)) {
            $response = array('success' => false, 'message' => '登录失败，请重试！');
            goto response;
        }

        if (!empty($syncSetting) && $syncSetting['enable'] && $syncSetting['type'] == 'dingtalk') {
            if (!$user['setup'] && isset($setData['emailDingTalk'])) {
                $this->getUserService()->setupAccount($user['id']);
            }
        } else {
            if (!$user['setup'] && isset($setData['email']) && stripos($setData['email'], '@edusoho.net') != false) {
                $this->getUserService()->setupAccount($user['id']);
            }
        }

        $currentUser = new CurrentUser();
        $currentUser->fromArray($user);
        $this->switchUser($request, $currentUser);

        if (!empty($oauthUser['avatar'])) {
            $this->getUserService()->changeAvatarFromImgUrl($user['id'], $oauthUser['avatar']);
        }

        $redirectUrl = $this->generateUrl('register_success', array(
            'goto' => $this->getTargetPath($request),
        ));
        $response = array('success' => true, '_target_path' => $redirectUrl);

        response:
        return $this->createJsonResponse($response);
    }

    protected function generateUser($type, $token, $oauthUser, $setData)
    {
        $registration = array();
        $randString = base_convert(sha1(uniqid(mt_rand(), true)), 16, 36);
        $oauthUser['name'] = preg_replace('/[^\x{4e00}-\x{9fa5}a-zA-z0-9_.]+/u', '', $oauthUser['name']);
        $oauthUser['name'] = str_replace(array('-'), array('_'), $oauthUser['name']);

        if (!SimpleValidator::nickname($oauthUser['name'])) {
            $oauthUser['name'] = '';
        }

        $tempType = $type;

        if (empty($oauthUser['name'])) {
            if ($type == 'weixinmob' || $type == 'weixinweb') {
                $tempType = 'weixin';
            }

            $oauthUser['name'] = $tempType.substr($randString, 9, 3);
        }

        $nameLength = mb_strlen($oauthUser['name'], 'utf-8');

        if ($nameLength > 10) {
            $oauthUser['name'] = mb_substr($oauthUser['name'], 0, 11, 'utf-8');
        }

        if (!empty($setData['nickname']) && !empty($setData['email'])) {
            $registration['nickname'] = $setData['nickname'];
            $registration['email'] = $setData['email'];
            $registration['emailOrMobile'] = $setData['email'];
        } else {
            $nicknames = array();
            $nicknames[] = isset($setData['nickname']) ? $setData['nickname'] : $oauthUser['name'];
            $nicknames[] = mb_substr($oauthUser['name'], 0, 8, 'utf-8').substr($randString, 0, 3);
            $nicknames[] = mb_substr($oauthUser['name'], 0, 8, 'utf-8').substr($randString, 3, 3);
            $nicknames[] = mb_substr($oauthUser['name'], 0, 8, 'utf-8').substr($randString, 6, 3);

            foreach ($nicknames as $name) {
                if ($this->getUserService()->isNicknameAvaliable($name)) {
                    $registration['nickname'] = $name;
                    break;
                }
            }

            if (empty($registration['nickname'])) {
                return array();
            }

            $syncSetting = $this->getSettingService()->get('sync_department_setting', array());
            if (!empty($syncSetting) && $syncSetting['enable'] && $syncSetting['type'] == 'dingtalk') {
                if (!empty($setData['emailDingtalk'])) {
                    $registration['email'] = $setData['emailDingTalk'];
                } else {
                    $registration['email'] = 'u_'.substr($randString, 0, 12).'@edusoho.net';
                }
            } else {
                $registration['email'] = 'u_'.substr($randString, 0, 12).'@edusoho.net';
            }
        }

        if ($this->getSensitiveService()->scanText($registration['nickname'])) {
            return $this->createMessageResponse('error', '用户名中含有敏感词！');
        }

        $registration['password'] = substr(base_convert(sha1(uniqid(mt_rand(), true)), 16, 36), 0, 8);
        $registration['token'] = $token;
        $registration['createdIp'] = $oauthUser['createdIp'];

        if (isset($setData['mobile']) && !empty($setData['mobile'])) {
            $registration['mobile'] = $setData['mobile'];
            $registration['emailOrMobile'] = $setData['mobile'];
        }

        if (isset($setData['invite_code']) && !empty($setData['invite_code'])) {
            $registration['invite_code'] = $setData['invite_code'];
        }

        $user = $this->getAuthService()->register($registration, $type);

        return $user;
    }

    private function isDepartmentMember($user)
    {
        $result = false;
        $orgs = $this->getOrgService()->findOrgsBySyncIds($user['department']);

        if ($orgs) {
            $result = true;
        }

        return $result;
    }

    private function isExistedUserByMobile($mobile)
    {
        $result = false;
        $user = $this->getUserService()->getUserByVerifiedMobile($mobile);

        if ($user) {
            $result = true;
        }

        return $result;
    }

    public function existAction(Request $request, $type)
    {
        $token = $request->getSession()->get('oauth_token');
        $client = $this->createOAuthClient($type);
        $oauthUser = $client->getUserInfo($token);
        $data = $request->request->all();

        $message = 'Email地址或手机号码或用户名输入错误';

        if (SimpleValidator::email($data['emailOrMobileOrNickname'])) {
            $user = $this->getUserService()->getUserByEmail($data['emailOrMobileOrNickname']);
            $message = '该Email地址尚未注册';
        } elseif (SimpleValidator::mobile($data['emailOrMobileOrNickname'])) {
            $user = $this->getUserService()->getUserByVerifiedMobile($data['emailOrMobileOrNickname']);
            $message = '该手机号码尚未注册';
        } elseif (SimpleValidator::nickname($data['emailOrMobileOrNickname'])) {
            $user = $this->getUserService()->getUserByNickname($data['emailOrMobileOrNickname']);
            $message = '该用户名尚未注册';
        }

        if (empty($user)) {
            $response = array('success' => false, 'message' => $message);
        } elseif (!$this->getUserService()->verifyPassword($user['id'], $data['password'])) {
            $response = array('success' => false, 'message' => '密码不正确，请重试！');
        } elseif ($this->getUserService()->getUserBindByTypeAndUserId($type, $user['id'])) {
            $response = array('success' => false, 'message' => sprintf('该%s帐号已经绑定了该第三方网站的其他帐号，如需重新绑定，请先到账户设置中取消绑定！', $this->setting('site.name')));
        } else {
            $response = array('success' => true, '_target_path' => $this->getTargetPath($request));
            $this->getUserService()->bindUser($type, $oauthUser['id'], $user['id'], $token);
            $currentUser = new CurrentUser();
            $currentUser->fromArray($user);
            $this->switchUser($request, $currentUser);
        }

        return $this->createJsonResponse($response);
    }

    protected function mateName($type)
    {
        switch ($type) {
            case 'weixinweb':
            case 'weixinmob':
                return '微信注册帐号';
            case 'dingtalkweb':
            case 'dingtalkmob':
                return '钉钉注册账号';
            case 'weibo':
                return '微博注册帐号';
            case 'qq':
                return 'QQ注册账号';
            case 'renren':
                return '人人注册账号';
            default:
                return '';
        }
    }

    protected function mateExistName($type)
    {
        switch ($type) {
            case 'weixinweb':
            case 'weixinmob':
                return '微信绑定已有账号';
            case 'dingtalkweb':
            case 'dingtalkmob':
                return '钉钉注册账号';
            case 'weibo':
                return '微博绑定已有账号';
            case 'qq':
                return 'QQ绑定已有账号';
            case 'renren':
                return '人人绑定已有账号';
            default:
                return '';
        }
    }

    protected function getOrgService()
    {
        return $this->createService('Org:OrgService');
    }

    protected function getSettingService()
    {
        return $this->createService('System:SettingService');
    }
}
