<?php

namespace CorporateTrainingBundle\Controller\My;

use AppBundle\Controller\BaseController;
use CorporateTrainingBundle\Biz\OfflineCourse\Service\OfflineCourseService;
use Symfony\Component\HttpFoundation\Request;
use AppBundle\Common\Paginator;
use AppBundle\Common\ArrayToolkit;

class OfflineCourseController extends BaseController
{
    public function learnAction(Request $request)
    {
        $user = $this->getCurrentUser();
        $conditions['userId'] = $user['id'];
        $members = $this->getOfflineCourseMemberService()->searchMembers(
            $conditions,
            array(),
            0,
            PHP_INT_MAX
        );

        $paginator = new Paginator(
            $request,
            count($members),
            10
        );

        if (!empty($members)) {
            $offlineCourseIds = ArrayToolkit::column($members, 'offlineCourseId');
            $members = ArrayToolkit::index($members, 'offlineCourseId');
            $conditions['ids'] = $offlineCourseIds;

            $offlineCourses = $this->getOfflineCourseService()->searchOfflineCourses(
                $conditions,
                array('startDate' => 'DESC'),
                $paginator->getOffsetCount(),
                $paginator->getPerPageCount()
            );

            foreach ($offlineCourses as &$offlineCourse) {
                $offlineCourse['member'] = $members[$offlineCourse['id']];
                $offlineCourse['month'] = date('m', $offlineCourse['startDate']);
                $offlineCourse['day'] = date('d', $offlineCourse['startDate']);
                $offlineCourse['year'] = date('Y', $offlineCourse['startDate']);
            }
            $offlineCourses = ArrayToolkit::group($offlineCourses, 'year');
        } else {
            $offlineCourses = array();
        }

        return $this->render(
            'my/learning/offline-course/offline-course.html.twig',
            array(
                'offlineCourses' => $offlineCourses,
                'filter' => 'courses_record',
                'paginator' => $paginator,
            )
        );
    }

    public function enrollmentRecordAction(Request $request)
    {
        $user = $this->getCurrentUser();
        $conditions['userId'] = $user['id'];

        $count = $this->getOfflineCourseEnrollmentRecordService()->countEnrollmentRecords($conditions);
        $paginator = new Paginator(
            $request,
            $count,
            10
        );

        $enrollmentRecords = $this->getOfflineCourseEnrollmentRecordService()->searchEnrollmentRecords(
            $conditions,
            array('submittedTime' => 'DESC'),
            $paginator->getOffsetCount(),
            $paginator->getPerPageCount()
        );

        $offlineCourseIds = ArrayToolkit::column($enrollmentRecords, 'offlineCourseId');
        $offlineCourses = $this->getOfflineCourseService()->findOfflineCoursesByIds($offlineCourseIds);
        $offlineCourses = ArrayToolkit::index($offlineCourses, 'id');

        foreach ($enrollmentRecords as &$enrollmentRecord) {
            $enrollmentRecord['offlineCourse'] = $offlineCourses[$enrollmentRecord['offlineCourseId']];
        }

        return $this->render(
            'my/learning/offline-course/offline-course.html.twig',
            array(
                'enrollmentRecords' => $enrollmentRecords,
                'filter' => 'enrollment_record',
                'paginator' => $paginator,
            )
        );
    }

    /**
     * @return OfflineCourseService
     */
    protected function getOfflineCourseService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:OfflineCourseService');
    }

    protected function getOfflineCourseMemberService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:MemberService');
    }

    protected function getOfflineCourseEnrollmentRecordService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:EnrollmentRecordService');
    }
}
