<?php

namespace CorporateTrainingBundle\Controller\OfflineCourse;

use AppBundle\Common\ArrayToolkit;
use AppBundle\Controller\Admin\BaseController;
use CorporateTrainingBundle\Biz\OfflineCourse\Service\MemberService;
use CorporateTrainingBundle\Biz\OfflineCourse\Service\OfflineCourseService;
use Symfony\Component\HttpFoundation\Request;

class OfflineCourseController extends BaseController
{
    public function detailAction(Request $request, $id)
    {
        $offlineCourse = $this->getOfflineCourseService()->getOfflineCourse($id);

        if ('published' != $offlineCourse['status']) {
            throw $this->createNotFoundException('没有该活动');
        }
        $type = (time() > $offlineCourse['endDate']) ? 'end' : 'ongoing';
        $members = $this->getOfflineCourseMemberService()->findMembersByOfflineCourseId($id);

        $userIds = ArrayToolkit::column($members, 'userId');
        $users = $this->getUserService()->findUsersByIds($userIds);

        return $this->render('offline-course/detail.html.twig', array(
            'offlineCourse' => $offlineCourse,
            'type' => $type,
            'users' => $users,
            'showType' => 'detail',
        ));
    }

    public function listAction(Request $request, $type)
    {
        $conditions = array(
            'status' => 'published',
        );

        $conditions['searchType'] = ($type == 'end' ? $type : 'ongoing');

        $count = $this->getOfflineCourseService()->countOfflineCourses($conditions);

        return $this->render('offline-course/list.html.twig', array(
            'type' => $type,
            'total' => $count,
        ));
    }

    public function ajaxGetRowHtmlAction(Request $request)
    {
        $conditions = array(
            'status' => 'published',
        );

        $searchType = $request->query->get('type', 'ongoing');

        $conditions['searchType'] = ($searchType == 'end' ? $searchType : 'ongoing');

        $start = $request->query->get('start', 0);

        $offlineCourses = $this->getOfflineCourseService()->searchOfflineCourses(
            $conditions,
            array('startDate' => 'ASC'),
            $start,
            5
        );

        return $this->render('offline-course/list-row.html.twig', array(
            'offlineCourses' => $offlineCourses,
            'type' => $searchType,
            'user' => $this->getCurrentUser(),
        ));
    }

    public function checkMemberAction(Request $request, $id)
    {
        $keyWord = $request->query->get('value');
        $user = $this->getUserService()->getUserByLoginField($keyWord);

        if (empty($user)) {
            $result = '用户不存在！';
        } else {
            $result = $this->getOfflineCourseMemberService()->isMember($id, $user['id']);

            if ($result) {
                $result = '用户已报名该线下活动！';
            } else {
                $result = !$result;
            }
        }

        return $this->createJsonResponse($result);
    }

    public function enrollAction(Request $request, $courseId)
    {
        $user = $this->getCurrentUser();

        if (!$user->isLogin()) {
            throw $this->createAccessDeniedException();
        }

        $offlineCourse = $this->getOfflineCourseService()->getOfflineCourse($courseId);
        if (empty($offlineCourse['requireAudit'])) {
            return $this->redirectToRoute('offline_course_attend', array('courseId' => $courseId));
        } else {
            return $this->redirectToRoute('offline_course_apply', array('courseId' => $courseId));
        }
    }

    public function attendAction(Request $request, $courseId)
    {
        $offlineCourse = $this->getOfflineCourseService()->getOfflineCourse($courseId);
        if ('POST' === $request->getMethod()) {
            $user = $this->getCurrentUser();
            $this->getOfflineCourseMemberService()->becomeMember($courseId, $user['id']);

            return $this->createJsonResponse(true);
        }

        return $this->render(
            '@CorporateTraining/offline-course/attend-modal.html.twig',
            array(
                'offlineCourse' => $offlineCourse,
            )
        );
    }

    public function applyAction(Request $request, $courseId)
    {
        $offlineCourse = $this->getOfflineCourseService()->getOfflineCourse($courseId);
        if ('POST' === $request->getMethod()) {
            $user = $this->getCurrentUser();
            $this->getOfflineCourseService()->applyAttendOfflineCourse($courseId, $user['id']);

            return $this->createJsonResponse(true);
        }

        return $this->render(
            '@CorporateTraining/offline-course/apply-modal.html.twig',
            array(
                'offlineCourse' => $offlineCourse,
            )
        );
    }

    public function signInAction(Request $request, $courseId)
    {
        $offlineCourse = $this->getOfflineCourseService()->getOfflineCourse($courseId);

        if ($offlineCourse['endDate'] < time()) {
            return $this->render('offline-course/sign-in/error.html.twig',
                array(
                    'courseId' => $courseId,
                    'message' => '活动已经结束',
                )
            );
        }

        if ('POST' === $request->getMethod()) {
            $field = $request->request->get('field');
            $user = $this->getUserService()->getUserByLoginField($field);

            if (empty($user)) {
                return $this->render('offline-course/sign-in/error.html.twig',
                    array(
                        'courseId' => $courseId,
                        'message' => '输入信息有误，请重新输入',
                    )
                );
            }

            $member = $this->getOfflineCourseMemberService()->getMemberByCourseIdAndUserId($courseId, $user['id']);
            if (empty($member)) {
                return $this->render('offline-course/sign-in/error.html.twig',
                    array(
                        'courseId' => $courseId,
                        'message' => '您还未报名该活动',
                    )
                );
            }

            if ($member['attendedStatus'] == 'attended') {
                return $this->render('offline-course/sign-in/error.html.twig',
                    array(
                        'courseId' => $courseId,
                        'message' => '该活动你已签到,不能重复签到',
                    )
                );
            }

            $this->getOfflineCourseMemberService()->signIn($member['id']);

            return $this->render('offline-course/sign-in/success.html.twig',
                array(
                    'courseId' => $courseId,
                    'message' => '签到成功',
                )
            );
        }

        return $this->render('offline-course/sign-in/index.html.twig',
            array(
                'offlineCourse' => $offlineCourse,
            )
        );
    }

    /**
     * @return OfflineCourseService
     */
    protected function getOfflineCourseService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:OfflineCourseService');
    }

    /**
     * @return MemberService
     */
    protected function getOfflineCourseMemberService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:MemberService');
    }
}
