<?php

namespace CorporateTrainingBundle\Controller\OfflineCourse;

use AppBundle\Common\ArrayToolkit;
use AppBundle\Common\Paginator;
use AppBundle\Controller\Admin\BaseController;
use CorporateTrainingBundle\Biz\OfflineCourse\Service\EnrollmentRecordService;
use CorporateTrainingBundle\Biz\OfflineCourse\Service\MemberService;
use CorporateTrainingBundle\Biz\OfflineCourse\Service\OfflineCourseService;
use CorporateTrainingBundle\Common\OrgToolkit;
use Symfony\Component\HttpFoundation\Request;

class OfflineCourseManageController extends BaseController
{
    public function createAction(Request $request)
    {
        if (!$this->getOfflineCourseService()->hasCourseCreateRole()) {
            return $this->createMessageResponse('error', '没有创建活动的权限');
        }

        if ($request->getMethod() == 'POST') {
            $formData = $request->request->all();

            $offlineCourse = $this->getOfflineCourseService()->createOfflineCourse($formData);

            return $this->redirect($this->generateUrl('offline_course_manage_base',
                array('id' => $offlineCourse['id'])));
        }

        return $this->render('offline-course-manage/create.html.twig');
    }

    public function headerAction($offlineCourse)
    {
        if (!$this->getOfflineCourseService()->hasCourseManageRole()) {
            return $this->createMessageResponse('error', '没有管理活动的权限');
        }

        $category = $this->getCategoryService()->getCategory($offlineCourse['categoryId']);

        return $this->render(
            'offline-course-manage/header.html.twig',
            array(
                'offlineCourse' => $offlineCourse,
                'category' => $category,
            )
        );
    }

    public function sidebarAction($offlineCourseId, $sideNav)
    {
        if (!$this->getOfflineCourseService()->hasCourseManageRole()) {
            return $this->createMessageResponse('error', '没有管理活动的权限');
        }

        $offlineCourse = $this->getOfflineCourseService()->getOfflineCourse($offlineCourseId);

        return $this->render(
            'offline-course-manage/sidebar.html.twig',
            array(
                'offlineCourse' => $offlineCourse,
                'side_nav' => $sideNav,
            )
        );
    }

    public function baseAction(Request $request, $id)
    {
        if (!$this->getOfflineCourseService()->hasCourseManageRole()) {
            return $this->createMessageResponse('error', '没有管理活动的权限');
        }

        $offlineCourse = $this->getOfflineCourseService()->getOfflineCourse($id);
        if ($request->getMethod() == 'POST') {
            $data = $request->request->all();
            $data = $this->conversionTime($data, $offlineCourse);
            $this->getOfflineCourseService()->updateOfflineCourse($id, $data);
            $this->setFlashMessage('success', '活动信息已保存！');

            return $this->redirect($this->generateUrl('offline_course_manage_base', array('id' => $id)));
        }

        $status = $this->decideStartDate($offlineCourse['startDate']);

        return $this->render(
            'offline-course-manage/base.html.twig',
            array(
                'offlineCourse' => $offlineCourse,
                'status' => $status,
            )
        );
    }

    public function coverAction(Request $request, $id)
    {
        if (!$this->getOfflineCourseService()->hasCourseManageRole()) {
            return $this->createMessageResponse('error', '没有管理活动的权限');
        }

        if ($request->isMethod('POST')) {
            $data = $request->request->all();
            $this->getOfflineCourseService()->changeCourseCover($id, $data);

            return $this->redirect($this->generateUrl('offline_course_manage_cover', array('id' => $id)));
        }

        $offlineCourse = $this->getOfflineCourseService()->getOfflineCourse($id);

        return $this->render(
            'offline-course-manage/cover.html.twig',
            array(
                'offlineCourse' => $offlineCourse,
            )
        );
    }

    public function coverCropAction(Request $request, $id)
    {
        if (!$this->getOfflineCourseService()->hasCourseManageRole()) {
            return $this->createMessageResponse('error', '没有管理活动的权限');
        }

        $offlineCourse = $this->getOfflineCourseService()->getOfflineCourse($id);

        if ($request->getMethod() == 'POST') {
            $data = $request->request->all();
            $this->getOfflineCourseService()->changeCourseCover($offlineCourse['id'],
                json_decode($data['images'], true));

            return $this->redirect($this->generateUrl('offline_course_manage_cover',
                array('id' => $offlineCourse['id'])));
        }

        $fileId = $request->getSession()->get('fileId');
        list($pictureUrl, $naturalSize, $scaledSize) = $this->getFileService()->getImgFileMetaInfo($fileId, 480, 270);

        return $this->render(
            'offline-course-manage/cover-crop.html.twig',
            array(
                'offlineCourse' => $offlineCourse,
                'pictureUrl' => $pictureUrl,
                'naturalSize' => $naturalSize,
                'scaledSize' => $scaledSize,
            )
        );
    }

    public function listAction(Request $request, $id)
    {
        if (!$this->getOfflineCourseService()->hasCourseManageRole()) {
            return $this->createMessageResponse('error', '没有管理活动的权限');
        }

        $offlineCourse = $this->getOfflineCourseService()->getOfflineCourse($id);
        $conditions = $request->query->all();
        $conditions = $this->prepareConditions($conditions);
        $conditions['courseId'] = $id;
        $conditions['joinStatus'] = 'join';

        $memberNum = $this->getOfflineCourseMemberService()->countMembers($conditions);

        $paginator = new Paginator(
            $request,
            $memberNum,
            20
        );

        $members = $this->getOfflineCourseMemberService()->searchMembers(
            $conditions,
            array('createdTime' => 'DESC'),
            $paginator->getOffsetCount(),
            $paginator->getPerPageCount()
        );

        if (empty($users)) {
            $users = $this->getUserService()->searchUsers(
                array('userIds' => ArrayToolkit::column($members, 'userId')),
                array('id' => 'ASC'),
                0, PHP_INT_MAX
            );
            $users = ArrayToolkit::index($users, 'id');
        }

        $members = $this->buildMemberOrgs($members);

        return $this->render(
            'offline-course-manage/list.html.twig',
            array(
                'members' => $members,
                'offlineCourse' => $offlineCourse,
                'users' => $users,
                'paginator' => $paginator,
            )
        );
    }

    public function addMemberAction(Request $request, $id)
    {
        if (!$this->getOfflineCourseService()->hasCourseManageRole()) {
            return $this->createMessageResponse('error', '没有管理活动的权限');
        }

        $offlineCourse = $this->getOfflineCourseService()->getOfflineCourse($id);

        if ($request->getMethod() == 'POST') {
            $fields = $request->request->all();

            $user = $this->getUserService()->getUserByLoginField($fields['queryfield']);

            $member = $this->getOfflineCourseMemberService()->becomeMemberByImport($id, $user['id']);

            $result = false;

            if ($member) {
                $result = true;
            }

            return $this->createJsonResponse(array('success' => $result));
        }

        return $this->render(
            'offline-course-manage/add-member-modal.html.twig',
            array(
                'offlineCourse' => $offlineCourse,
            )
        );
    }

    public function attendMemberAction(Request $request, $id)
    {
        if (!$this->getOfflineCourseService()->hasCourseManageRole()) {
            return $this->createMessageResponse('error', '没有管理活动的权限');
        }

        $member = $this->getOfflineCourseMemberService()->getMember($id);

        if ($request->getMethod() == 'POST') {
            $fields = $request->request->all();
            $member = $this->getOfflineCourseMemberService()->attendMember($id, $fields['attendedStatus']);

            return $this->render(
                'offline-course-manage/list-tr.html.twig',
                array(
                    'member' => $member,
                )
            );
        }

        return $this->render(
            'offline-course-manage/attend-member-modal.html.twig',
            array(
                'member' => $member,
            )
        );
    }

    public function gradeMemberAction(Request $request, $id)
    {
        if (!$this->getOfflineCourseService()->hasCourseManageRole()) {
            return $this->createMessageResponse('error', '没有管理活动的权限');
        }

        $member = $this->getOfflineCourseMemberService()->getMember($id);

        if ($request->getMethod() == 'POST') {
            $fields = $request->request->all();
            $member = $this->getOfflineCourseMemberService()->gradeMember($id, $fields);

            return $this->render(
                'offline-course-manage/list-tr.html.twig',
                array(
                    'member' => $member,
                )
            );
        }

        return $this->render(
            'offline-course-manage/grade-member-modal.html.twig',
            array(
                'member' => $member,
            )
        );
    }

    public function deleteMemberAction(Request $request, $id)
    {
        if (!$this->getOfflineCourseService()->hasCourseManageRole()) {
            return $this->createMessageResponse('error', '没有管理活动的权限');
        }

        $result = $this->getOfflineCourseMemberService()->deleteMember($id);

        return $this->createJsonResponse($result);
    }

    public function checkMemberAction(Request $request, $id)
    {
        if (!$this->getOfflineCourseService()->hasCourseManageRole()) {
            return $this->createMessageResponse('error', '没有管理活动的权限');
        }

        $keyWord = $request->query->get('value');
        $user = $this->getUserService()->getUserByLoginField($keyWord);

        if (empty($user)) {
            $result = '用户不存在！';
        } else {
            $result = $this->getOfflineCourseMemberService()->isMember($id, $user['id']);

            if ($result) {
                $result = '用户已报名该线下活动！';
            } else {
                $result = !$result;
            }
        }

        return $this->createJsonResponse($result);
    }

    public function statisticsAction(Request $request, $id)
    {
        if (!$this->getOfflineCourseService()->hasCourseManageRole()) {
            return $this->createMessageResponse('error', '没有管理活动的权限');
        }

        $offlineCourse = $this->getOfflineCourseService()->getOfflineCourse($id);

        return $this->render(
            'offline-course-manage/statistics.html.twig',
            array(
                'offlineCourse' => $offlineCourse,
            )
        );
    }

    public function attendStatisticAction(Request $request, $id)
    {
        if (!$this->getOfflineCourseService()->hasCourseManageRole()) {
            return $this->createMessageResponse('error', '没有管理活动的权限');
        }

        $statistic = $this->getOfflineCourseMemberService()->statisticMembersAttendStatusByCourseId($id);

        if (empty($statistic)) {
            $result = array(
                array('name' => '未考勤', 'value' => 0),
            );

            return $this->createJsonResponse($result);
        }

        $replace = array(
            'none' => '未考勤',
            'attended' => '正常出勤',
            'unattended' => '缺勤',
        );

        $result = $this->buildStatistic($replace, ArrayToolkit::index($statistic, 'attendedStatus'));

        return $this->createJsonResponse($result);
    }

    public function passStatisticAction(Request $request, $id)
    {
        if (!$this->getOfflineCourseService()->hasCourseManageRole()) {
            return $this->createJsonResponse('error', '没有管理活动的权限');
        }

        $statistic = $this->getOfflineCourseMemberService()->statisticMemberPassStatusByCourseId($id);

        if (empty($statistic)) {
            $result = array(
                array('name' => '未考核', 'value' => 0),
            );

            return $this->createJsonResponse($result);
        }

        $replace = array(
            'none' => '未考核',
            'passed' => '通过',
            'unpassed' => '未通过',
        );

        $result = $this->buildStatistic($replace, ArrayToolkit::index($statistic, 'passedStatus'));

        return $this->createJsonResponse($result);
    }

    public function scoreStatisticAction(Request $request, $id)
    {
        if (!$this->getOfflineCourseService()->hasCourseManageRole()) {
            return $this->createMessageResponse('error', '没有管理活动的权限');
        }

        $statistic = $this->getOfflineCourseMemberService()->statisticMemberScoreByCourseId($id);

        $result = array();

        foreach ($statistic[0] as $key => $data) {
            $result[] = array(
                'scoreRange' => $key,
                'num' => empty($data) ? 0 : $data,
            );
        }

        return $this->createJsonResponse($result);
    }

    protected function buildStatistic($replace, $originData)
    {
        $result = array();

        foreach ($replace as $key => $value) {
            $result[] = array(
                'name' => $value,
                'value' => empty($originData[$key]['count']) ? 0 : (int) $originData[$key]['count'],
            );
        }

        return $result;
    }

    public function publishAction(Request $request, $id)
    {
        if (!$this->getOfflineCourseService()->hasCoursePublishRole()) {
            return $this->createMessageResponse('error', '没有发布活动的权限');
        }

        $offlineCourse = $this->getOfflineCourseService()->getOfflineCourse($id);
        if ($offlineCourse['startDate']) {
            $this->getOfflineCourseService()->publishOfflineCourse($id);
            $success = true;
            $message = $this->renderCourseTr($id, $request)->getContent();
        } else {
            $success = false;
            $message = '活动信息未填写';
        }

        return $this->createJsonResponse(array(
            'success' => $success,
            'message' => $message,
        ));
    }

    public function closeAction(Request $request, $id)
    {
        if (!$this->getOfflineCourseService()->hasCourseCloseRole()) {
            return $this->createMessageResponse('error', '没有关闭活动的权限');
        }

        $this->getOfflineCourseService()->closeOfflineCourse($id);

        return $this->renderCourseTr($id, $request);
    }

    protected function renderCourseTr($courseId, $request)
    {
        $offlineCourse = $this->getOfflineCourseService()->getOfflineCourse($courseId);

        return $this->render('admin/offline-course/offline-course-table-tr.html.twig', array(
            'offlineCourse' => $offlineCourse,
        ));
    }

    protected function prepareConditions($conditions)
    {
        if (isset($conditions['keyword'])) {
            $conditions['keyword'] = trim($conditions['keyword']);
        }
        if (isset($conditions['keywordType']) && !empty($conditions['keyword'])) {
            $conditions[$conditions['keywordType']] = $conditions['keyword'];
            unset($conditions['keywordType']);
            unset($conditions['keyword']);
        }

        if (!empty($conditions)) {
            $users = $this->getUserService()->searchUsers(
                $conditions,
                array('id' => 'ASC'),
                0, PHP_INT_MAX
            );

            $users = ArrayToolkit::index($users, 'id');
            $userIds = ArrayToolkit::column($users, 'id');
            $conditions = array(
                'userIds' => empty($userIds) ? array(-1) : $userIds,
            );
        }

        return $conditions;
    }

    public function verifyListAction(Request $request, $id)
    {
        if (!$this->getOfflineCourseService()->hasCourseManageRole()) {
            return $this->createMessageResponse('error', '没有管理活动的权限');
        }

        $offlineCourse = $this->getOfflineCourseService()->getOfflineCourse($id);
        $conditions = $request->query->all();
        $conditions = $this->prepareConditions($conditions);
        $conditions['status'] = $request->query->get('verifyStatus', 'submitted');
        if ($conditions['status'] == 'all') {
            $conditions['status'] = '';
        }
        $conditions['offlineCourseId'] = $id;
        $recordCounts = $this->getEnrollmentRecordService()->countEnrollmentRecords($conditions);
        $paginator = new Paginator(
            $request,
            $recordCounts,
            20
        );

        $records = $this->getEnrollmentRecordService()->searchEnrollmentRecords(
            $conditions,
            array('submittedTime' => 'DESC'),
            $paginator->getOffsetCount(),
            $paginator->getPerPageCount()
        );

        $userIds = ArrayToolkit::column($records, 'userId');
        $users = $this->getUserService()->findUsersByIds($userIds);
        $profiles = $this->getUserService()->findUserProfilesByIds($userIds);

        return $this->render(
            '@CorporateTraining/offline-course-manage/verify-list.html.twig', array(
                'offlineCourse' => $offlineCourse,
                'records' => $records,
                'users' => $users,
                'profiles' => $profiles,
                'paginator' => $paginator,
            )
        );
    }

    public function verifyApplyAction(Request $request, $id)
    {
        if (!$this->getOfflineCourseService()->hasCourseManageRole()) {
            return $this->createMessageResponse('error', '没有管理活动的权限');
        }

        $record = $this->getEnrollmentRecordService()->getEnrollmentRecord($id);

        $offlineCourse = $this->getOfflineCourseService()->getOfflineCourse($record['offlineCourseId']);

        if ('POST' === $request->getMethod()) {
            $fields = $request->request->all();
            if ($fields['verifyStatus'] == 'rejected') {
                $this->getEnrollmentRecordService()->rejectOfflineCourseApply($record['id'], $fields);
            } elseif ($fields['verifyStatus'] == 'approved') {
                $this->getEnrollmentRecordService()->passOfflineCourseApply($record['id']);
            }

            return $this->createJsonResponse(true);
        }

        return $this->render('@CorporateTraining/offline-course-manage/verify-enrollment-apply.html.twig', array(
            'record' => $record,
            'OfflineCourse' => $offlineCourse,
        ));
    }

    public function learnRecordAction(Request $request, $id)
    {
        if (!$this->getOfflineCourseService()->hasCourseManageRole()) {
            return $this->createMessageResponse('error', '没有管理活动的权限');
        }

        $record = $this->getEnrollmentRecordService()->getEnrollmentRecord($id);
        $user = $this->getUserService()->getUser($record['userId']);
        $post = $this->getPostService()->getPost($user['postId']);
        $userInfo = array(
            'id' => $user['id'],
            'name' => $user['nickname'],
            'post' => $post['name'],
            'orgIds' => $user['orgIds'],
        );

        $postCourseDatas = $this->calculatePostCoursesDatas($post['id'], $user['id']);

        return $this->render('@CorporateTraining/offline-course-manage/record-enrollment-apply.html.twig', array(
            'userInfo' => $userInfo,
            'postCourseCount' => $postCourseDatas['postCourseCount'],
            'totalLearnTime' => $postCourseDatas['totalLearnTime'],
            'learnedCoursesNum' => $postCourseDatas['learnedCoursesNum'],
            'tab_types' => $request->get('tab_types', 'assignmentRecord'),
        ));
    }

    public function calculatePostCoursesDatas($postId, $userId)
    {
        $postCourses = $this->getPostCourseService()->findPostCoursesByPostId($postId);

        if (!empty($postCourses)) {
            $courseIds = ArrayToolkit::column($postCourses, 'courseId');
            $courses = $this->getCourseService()->findCoursesByIds($courseIds);
            $totalLearnTime = $this->calculatePostCoursesLearnTime($userId, $courses);
            $learnedCoursesNum = $this->getPostCourseService()->countUserLearnedPostCourses($postId, $courseIds);
        }

        $datas = array(
            'postCourseCount' => empty($postCourses) ? 0 : count($postCourses),
            'totalLearnTime' => empty($totalLearnTime) ? 0 : $totalLearnTime,
            'learnedCoursesNum' => empty($learnedCoursesNum) ? 0 : $learnedCoursesNum,
        );

        return $datas;
    }

    public function calculatePostCoursesLearnTime($userId, $courses)
    {
        foreach ($courses as $key => &$course) {
            $course['learnTime'] = $this->getTaskResultService()->sumLearnTimeByCourseIdAndUserId($key, $userId);
        }
        $learnTime = ArrayToolkit::column($courses, 'learnTime');
        $totalLearnTime = array_sum($learnTime);

        return $totalLearnTime;
    }

    public function batchAuditAction(Request $request)
    {
        return $this->render(
            'offline-course-manage/batch-audit-modal.html.twig'
        );
    }

    public function batchPassAction(Request $request)
    {
        if ($request->getMethod() == 'POST') {
            $recordIds = $request->request->get('recordIds');
            $this->getEnrollmentRecordService()->batchPass(explode(',', $recordIds));

            return $this->createJsonResponse(true);
        }
    }

    public function batchRejectAction(Request $request)
    {
        if ($request->getMethod() == 'POST') {
            $recordIds = $request->request->get('recordIds');
            $this->getEnrollmentRecordService()->batchReject(explode(',', $recordIds));

            return $this->createJsonResponse(true);
        }
    }

    protected function conversionTime($data, $offlineCourse)
    {
        if (array_key_exists('startDate', $data)) {
            $data['startDate'] = strtotime($data['startDate']);
        } else {
            $data['startDate'] = $offlineCourse['startDate'];
        }

        if (array_key_exists('endDate', $data)) {
            $data['endDate'] = strtotime($data['endDate']);
        } else {
            $data['endDate'] = $offlineCourse['endDate'];
        }

        if (array_key_exists('enrollmentEndDate', $data)) {
            $data['enrollmentEndDate'] = strtotime($data['enrollmentEndDate']);
        } else {
            $data['enrollmentEndDate'] = $offlineCourse['enrollmentEndDate'];
        }

        return $data;
    }

    protected function decideStartDate($startDate)
    {
        if ($startDate && time() >= $startDate) {
            $status = true;
        } else {
            $status = false;
        }

        return $status;
    }

    public function viewSignQrcodeAction(Request $request, $courseId)
    {
        if (!$this->getOfflineCourseService()->hasCourseSignInRole()) {
            return $this->createMessageResponse('error', '没有获取签到二维码的权限');
        }

        $user = $this->getCurrentUser();
        $offlineCourse = $this->getOfflineCourseService()->getOfflineCourse($courseId);
        if (empty($offlineCourse)) {
            return $this->createMessageResponse('error', '活动不存在');
        }
        $qrcodeImgUrl = $this->qrcode($courseId, $user['id']);

        return $this->render('offline-course/sign-in/qr-code.html.twig',
            array(
                'offlineCourse' => $offlineCourse,
                'qrcodeImgUrl' => $qrcodeImgUrl,
            )
        );
    }

    public function evaluateShowAction(Request $request, $memberId)
    {
        if (!$this->getOfflineCourseService()->hasCourseManageRole()) {
            return $this->createMessageResponse('error', '没有查看评价的权限');
        }

        $member = $this->getOfflineCourseMemberService()->getMember($memberId);

        return $this->render('offline-course-manage/evaluate-modal.html.twig',
            array('member' => $member)
        );
    }

    protected function qrcode($courseId, $userId)
    {
        $token = $this->getTokenService()->makeToken('qrcode', array(
            'data' => array(
                'url' => $this->generateUrl('offline_course_sign_in', array('courseId' => $courseId), true),
            ),
            'userId' => $userId,
            'duration' => 3600 * 24 * 365,
        ));

        $url = $this->generateUrl('common_parse_qrcode', array('token' => $token['token']), true);

        return $this->generateUrl('common_qrcode', array('text' => $url), true);
    }

    protected function buildMemberOrgs($members)
    {
        foreach ($members as $key => $member) {
            $user = $this->getUserService()->getUser($member['userId']);
            $orgs = $this->getOrgService()->findOrgsByIds($user['orgIds']);
            $members[$key]['orgsName'] = OrgToolkit::buildOrgsNames($user['orgIds'], $orgs);
            $members[$key]['org'] = empty($orgs[0]) ? array() : $orgs[0];
        }

        return $members;
    }

    /**
     * @return TokenService
     */
    protected function getTokenService()
    {
        return $this->createService('User:TokenService');
    }

    /**
     * @return OfflineCourseService
     */
    protected function getOfflineCourseService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:OfflineCourseService');
    }

    /**
     * @return MemberService
     */
    protected function getOfflineCourseMemberService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:MemberService');
    }

    /**
     * @return EnrollmentRecordService
     */
    protected function getEnrollmentRecordService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:EnrollmentRecordService');
    }

    /**
     * @return FileService
     */
    protected function getFileService()
    {
        return $this->createService('Content:FileService');
    }

    /**
     * @return PostService
     */
    protected function getPostService()
    {
        return $this->createService('CorporateTrainingBundle:Post:PostService');
    }

    /**
     * @return UserService
     */
    protected function getUserService()
    {
        return $this->createService('CorporateTrainingBundle:User:UserService');
    }

    /**
     * @return OrgService
     */
    protected function getOrgService()
    {
        return $this->createService('Org:OrgService');
    }

    protected function getCategoryService()
    {
        return $this->createService('Taxonomy:CategoryService');
    }

    protected function getCourseService()
    {
        return $this->createService('Course:CourseService');
    }

    protected function getCourseSetService()
    {
        return $this->createService('Course:CourseSetService');
    }

    protected function getTaskResultService()
    {
        return $this->createService('Task:TaskResultService');
    }

    protected function getPostCourseService()
    {
        return $this->createService('CorporateTrainingBundle:PostCourse:PostCourseService');
    }

    protected function getTrainingAssignmentService()
    {
        return $this->createService('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentService');
    }

    protected function getTrainingAssignmentMemberService()
    {
        return $this->createService('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentMemberService');
    }

    protected function createAssignmentStrategy()
    {
        return $this->getBiz()->offsetGet('assignment_strategy_context')->createStrategy();
    }
}
