<?php

namespace CorporateTrainingBundle\Controller;

use AppBundle\Common\ArrayToolkit;
use Symfony\Component\HttpFoundation\Request;
use AppBundle\Common\Paginator;
use AppBundle\Controller\SearchController as BaseController;

class SearchController extends BaseController
{
    public function indexAction(Request $request)
    {
        $paginator = null;

        $currentUser = $this->getCurrentUser();

        $keywords = $request->query->get('q');
        $keywords = $this->filterKeyWord(trim($keywords));

        $cloud_search_setting = $this->getSettingService()->get('cloud_search', array());

        if (isset($cloud_search_setting['search_enabled']) && $cloud_search_setting['search_enabled'] && $cloud_search_setting['status'] == 'ok') {
            return $this->redirect(
                $this->generateUrl(
                    'cloud_search',
                    array(
                        'q' => $keywords,
                        'type' => $request->query->get('type'),
                    )
                )
            );
        }

        $vip = $this->getAppService()->findInstallApp('Vip');

        $isShowVipSearch = $vip && version_compare($vip['version'], '1.0.7', '>=');

        $currentUserVipLevel = '';
        $vipLevelIds = '';

        if ($isShowVipSearch) {
            $currentUserVip = $this->getVipService()->getMemberByUserId($currentUser['id']);
            if (!empty($currentUserVip) && isset($currentUserVip['levelId'])) {
                $currentUserVipLevel = $this->getLevelService()->getLevel($currentUserVip['levelId']);
                $vipLevels = $this->getLevelService()->findAllLevelsLessThanSeq($currentUserVipLevel['seq']);
                $vipLevelIds = ArrayToolkit::column($vipLevels, 'id');
            }
        }

        $parentId = 0;
        $categories = $this->getCategoryService()->findAllCategoriesByParentId($parentId);

        $categoryIds = array();

        foreach ($categories as $key => $category) {
            $categoryIds[$key] = $category['name'];
        }

        $categoryId = $request->query->get('categoryIds');
        $filter = $request->query->get('filter');

        $conditions = array(
            'status' => 'published',
            'title' => $keywords,
            'categoryId' => $categoryId,
            'parentId' => 0,
        );

        if ($filter == 'vip') {
            $conditions['vipLevelIds'] = $vipLevelIds;
        } elseif ($filter == 'live') {
            $conditions['type'] = 'live';
        } elseif ($filter == 'free') {
            $conditions['minCoursePrice'] = '0.00';
        }

        $count = $this->getCourseSetService()->countCourseSets($conditions);
        $paginator = new Paginator(
            $this->get('request'),
            $count, 12
        );

        $orgIds = $currentUser->getOrgIds();
        $rootOrgIds = array(1);
        $conditions['orgIds'] = array_merge($rootOrgIds, $orgIds);

        $courseSets = $this->getCourseSetService()->searchCourseSets(
            $conditions,
            array('updatedTime' => 'desc'),
            $paginator->getOffsetCount(),
            $paginator->getPerPageCount()
        );

        return $this->render(
            'search/index.html.twig',
            array(
                'courseSets' => $courseSets,
                'paginator' => $paginator,
                'keywords' => $keywords,
                'isShowVipSearch' => $isShowVipSearch,
                'currentUserVipLevel' => $currentUserVipLevel,
                'categoryIds' => $categoryIds,
                'filter' => $filter,
                'count' => count($courseSets),
            )
        );
    }

    private function filterKeyWord($keyword)
    {
        $keyword = str_replace('<', '', $keyword);
        $keyword = str_replace('>', '', $keyword);
        $keyword = str_replace("'", '', $keyword);
        $keyword = str_replace('"', '', $keyword);
        $keyword = str_replace('=', '', $keyword);
        $keyword = str_replace('&', '', $keyword);
        $keyword = str_replace('/', '', $keyword);

        return $keyword;
    }
}
