<?php

namespace CorporateTrainingBundle\Controller\StudyCenter;

use AppBundle\Common\Paginator;
use AppBundle\Common\ArrayToolkit;
use Symfony\Component\HttpFoundation\Request;
use AppBundle\Controller\BaseController;

class CourseStatisticsController extends BaseController
{
    public function statisticsByCategoriesAction(Request $request, $userId, $source = null)
    {
        $courses = $this->getCourseService()->findUserLearnCourses($userId, 0, PHP_INT_MAX);
        $courses = $this->getCourseSetService()->findCourseSetsByCourseIds(ArrayToolkit::column($courses, 'courseId'));

        $coursesGroupByCategory = ArrayToolkit::group($courses, 'categoryId');

        $topFiveCategorySummaryDatas = $this->getTopFiveCategorySummaryDatas($userId, $coursesGroupByCategory);

        $view = 'study-center/course-statistics/statistics-by-categories.html.twig';
        if ($source == 'user_learn') {
            $view = 'data-report/course-statistics/statistics-by-categories.html.twig';
        } elseif ($source == 'offline_course') {
            $view = 'offline-course-manage/study-record/statistics-by-categories.html.twig';
        }

        return $this->render($view, array(
            'topFiveCategorySummaryDatas' => $topFiveCategorySummaryDatas,
            'userId' => $userId,
            'tab_types' => $request->get('tab_types', 'learningStatistics'),
            )
        );
    }

    public function detailAction(Request $request, $userId = null)
    {
        if (!empty($userId)) {
            $user = $this->getUserService()->getUser($userId);
        } else {
            $user = $this->getCurrentUser();
        }

        $paginator = new Paginator(
            $request,
            $this->getCourseService()->countUserLearnCourse($user['id']),
            10
        );

        $courses = $this->getCourseService()->findUserLearnCourses(
            $user['id'],
            $paginator->getOffsetCount(),
            $paginator->getPerPageCount()
        );

        $courses = $this->calculateWatchTimeAndLearnTimeAndMember($user['id'], $courses);

        return $this->render(
            'study-center/course-statistics/course-score-detail.html.twig',
            array(
            'courses' => empty($courses) ? array() : $courses,
            'paginator' => $paginator,
            'user' => $user,
            )
        );
    }

    public function detailShowAction(Request $request, $courseId, $userId = null)
    {
        if (!empty($userId)) {
            $user = $this->getUserService()->getUser($userId);
        } else {
            $user = $this->getCurrentUser();
        }

        list($testPapers, $testPaperResults) = $this->findTestPapersAndResults($courseId, $user['id']);

        return $this->render(
            'study-center/course-statistics/course-score-detail-modal.html.twig',
            array(
                'testPaperResults' => empty($testPaperResults) ? array() : $testPaperResults,
                'testPapers' => ArrayToolkit::index($testPapers, 'id'),
                'user' => $user,
            )
        );
    }

    private function getTopFiveCategorySummaryDatas($userId, $coursesGroupByCategory)
    {
        $coursesGroupByCategoryCount = array();
        foreach ($coursesGroupByCategory as $key => $courses) {
            $coursesGroupByCategoryCount[$key] = count($courses);
        }
        arsort($coursesGroupByCategoryCount);

        $count = 1;
        $topFiveCategoryDatas = array();
        foreach ($coursesGroupByCategoryCount as $key => $value) {
            if ($count++ > 5) {
                break;
            }
            $topFiveCategoryDatas[$key]['courseNum'] = $value;
            $categoryIds = ArrayToolkit::column($coursesGroupByCategory[$key], 'categoryId');
            $courseIds = ArrayToolkit::column($coursesGroupByCategory[$key], 'id');
            $conditions['courseIds'] = !empty($courseIds) ? $courseIds : array(-1);
            $conditions['userId'] = $userId;
            $coursesWatchTime = $this->getTaskResultService()->sumWatchTimeByCategoryIdAndUserId($categoryIds[0], $userId);
            $topFiveCategoryDatas[$key]['watchTime'] = $coursesWatchTime;
            $coursesLearnTime = $this->getTaskResultService()->sumLearnTimeByCategoryIdAndUserId($categoryIds[0], $userId);
            $topFiveCategoryDatas[$key]['learnTime'] = $coursesLearnTime;
        }

        return $topFiveCategoryDatas;
    }

    private function calculateWatchTimeAndLearnTimeAndMember($userId, &$courses)
    {
        foreach ($courses as &$course) {
            $course['watchTime'] = $this->getTaskResultService()->sumWatchTimeByCourseIdAndUserId($course['courseId'], $userId);

            $course['learnTime'] = $this->getTaskResultService()->sumLearnTimeByCourseIdAndUserId($course['courseId'], $userId);

            $finishedTimes = $this->getTaskResultService()->findFinishedTimeByCourseIdGroupByUserId($course['courseId']);
            if (!empty($finishedTimes)) {
                foreach ($finishedTimes as &$finishedTime) {
                    if ($finishedTime['userId'] == $userId) {
                        $course['finishedTime'] = $finishedTime['finishedTime'];
                        break;
                    }
                }
            }
        }

        return $courses;
    }

    private function findTestPapersAndResults($courseId, $userId)
    {
        $course = $this->getCourseService()->getCourse($courseId);
        $testPapers = $this->getTestpaperService()->searchTestpapers(
            array('courseSetId' => $course['courseSetId'], 'status' => 'open', 'type' => 'testpaper'),
            array('createdTime' => 'DESC'),
            0, PHP_INT_MAX
        );

        $testPaperResults = array();
        if (!empty($testPapers)) {
            $testPaperResults = $this->getTestpaperService()->searchTestpaperResults(
                array('testIds' => ArrayToolkit::column($testPapers, 'id'), 'userId' => $userId),
                array('id' => 'DESC'),
                0, PHP_INT_MAX
            );
        }

        return array($testPapers, $testPaperResults);
    }

    protected function getCourseService()
    {
        return $this->createService('Course:CourseService');
    }

    protected function getCourseSetService()
    {
        return $this->createService('Course:CourseSetService');
    }

    protected function getMemberService()
    {
        return $this->createService('Course:MemberService');
    }

    protected function getActivityLearnLogService()
    {
        return $this->createService('Activity:ActivityLearnLogService');
    }

    protected function getTestpaperService()
    {
        return $this->createService('Testpaper:TestpaperService');
    }

    protected function getTaskResultService()
    {
        return $this->createService('Task:TaskResultService');
    }
}
