<?php

namespace CorporateTrainingBundle\Controller\StudyCenter;

use AppBundle\Common\Paginator;
use Symfony\Component\HttpFoundation\Request;
use AppBundle\Controller\BaseController;

class DepartmentController extends BaseController
{
    public function coursesAction(Request $request)
    {
        $user = $this->getCurrentUser();
        $conditions = $request->query->all();

        $courseSetting = $this->getSettingService()->get('course', array());

        if (!isset($courseSetting['explore_default_orderBy'])) {
            $courseSetting['explore_default_orderBy'] = 'latest';
        }

        $orderBy = $courseSetting['explore_default_orderBy'] == 'recommendedSeq' ? 'studentNum' : $courseSetting['explore_default_orderBy'];
        $orderBy = empty($conditions['orderBy']) ? $orderBy : $conditions['orderBy'];
        unset($conditions['orderBy']);

        $conditions['parentId'] = 0;
        $conditions['status'] = 'published';
        $conditions['orgIds'] = $this->getOrgIds($user['orgIds']);

        $paginator = new Paginator(
            $this->get('request'),
            $this->getCourseSetService()->countCourseSets($conditions),
            9
        );

        $courseSets = $this->getCourseSetService()->searchCourseSets(
            $conditions,
            $orderBy,
            $paginator->getOffsetCount(),
            $paginator->getPerPageCount()
        );

        return $this->render(
            'study-center/department-courses/courses.html.twig',
            array(
            'user' => $user,
            'courseSets' => $courseSets,
            'paginator' => $paginator,
            'orderBy' => $orderBy ?: null,
            )
        );
    }

    public function classroomsAction(Request $request)
    {
        $user = $this->getCurrentUser();

        $conditions = $request->query->all();
        $conditions['status'] = 'published';
        $conditions['showable'] = 1;

        $classroomSetting = $this->getSettingService()->get('classroom');

        if (!isset($classroomSetting['explore_default_orderBy'])) {
            $classroomSetting['explore_default_orderBy'] = 'createdTime';
        }

        $orderBy = empty($conditions['orderBy']) ? $classroomSetting['explore_default_orderBy'] : $conditions['orderBy'];

        if ($orderBy == 'recommendedSeq') {
            $conditions['recommended'] = 1;
            $orderBys = array($orderBy => 'asc');
        } else {
            $orderBys = array($orderBy => 'desc');
        }

        unset($conditions['orderBy']);
        $conditions['orgIds'] = $this->getOrgIds($user['orgIds']);

        $paginator = new Paginator(
            $this->get('request'),
            $this->getClassroomService()->countClassrooms($conditions),
            9
        );

        $classrooms = $this->getClassroomService()->searchClassrooms(
            $conditions,
            $orderBys,
            $paginator->getOffsetCount(),
            $paginator->getPerPageCount()
        );

        return $this->render(
            'study-center/department-courses/classrooms.html.twig',
            array(
            'user' => $user,
            'classrooms' => $classrooms,
            'paginator' => $paginator,
            'orderBy' => $orderBy ?: null,
            )
        );
    }

    protected function getOrgIds($orgIds)
    {
        if (count($orgIds) > 1 && in_array('1.', $orgIds)) {
            foreach ($orgIds as $key => $orgId) {
                if ($orgId == '1.') {
                    unset($orgIds[$key]);
                    break;
                }
            }
        }

        return $orgIds;
    }

    protected function getCourseService()
    {
        return $this->createService('Course:CourseService');
    }

    protected function getCourseSetService()
    {
        return $this->createService('Course:CourseSetService');
    }

    protected function getOrgService()
    {
        return $this->createService('Org:OrgService');
    }

    protected function getSettingService()
    {
        return $this->createService('System:SettingService');
    }

    protected function getClassroomService()
    {
        return $this->createService('Classroom:ClassroomService');
    }
}
