<?php

namespace CorporateTrainingBundle\Controller\StudyCenter;

use AppBundle\Common\Paginator;
use AppBundle\Common\ArrayToolkit;
use Symfony\Component\HttpFoundation\Request;
use AppBundle\Controller\BaseController;

class MyCoursesController extends BaseController
{
    public function learningAction(Request $request)
    {
        $user = $this->getCurrentUser();

        $paginator = new Paginator(
            $request,
            $this->getCourseService()->countUserLearningCourses($user['id']),
            12
        );

        $courses = $this->getCourseService()->findUserLearningCourses(
            $user['id'],
            $paginator->getOffsetCount(),
            $paginator->getPerPageCount()
        );

        $setIds = ArrayToolkit::column($courses, 'courseSetId');
        $courseSets = $this->getCourseSetService()->findCourseSetsByIds($setIds);
        $courseSets = ArrayToolkit::index($courseSets, 'id');

        return $this->render(
            'study-center/my-courses/learning.html.twig',
            array(
                'courses' => $courses,
                'courseSets' => $courseSets,
                'paginator' => $paginator,
            )
        );
    }

    public function learnedAction(Request $request)
    {
        $currentUser = $this->getCurrentUser();
        $paginator = new Paginator(
            $this->get('request'),
            $this->getCourseService()->countUserLearnedCourses($currentUser['id']),
            12
        );

        $courses = $this->getCourseService()->findUserLearnedCourses(
            $currentUser['id'],
            $paginator->getOffsetCount(),
            $paginator->getPerPageCount()
        );

        $userIds = array();

        foreach ($courses as $key => $course) {
            $userIds = array_merge($userIds, $course['teacherIds']);
            $learnTime = $this->getTaskResultService()->sumLearnTimeByCourseIdAndUserId(
                $course['id'],
                $currentUser['id']
            );

            $courses[$key]['learnTime'] = $learnTime;
        }
        $users = $this->getUserService()->findUsersByIds($userIds);

        return $this->render(
            'study-center/my-courses/learned.html.twig',
            array(
                'courses' => $courses,
                'users' => $users,
                'paginator' => $paginator,
            )
        );
    }

    protected function getTaskResultService()
    {
        return $this->createService('Task:TaskResultService');
    }

    protected function getUserService()
    {
        return $this->createService('User:UserService');
    }

    protected function getCourseService()
    {
        return $this->createService('Course:CourseService');
    }

    protected function getCourseSetService()
    {
        return $this->createService('Course:CourseSetService');
    }

    protected function getPostService()
    {
        return $this->createService('CorporateTrainingBundle:Post:PostService');
    }

    protected function getPostCourseService()
    {
        return $this->createService('CorporateTrainingBundle:PostCourse:PostCourseService');
    }

    protected function getThreadService()
    {
        return $this->createService('Course:ThreadService');
    }

    protected function getOrgService()
    {
        return $this->createService('Org:OrgService');
    }
}
