<?php

namespace CorporateTrainingBundle\Controller\StudyCenter;

use AppBundle\Common\ArrayToolkit;
use Symfony\Component\HttpFoundation\Request;
use AppBundle\Controller\BaseController;

class MyTaskController extends BaseController
{
    public function myTaskHeaderAction(Request $request, $tab_nav)
    {
        $user = $this->getCurrentUser();
        $postCourses = $this->getPostCourseService()->findPostCoursesByPostId($user['postId']);

        if (!empty($postCourses)) {
            $courseIds = ArrayToolkit::column($postCourses, 'courseId');
            $learnedCoursesNum = $this->getPostCourseService()->countUserLearnedPostCourses($user['id'], $courseIds);
            $unfinishedPostCoursesNum = count($this->getPostCourseService()->findUserLearningPostCourses($user['id'], $courseIds));
            $learnTime = $this->sumLearnTime($user['id'], $user['postId']);
            $recentStartTask = $this->findRecentStartTask($user['id'], $courseIds);
        }

        $assignmentIds = $this->getTrainingAssignmentMemberService()->searchTrainingAssignmentMembers(
            array('userId' => $user['id'], 'statuses' => array('notStart', 'ongoing')),
            array(),
            0,
            PHP_INT_MAX
        );
        $trainingIds = ArrayToolkit::column($assignmentIds, 'assignmentId');
        if ($trainingIds) {
            $unfinishedTrainingsNum = $this->getTrainingAssignmentService()->countTrainingAssignments(array('ids' => $trainingIds, 'status' => 'published'));
        }

        return $this->render(
            'study-center/my-task/header.html.twig',
            array(
                'courseIds' => empty($courseIds) ? array() : $courseIds,
                'learnTime' => empty($learnTime) ? 0 : $learnTime,
                'learnedCoursesNum' => empty($learnedCoursesNum) ? 0 : $learnedCoursesNum,
                'recentStartTask' => empty($recentStartTask) ? array() : $recentStartTask,
                'tab_nav' => $tab_nav,
                'trainingNum' => empty($unfinishedTrainingsNum) ? null : $unfinishedTrainingsNum,
                'postNum' => empty($unfinishedPostCoursesNum) ? null : $unfinishedPostCoursesNum,
            )
        );
    }

    public function postCourseListAction(Request $request)
    {
        $user = $this->getCurrentUser();

        $postCourses = $this->getPostCourseService()->searchPostCourses(
            array('postId' => $user['postId']),
            array('seq' => 'ASC'),
            0,
            PHP_INT_MAX
        );

        if (!empty($postCourses)) {
            $courseIds = ArrayToolkit::column($postCourses, 'courseId');
            $courses = $this->getPostCourseService()->findUserLearningPostCourses($user['id'], $courseIds);
            $courses = $this->caculateProgress($user['id'], $courses);
            $courses = $this->caculateLearnTime($user['id'], $courses);
            $courses = $this->sortCourseByPostCourseIds(ArrayToolkit::index($courses, 'id'), $courseIds);
        }

        return $this->render(
            'study-center/my-task/post-courses.html.twig',
            array(
            'courses' => empty($courses) ? array() : $courses,
            'currentTime' => time(),
            )
        );
    }

    public function trainingListAction(Request $request)
    {
        $user = $this->getCurrentUser();
        $assignmentIds = $this->getTrainingAssignmentMemberService()->searchTrainingAssignmentMembers(
            array('userId' => $user['id'], 'statuses' => array('notStart', 'ongoing')),
            array(),
            0,
            PHP_INT_MAX
        );
        $trainingIds = ArrayToolkit::column($assignmentIds, 'assignmentId');

        $trainings = array();

        if ($trainingIds) {
            $assignmentTrainings = $this->getTrainingAssignmentService()->searchTrainingAssignments(
                array('ids' => $trainingIds, 'status' => 'published'),
                array(),
                0,
                20
            );

            $trainings = $this->manageAssignmentTrainings($assignmentTrainings);
        }

        return $this->render('study-center/my-task/trainings.html.twig',
            array(
                'trainings' => empty($trainings) ? array() : $trainings,
                'total' => count($trainingIds),
            )
        );
    }

    public function taskListAction(Request $request, $courseId, $tab_type, $userId)
    {
        $course = $this->getCourseService()->getCourse($courseId);

        return $this->forward('CorporateTrainingBundle:StudyCenter/Task:list', array(
            'course' => $course,
            'tab_type' => $tab_type,
            'userId' => $userId,
        ));
    }

    public function courseListAction(Request $request, $id, $type, $userId)
    {
        $training = $this->getTrainingAssignmentService()->getTrainingAssignment($id);

        $trainingAssignmentItems = $this->getTrainingAssignmentItemService()->searchTrainingAssignmentItems(
            array('assignmentId' => $id),
            array('seq' => 'ASC'),
            0,
            PHP_INT_MAX
        );

        if (empty($trainingAssignmentItems)) {
            $courses = array();
        } else {
            $courseIds = ArrayToolkit::column($trainingAssignmentItems, 'targetId');

            $courses = $this->getCourseSetService()->searchCourseSets(
                array('ids' => $courseIds, 'status' => 'published'),
                array(),
                0,
                PHP_INT_MAX
            );

            $courseIds = ArrayToolkit::column($courses, 'id');

            $courses = $this->getCourseService()->findCoursesByIds($courseIds);

            $courses = $this->caculateProgress($userId, $courses);

            $courses = $this->getToLearnCourse($courses, $userId);
        }

        return $this->render('study-center/widget/product-list.html.twig', array(
            'courses' => $courses,
            'userId' => $userId,
            'training' => $training,
        ));
    }

    public function courseTaskListAction(Request $request, $course, $tab_type, $userId)
    {
        $courseItems = $this->getCourseService()->findCourseItems($course['id']);

        $user = $this->getUserService()->getUser($userId);
        if (empty($user)) {
            $user = $this->getCurrentUser();
        }

        $member = $this->getMemberService()->findMemberUserIdsByCourseId($course['id']);
        if (!in_array($user['id'], $member)) {
            $this->getMemberService()->becomeStudent($course['id'], $user['id']);
        }

        return $this->render(
            'study-center/task-list/list.html.twig',
            array(
                'course' => $course,
                'courseItems' => $courseItems,
                'member' => $member,
                'tab_type' => $tab_type,
                'userId' => $userId,
            )
        );
    }

    public function ajaxGetTrainingRowHtmlAction(Request $request)
    {
        $user = $this->getCurrentUser();
        $assignmentIds = $this->getTrainingAssignmentMemberService()->searchTrainingAssignmentMembers(
            array('userId' => $user['id'], 'statuses' => array('notStart', 'ongoing')),
            array(),
            0,
            PHP_INT_MAX
        );
        $trainingIds = ArrayToolkit::column($assignmentIds, 'assignmentId');
        $conditions = array(
            'ids' => $trainingIds,
            'status' => 'published',
        );

        $start = $request->query->get('start', 0);

        $trainings = $this->getTrainingAssignmentService()->searchTrainingAssignments(
            $conditions,
            array('createdTime' => 'ASC'),
            $start,
            20
        );

        $trainings = $this->manageAssignmentTrainings($trainings);

        return $this->render('study-center/my-task/training-item.html.twig', array(
            'trainings' => $trainings,
            'user' => $this->getCurrentUser(),
        ));
    }

    private function getToLearnCourse($courses, $userId)
    {
        foreach ($courses as &$course) {
            $course['toLearnCourse'] = null;
        }

        foreach ($courses as &$course) {
            $toLearnCourse = $this->findToLearnTaskByCourseId($userId, $course['id']);
            if (!empty($toLearnCourse)) {
                $course['toLearnCourse'] = $toLearnCourse;
                break;
            }
        }

        return $courses;
    }

    private function sumLearnTime($userId, $postId)
    {
        $learnTime = $this->getTaskResultService()->sumLearnTimeByPostIdAndUserId($postId, $userId);

        return $learnTime;
    }

    private function sortCourseByPostCourseIds($courses, $courseIds)
    {
        $coursesOrderByIds = array();
        foreach ($courseIds as $key => $courseId) {
            if (in_array($courseId, array_keys($courses))) {
                array_push($coursesOrderByIds, $courses[$courseId]);
            }
            continue;
        }

        return ArrayToolkit::index($coursesOrderByIds, 'id');
    }

    private function findRecentStartTask($userId, $courseIds)
    {
        $taskResults = $this->getTaskResultService()->searchTaskResults(
            array('userId' => $userId, 'courseIds' => $courseIds),
            array('createdTime' => 'DESC'),
            0, 1
        );

        $taskResult = array_shift($taskResults);

        if (empty($taskResult)) {
            return array();
        }

        $task = $this->getTaskService()->getTask($taskResult['courseTaskId']);

        return $task;
    }

    private function caculateProgress($userId, $courses)
    {
        foreach ($courses as &$course) {
            $course['finishedTaskNum'] = $this->getTaskResultService()->countTaskResults(array(
                'courseId' => $course['id'],
                'userId' => $userId,
                'status' => 'finish',
            ));

            $progress = empty($course['taskNum']) ? 0 : round($course['finishedTaskNum'] / $course['taskNum'], 2) * 100;
            $course['progress'] = $progress > 100 ? 100 : $progress;

            $course['toLearnTask'] = $this->findToLearnTaskByCourseId($userId, $course['id']);
        }

        return $courses;
    }

    private function caculateLearnTime($userId, $courses)
    {
        foreach ($courses as &$course) {
            $course['learnTime'] = $this->getTaskResultService()->sumLearnTimeByCourseIdAndUserId(
                $course['id'],
                $userId
            );
        }

        return $courses;
    }

    private function findToLearnTaskByCourseId($userId, $courseId)
    {
        if ($this->getMemberService()->isCourseMember($courseId, $userId)) {
            $toLearnTasks = $this->getTaskService()->findToLearnTasksByCourseId($courseId);
        } else {
            $toLearnTasks = $this->getTaskService()->searchTasks(
                array('courseId' => $courseId, 'status' => 'published'),
                array('seq' => 'ASC'),
                0, 1
            );
        }

        return empty($toLearnTasks) ? array() : $toLearnTasks[0];
    }

    protected function manageAssignmentTrainings($assignmentTrainings)
    {
        $user = $this->getCurrentUser();
        $currentTime = time();

        $trainings = array();
        foreach ($assignmentTrainings as $assignmentTraining) {
            $courses = $this->getCourseService()->findUserLearnedCourses($user['id'], 0, PHP_INT_MAX);
            $coursesIds = ArrayToolkit::column($courses, 'id');

            $trainingAssignmentItems = $this->getTrainingAssignmentItemService()->findTrainingAssignmentItemsByAssignmentId($assignmentTraining['id']);
            $trainingTargetIds = ArrayToolkit::column($trainingAssignmentItems, 'targetId');
            $unfinishedCourseIds = array_diff($trainingTargetIds, $coursesIds);

            $courses = $this->getCourseSetService()->searchCourseSets(
                array('ids' => $unfinishedCourseIds, 'status' => 'published'),
                array(),
                0,
                PHP_INT_MAX
            );

            $unfinishedCourseIds = ArrayToolkit::column($courses, 'id');

            $assignmentTrainingItem = $this->getTrainingAssignmentItemService()->searchTrainingAssignmentItems(
                array('assignmentId' => $assignmentTraining['id'], 'targetIds' => $unfinishedCourseIds),
                array('seq' => 'ASC'),
                0,
                1
            );

            $assignmentTraining['delay'] = null;
            $assignmentTraining['toLearnTaskByCourseId'] = null;

            if (!empty($assignmentTraining['startTime']) && $assignmentTraining['endTime'] < $currentTime) {
                $assignmentTraining['delay'] = 'delay';
            }
            $assignmentTraining['targetType'] = 'course';
            if ($assignmentTrainingItem) {
                $assignmentTraining['toLearnTaskByCourseId'] = $this->findToLearnTaskByCourseId($user['id'], $assignmentTrainingItem['0']['targetId']);
            }
            $assignmentTraining['progress'] = $this->createAssignmentStrategy($assignmentTraining['id'])->calculateMemberLearnProgress($assignmentTraining, $user['id']);
            array_push($trainings, $assignmentTraining);
        }

        return $trainings;
    }

    protected function getPostCourseService()
    {
        return $this->createService('CorporateTrainingBundle:PostCourse:PostCourseService');
    }

    protected function getCourseService()
    {
        return $this->createService('Course:CourseService');
    }

    protected function getClassroomService()
    {
        return $this->createService('Classroom:ClassroomService');
    }

    protected function getCourseSetService()
    {
        return $this->createService('Course:CourseSetService');
    }

    protected function getMemberService()
    {
        return $this->createService('Course:MemberService');
    }

    protected function getActivityLearnLogService()
    {
        return $this->createService('Activity:ActivityLearnLogService');
    }

    protected function getTaskService()
    {
        return $this->createService('Task:TaskService');
    }

    protected function getTaskResultService()
    {
        return $this->createService('Task:TaskResultService');
    }

    protected function getTrainingAssignmentMemberService()
    {
        return $this->createService('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentMemberService');
    }

    protected function getTrainingAssignmentService()
    {
        return $this->createService('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentService');
    }

    protected function getTrainingAssignmentItemService()
    {
        return $this->createService('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentItemService');
    }

    protected function createAssignmentStrategy($assignmentId)
    {
        $strategy = $this->getBiz()->offsetGet('assignment_strategy_context');

        return $strategy->createStrategy($assignmentId);
    }
}
