<?php

namespace CorporateTrainingBundle\Controller;

use CorporateTrainingBundle\Biz\PostCourse\Service\PostCourseService;
use CorporateTrainingBundle\Biz\PostCourse\Service\UserPostCourseService;
use CorporateTrainingBundle\Biz\TrainingAssignment\Service\TrainingAssignmentMemberService;
use Symfony\Component\HttpFoundation\Request;
use AppBundle\Controller\TaskController as BaseTaskController;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;
use Codeages\Biz\Framework\Service\Exception\AccessDeniedException as ServiceAccessDeniedException;

class TaskController extends BaseTaskController
{
    public function showAction(Request $request, $courseId, $id)
    {
        $preview = $request->query->get('preview');

        $user = $this->getCurrentUser();
        if (!$user->isLogin()) {
            return $this->createMessageResponse('info', '请先登录', '', 3, $this->generateUrl('login'));
        }

        $course = $this->getCourseService()->getCourse($courseId);

        $member = $this->getCourseMemberService()->getCourseMember($courseId, $user['id']);

        /*
         * 内训版：岗位课程活培训项目直接加入
         */

        $canAutoJoin = $this->canUserAutoJoinCourse($user, $course, $member);

        if ($canAutoJoin) {
            $member = $this->getCourseMemberService()->becomeStudent($courseId, $user['id']);
        }

        try {
            $task = $this->tryLearnTask($courseId, $id, (bool) $preview);
        } catch (AccessDeniedException $accessDeniedException) {
            return $this->handleAccessDeniedException($accessDeniedException, $request, $id);
        } catch (ServiceAccessDeniedException $deniedException) {
            return $this->handleAccessDeniedException($deniedException, $request, $id);
        }

        if ($member['locked']) {
            return $this->redirectToRoute('my_course_show', array('id' => $courseId));
        }

        if ($this->isCourseExpired($course) && !$this->getCourseService()->hasCourseManagerRole($course['id'])) {
            return $this->redirectToRoute('course_show', array('id' => $courseId));
        }

        if ($member !== null && $member['role'] != 'teacher' && !$this->getCourseMemberService()->isMemberNonExpired(
                $course,
                $member
            )
        ) {
            return $this->redirect($this->generateUrl('my_course_show', array('id' => $courseId)));
        }

        $activityConfig = $this->getActivityConfigByTask($task);

        if ($member !== null && $member['role'] === 'student' && $activityConfig->allowTaskAutoStart($task)) {
            $this->getActivityService()->trigger(
                $task['activityId'],
                'start',
                array(
                    'task' => $task,
                )
            );
        }

        $taskResult = $this->getTaskResultService()->getUserTaskResultByTaskId($id);
        if (empty($taskResult)) {
            $taskResult = array('status' => 'none');
        }

        if ($taskResult['status'] == 'finish') {
            $progress = $this->getLearningDataAnalysisService()->getUserLearningProgress($courseId, $user['id']);
            $finishedRate = $progress['percent'];
        }
        list($previousTask, $nextTask) = $this->getPreviousTaskAndTaskResult($task);
        $this->freshTaskLearnStat($request, $task['id']);

        if (in_array($user['id'], $course['teacherIds'])) {
            $isCourseTeacher = true;
        }

        return $this->render(
            'task/show.html.twig',
            array(
                'course' => $course,
                'member' => $member,
                'task' => $task,
                'taskResult' => $taskResult,
                'nextTask' => empty($nextTask) ? array() : $nextTask,
                'previousTask' => $previousTask,
                'finishedRate' => empty($finishedRate) ? 0 : $finishedRate,
                'isCourseTeacher' => empty($isCourseTeacher) ? false : $isCourseTeacher,
                'allowEventAutoTrigger' => $activityConfig->allowEventAutoTrigger(),
            )
        );
    }

    protected function canStartTask($task)
    {
        $activity = $this->getActivityService()->getActivity($task['activityId']);
        $config = $this->getActivityService()->getActivityConfig($activity['mediaType']);

        return $config->allowTaskAutoStart($activity);
    }

    private function freshTaskLearnStat(Request $request, $taskId)
    {
        $key = 'task.'.$taskId;
        $session = $request->getSession();
        $taskStore = $session->get($key, array());
        $taskStore['start'] = time();
        $taskStore['lastTriggerTime'] = 0;

        $session->set($key, $taskStore);
    }

    protected function canUserAutoJoinCourse($user, $course, $member)
    {
        if (!empty($member) || !empty($course['parentId'])) {
            return false;
        }

        $isBelongPostCourse = $this->getUserPostCourseService()->isCourseBelongToUserPostCourse($course['id'], $user);

        $isBelongAssignment = $this->getTrainingAssignmentMemberService()->isBelongToUserAssignment($user['id'], $course['id'], 'course');

        return $isBelongPostCourse || $isBelongAssignment;
    }

    /**
     * @return PostCourseService
     */
    protected function getPostCourseService()
    {
        return $this->createService('CorporateTrainingBundle:PostCourse:PostCourseService');
    }

    /**
     * @return UserPostCourseService
     */
    protected function getUserPostCourseService()
    {
        return $this->createService('CorporateTrainingBundle:PostCourse:UserPostCourseService');
    }

    /**
     * @return TrainingAssignmentMemberService
     */
    protected function getTrainingAssignmentMemberService()
    {
        return $this->createService('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentMemberService');
    }
}
