<?php

namespace CorporateTrainingBundle\Controller\TrainingAssignment;

use AppBundle\Common\ArrayToolkit;
use AppBundle\Common\Paginator;
use AppBundle\Controller\BaseController;
use Biz\Org\Service\OrgService;
use CorporateTrainingBundle\Biz\Post\Service\PostService;
use CorporateTrainingBundle\Biz\TrainingAssignment\Service\TrainingAssignmentMemberService;
use CorporateTrainingBundle\Biz\TrainingAssignment\Service\TrainingAssignmentService;
use CorporateTrainingBundle\Biz\User\Service\UserOrgService;
use CorporateTrainingBundle\Biz\UserAttribute\Service\UserAttributeService;
use Symfony\Component\HttpFoundation\Request;

class MemberManageController extends BaseController
{
    public function memberManageListAction(Request $request, $id)
    {
        if (!$this->getTrainingAssignmentService()->canManageTrainingAssignment($id)) {
            return $this->createMessageResponse('error', '没有部门管理的权限');
        }

        $trainingAssignment = $this->getTrainingAssignmentService()->getTrainingAssignment($id);
        if (empty($trainingAssignment)) {
            throw $this->createNotFoundException('没有该培训项目');
        }

        $conditions = $request->query->all();
        $conditions['assignmentId'] = $trainingAssignment['id'];

        $conditions = $this->prepareMemberListSearchConditions($conditions, $trainingAssignment);

        $count = $this->getTrainingAssignmentMemberService()->countTrainingAssignmentMembers(
            $conditions
        );

        $paginator = new Paginator(
            $request,
            $count,
            20
        );

        $members = $this->getTrainingAssignmentMemberService()->searchTrainingAssignmentMembers(
            $conditions,
            array('id' => 'DESC'),
            $paginator->getOffsetCount(),
            $paginator->getPerPageCount()
        );

        $members = $this->createAssignmentStrategy()->appendProgress($trainingAssignment, $members);

        $userIds = ArrayToolkit::column($members, 'userId');
        $userProfiles = $this->getUserService()->findUserProfilesByIds($userIds);
        $userProfiles = ArrayToolkit::index($userProfiles, 'id');
        $users = $this->getUserService()->findUsersByIds($userIds);
        $users = ArrayToolkit::index($users, 'id');

        $orgs = $this->getOrgService()->findOrgsByIds(ArrayToolkit::column($users, 'orgId'));
        $orgs = ArrayToolkit::index($orgs, 'id');

        $posts = $this->getPostService()->findPostsByIds(ArrayToolkit::column($users, 'postId'));
        $posts = ArrayToolkit::index($posts, 'id');
        $usersTotalLearnTime = $this->createAssignmentStrategy()->calculateMembersTotalLearnTime(
            $trainingAssignment,
            $userIds
        );

        return $this->render(
            '@App/training-assignment/member-manage-list.html.twig',
            array(
                'members' => $members,
                'users' => $users,
                'userProfiles' => $userProfiles,
                'orgs' => $orgs,
                'posts' => $posts,
                'paginator' => $paginator,
                'usersTotalLearnTime' => $usersTotalLearnTime,
                'assignment' => $trainingAssignment,
                'orgIds' => implode(',', $conditions['orgIds']),
            )
        );
    }

    public function memberMatchAction(Request $request)
    {
        $orgs = $this->getUserManageOrgs();
        $orgIds = ArrayToolkit::column($orgs, 'id');
        $queryString = $request->query->get('q');
        $attributes = array('user', 'org', 'post', 'userGroup');
        $name = $this->getUserAttributeService()->searchAttributesName($attributes, $queryString, $orgIds);

        return $this->createJsonResponse($name);
    }

    public function createMemberAction(Request $request, $id)
    {
        if (!$this->getTrainingAssignmentService()->canManageTrainingAssignment($id)) {
            return $this->createMessageResponse('error', '没有部门管理的权限');
        }

        if ($request->getMethod() == 'POST') {
            $data = $request->request->all();

            $trainingMembers = json_decode($data['trainingMembers'], true);

            if (empty($trainingMembers)) {
                return $this->createMessageResponse('error', '请选择要添加的用户');
            }
            $orgs = $this->getUserManageOrgs();
            $orgIds = ArrayToolkit::column($orgs, 'id');
            $userIds = $this->getUserAttributeService()->findDistinctUserIdsByAttributes($trainingMembers, $orgIds);

            if (count($userIds) > 2000) {
                return $this->createJsonResponse(array('success' => false, 'message' => '您不能一次添加超过2000个用户'));
            }

            $this->getTrainingAssignmentMemberService()->batchBecomeMember($id, $userIds);

            return $this->createJsonResponse(array('success' => true));
        }

        return $this->render('training-assignment/member-create-modal.html.twig',
            array(
                'id' => $id,
            )
        );
    }

    public function ajaxRemoveAssignmentMemberAction(Request $request, $id)
    {
        $result = $this->getTrainingAssignmentMemberService()->deleteTrainingAssignmentMember($id);

        if ($result) {
            return $this->createJsonResponse(true);
        }

        return $this->createJsonResponse(false);
    }

    public function membersRemoveAction(Request $request, $assignmentId)
    {
        $userIds = $request->request->get('ids');
        $this->getTrainingAssignmentMemberService()->batchDeleteMembers($assignmentId, $userIds);

        return $this->createJsonResponse(true);
    }

    public function memberScoreModalShowAction(Request $request, $assignmentId, $userId)
    {
        $trainingAssignment = $this->getTrainingAssignmentService()->getTrainingAssignment($assignmentId);
        if (empty($trainingAssignment)) {
            throw $this->createNotFoundException('没有该培训项目');
        }

        $results = $this->createAssignmentStrategy()->getUserAssignmentScoreResults($trainingAssignment, $userId);

        return $this->render(
            'training-assignment/member/member-score-modal.html.twig',
            array(
                'results' => $results,
            )
        );
    }

    public function recordAction(Request $request, $source = null)
    {
        $user = $this->getCurrentUser();
        $conditions = array(
            'userId' => $user['id'],
        );
        $memberCount = $this->getTrainingAssignmentMemberService()->countTrainingAssignmentMembers($conditions);

        $paginator = new Paginator(
            $request,
            $memberCount,
            20
        );

        $members = $this->getTrainingAssignmentMemberService()->searchTrainingAssignmentMembers(
            $conditions,
            array('createdTime' => 'DESC'),
            $paginator->getOffsetCount(),
            $paginator->getPerPageCount()
        );

        $assignments = $this->buildMemberAssignment($members);

        $view = 'study-center/department-manage/training-assignment/record.html.twig';
        if ($source == 'user_learn') {
            $view = 'data-report/course-statistics/assignment-record.html.twig';
        } elseif ($source == 'offline_course') {
            $view = 'offline-course-manage/study-record/assignment-record.html.twig';
        }

        return $this->render(
            $view,
            array(
                'userId' => $request->get('userId'),
                'assignments' => $assignments,
                'paginator' => $paginator,
                'tab_types' => $request->get('tab_types', 'assignmentRecord'),
            )
        );
    }

    private function buildMemberAssignment($members)
    {
        foreach ($members as &$member) {
            $assignment = $this->getTrainingAssignmentService()->getTrainingAssignment($member['assignmentId']);
            $member['name'] = $assignment['name'];
            $member['endTime'] = $assignment['endTime'];
            $learnTime = $this->createAssignmentStrategy()->calculateMembersTotalLearnTime($assignment, array($member['userId']));
            $member['learnTime'] = empty($learnTime) ? 0 : $learnTime[$member['userId']];
            $member['progress'] = $this->createAssignmentStrategy($assignment['id'])->calculateMemberLearnProgress($assignment, $member['userId']);
        }

        return $members;
    }

    protected function prepareMemberListSearchConditions($conditions, $trainingAssignment)
    {
        if (!isset($conditions['orgIds'])) {
            $orgs = $this->getUserManageOrgs();
            $orgIds = ArrayToolkit::column($orgs, 'id');
        }

        $conditions['orgIds'] = empty($orgIds) ? explode(',', $conditions['orgIds']) : $orgIds;

        $orgUserIds = $this->getAssignmentOrgUserIds($conditions['assignmentId'], $conditions['orgIds']);
        $conditions['userIds'] = $orgUserIds;

        if (!empty($conditions['username'])) {
            $userIds = $this->getUserIdsByNickNameOrTrueName($conditions['username']);
            if (empty($conditions['userIds']) || empty($userIds)) {
                $conditions['userIds'] = array();
            } else {
                $conditions['userIds'] = array_intersect($conditions['userIds'], $userIds);
            }
            unset($conditions['username']);
        }

        $conditions['userIds'] = empty($conditions['userIds']) ? array(-1) : $conditions['userIds'];

        if (!empty($conditions['status']) && $conditions['status'] == 'all') {
            unset($conditions['status']);
        }

        if (!empty($conditions['status']) && $conditions['status'] == 'unfinished') {
            unset($conditions['status']);
            $conditions['statuses'] = array('notStart', 'ongoing');
        }

        if (empty($trainingAssignment['startTime']) && empty($trainingAssignment['endTime'])) {
            unset($conditions['showDelay']);
        }

        if (!empty($conditions['showDelay']) && $conditions['status'] == 'finished' && !empty($trainingAssignment['startTime']) && !empty($trainingAssignment['endTime'])) {
            $conditions['finishedTime_GT'] = $trainingAssignment['endTime'];
        }

        return $conditions;
    }

    protected function getAssignmentOrgUserIds($assignmentId, $orgIds)
    {
        $members = $this->getTrainingAssignmentMemberService()->findMembersByAssignmentId($assignmentId);

        $userIds = ArrayToolkit::column($members, 'userId');

        $users = $this->getUserOrgService()->searchUserOrgs(
            array('orgIds' => $orgIds, 'userIds' => $userIds),
            array(),
            0,
            PHP_INT_MAX
        );

        return ArrayToolkit::column($users, 'userId');
    }

    protected function getUserIdsByNickNameOrTrueName($name)
    {
        $users = $this->getUserService()->searchUsers(
            array('nickname' => $name),
            array('id' => 'DESC'),
            0,
            20
        );

        $userIds = ArrayToolkit::column($users, 'id');

        $users = $this->getUserService()->searchUserProfiles(
            array('truename' => $name),
            array('id' => 'DESC'),
            0,
            20
        );

        $userIds = array_merge($userIds, ArrayToolkit::column($users, 'id'));

        return $userIds;
    }

    protected function createAssignmentStrategy()
    {
        return $this->getBiz()->offsetGet('assignment_strategy_context')->createStrategy();
    }

    /**
     * @return TrainingAssignmentService
     */
    protected function getTrainingAssignmentService()
    {
        return $this->createService('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentService');
    }

    /**
     * @return TrainingAssignmentMemberService
     */
    protected function getTrainingAssignmentMemberService()
    {
        return $this->createService('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentMemberService');
    }

    /**
     * @return OrgService
     */
    protected function getOrgService()
    {
        return $this->createService('Org:OrgService');
    }

    /**
     * @return PostService
     */
    protected function getPostService()
    {
        return $this->createService('CorporateTrainingBundle:Post:PostService');
    }

    /**
     * @return UserOrgService
     */
    protected function getUserOrgService()
    {
        return $this->createService('CorporateTrainingBundle:User:UserOrgService');
    }

    /**
     * @return UserAttributeService
     */
    protected function getUserAttributeService()
    {
        return $this->createService('CorporateTrainingBundle:UserAttribute:UserAttributeService');
    }
}
