<?php

namespace CorporateTrainingBundle\Controller\TrainingAssignment;

use AppBundle\Common\ArrayToolkit;
use AppBundle\Common\Exception\InvalidArgumentException;
use AppBundle\Common\Paginator;
use AppBundle\Controller\BaseController;
use Symfony\Component\HttpFoundation\Request;

class TrainingAssignmentManageController extends BaseController
{
    public function createAction(Request $request)
    {
        $user = $this->getCurrentUser();
        if (!$this->getTrainingAssignmentService()->hasManageTrainingAssignmentRole()) {
            return $this->createMessageResponse('error', '没有项目指派的权限');
        }

        if ($request->getMethod() == 'POST') {
            $fields = $request->request->all();
            $org = $this->getOrgService()->getOrgByOrgCode($fields['orgCode']);
            $fields['orgId'] = empty($org) ? 1 : $org['id'];

            $assignment = $this->getTrainingAssignmentService()->createTrainingAssignment($fields);

            $this->setFlashMessage('success', '项目信息已保存！');

            return $this->redirect($this->generateUrl('training_assignment_manage_base', array('id' => $assignment['id'])));
        }

        return $this->render('training-assignment/create.html.twig', array(
                'user' => $user,
            )
        );
    }

    public function baseAction(Request $request, $id)
    {
        if (!$this->getTrainingAssignmentService()->canManageTrainingAssignment($id)) {
            return $this->createMessageResponse('error', '没有项目指派的权限');
        }

        $assignment = $this->getTrainingAssignmentService()->getTrainingAssignment($id);

        if ($request->getMethod() == 'POST') {
            $fields = $request->request->all();
            $org = $this->getOrgService()->getOrgByOrgCode($fields['orgCode']);
            $fields['startTime'] = empty($fields['startDateTime']) ? 0 : strtotime($fields['startDateTime']);
            $fields['endTime'] = empty($fields['endDateTime']) ? 0 : strtotime($fields['endDateTime'].' 23:59:59');
            $fields['orgId'] = empty($org) ? 1 : $org['id'];
            $this->getTrainingAssignmentService()->updateTrainingAssignment($id, $fields);
            $this->setFlashMessage('success', '项目信息已保存！');

            return $this->redirect($this->generateUrl('training_assignment_manage_base', array('id' => $id)));
        }

        return $this->render('training-assignment/base.html.twig', array(
                'assignment' => $assignment,
            )
        );
    }

    public function coverAction(Request $request, $id)
    {
        if (!$this->getTrainingAssignmentService()->canManageTrainingAssignment($id)) {
            return $this->createMessageResponse('error', '没有项目指派的权限');
        }

        if ($request->isMethod('POST')) {
            $data = $request->request->all();
            $this->getTrainingAssignmentService()->changeCover($id, $data);

            return $this->redirect($this->generateUrl('training_assignment_manage_cover', array('id' => $id)));
        }

        $assignment = $this->getTrainingAssignmentService()->getTrainingAssignment($id);

        return $this->render('training-assignment/cover.html.twig', array(
                'assignment' => $assignment,
            )
        );
    }

    public function coverCropAction(Request $request, $id)
    {
        if (!$this->getTrainingAssignmentService()->canManageTrainingAssignment($id)) {
            return $this->createMessageResponse('error', '没有项目指派的权限');
        }

        $assignment = $this->getTrainingAssignmentService()->getTrainingAssignment($id);

        if ($request->getMethod() == 'POST') {
            $data = $request->request->all();
            $this->getTrainingAssignmentService()->changeCover($assignment['id'],
                json_decode($data['images'], true));

            return $this->redirect($this->generateUrl('training_assignment_manage_cover',
                array('id' => $assignment['id'])));
        }

        $fileId = $request->getSession()->get('fileId');
        list($pictureUrl, $naturalSize, $scaledSize) = $this->getFileService()->getImgFileMetaInfo($fileId, 480, 270);

        return $this->render(
            'training-assignment/cover-crop.html.twig',
            array(
                'assignment' => $assignment,
                'pictureUrl' => $pictureUrl,
                'naturalSize' => $naturalSize,
                'scaledSize' => $scaledSize,
            )
        );
    }

    public function headerAction($assignment)
    {
        if (!$this->getTrainingAssignmentService()->canManageTrainingAssignment($assignment['id'])) {
            return $this->createMessageResponse('error', '没有项目指派的权限');
        }

        return $this->render('training-assignment/header.html.twig', array(
                'assignment' => $assignment,
            )
        );
    }

    public function sidebarAction($id, $sideNav)
    {
        if (!$this->getTrainingAssignmentService()->canManageTrainingAssignment($id)) {
            return $this->createMessageResponse('error', '没有项目指派的权限');
        }

        $assignment = $this->getTrainingAssignmentService()->getTrainingAssignment($id);

        return $this->render('training-assignment/sidebar.html.twig', array(
                'assignment' => $assignment,
                'side_nav' => $sideNav,
            )
        );
    }

    public function trainingAssignmentManageListAction(Request $request)
    {
        if (!$this->getTrainingAssignmentService()->hasManageTrainingAssignmentRole()) {
            return $this->createMessageResponse('error', '没有项目指派的权限');
        }

        $conditions = $request->query->all();
        $conditions = $this->prepareManageListSearchConditions($conditions);

        $assignmentCount = $this->getTrainingAssignmentService()->countTrainingAssignments($conditions);
        $paginator = new Paginator(
            $request,
            $assignmentCount,
            20
        );

        $assignments = $this->getTrainingAssignmentService()->searchTrainingAssignments(
            $conditions,
            array('createdTime' => 'DESC'),
            $paginator->getOffsetCount(),
            $paginator->getPerPageCount()
        );

        foreach ($assignments as &$assignment) {
            $assignment['memberNum'] = $this->getAssignmentMemberNum($assignment['id'], $conditions['orgIds']);
            $assignment['itemNum'] = $this->getAssignmentCourseNum($assignment['id'], $conditions['orgIds']);
        }

        $orgIds = ArrayToolkit::column($assignments, 'orgId');
        $orgs = $this->getOrgService()->findOrgsByIds($orgIds);
        $orgs = ArrayToolkit::index($orgs, 'id');

        return $this->render(
            'training-assignment/manage-list.html.twig',
            array(
                'assignments' => $assignments,
                'orgs' => $orgs,
                'orgIds' => implode(',', $conditions['orgIds']),
                'paginator' => $paginator,
            )
        );
    }

    public function ajaxPublishAction(Request $request, $id)
    {
        if (!$this->getTrainingAssignmentService()->canManageTrainingAssignment($id)) {
            return $this->createJsonResponse(array('success' => 'false', 'message' => '没有发布项目的权限'));
        }

        $assignment = $this->getTrainingAssignmentService()->getTrainingAssignment($id);
        if (empty($assignment)) {
            return $this->createJsonResponse(array('success' => 'false', 'message' => '该项目不存在！'));
        }

        $this->getTrainingAssignmentService()->publishTrainingAssignment($id);

        return $this->createJsonResponse(array('success' => true, 'message' => ''));
    }

    public function ajaxCloseAction(Request $request, $id)
    {
        if (!$this->getTrainingAssignmentService()->canManageTrainingAssignment($id)) {
            return $this->createJsonResponse(false);
        }

        $assignment = $this->getTrainingAssignmentService()->getTrainingAssignment($id);

        if (empty($assignment)) {
            return $this->createJsonResponse(false);
        }

        $result = $this->getTrainingAssignmentService()->closeTrainingAssignment($id);
        if ($result) {
            return $this->createJsonResponse(true);
        }

        return $this->createJsonResponse(false);
    }

    public function ajaxRemoveAction(Request $request, $id)
    {
        if (!$this->getTrainingAssignmentService()->canManageTrainingAssignment($id)) {
            return $this->createJsonResponse(false);
        }

        $assignment = $this->getTrainingAssignmentService()->getTrainingAssignment($id);

        if (empty($assignment)) {
            return $this->createJsonResponse(false);
        }

        $result = $this->getTrainingAssignmentService()->deleteTrainingAssignmentById($id);

        if ($result) {
            return $this->createJsonResponse(true);
        }

        return $this->createJsonResponse(false);
    }

    public function coursesAction(Request $request, $id)
    {
        if (!$this->getTrainingAssignmentService()->canManageTrainingAssignment($id)) {
            return $this->createJsonResponse(array('success' => 'false', 'message' => '没有发布项目的权限'));
        }

        $assignment = $this->getTrainingAssignmentService()->getTrainingAssignment($id);
        if (empty($assignment)) {
            return $this->createJsonResponse(array('success' => 'false', 'message' => '该项目不存在！'));
        }

        if ($request->isMethod('POST')) {
            $data = $request->request->all();
            if (empty($data) || !isset($data['assignmentCourses'])) {
                throw new InvalidArgumentException('Empty Data');
            }

            $assignmentCourses = explode(',', $data['assignmentCourses']);
            if (empty($assignmentCourses)) {
                throw new InvalidArgumentException('Empty Data');
            }

            $result = array_unique($assignmentCourses);
            if ($assignmentCourses != $result) {
                throw new InvalidArgumentException('Repeat Data');
            }

            $this->getTrainingAssignmentItemService()->setAssignmentCourseItems($id, $assignmentCourses);
            $this->setFlashMessage('success', '内容保存成功');

            return $this->redirectToRoute(
                'training_assignment_manage_courses',
                array('id' => $id)
            );
        }

        $items = $this->getTrainingAssignmentItemService()->searchTrainingAssignmentItems(
            array('assignmentId' => $id, 'targetType' => 'course'),
            array('seq' => 'ASC'),
            0,
            PHP_INT_MAX
        );

        $courseIds = ArrayToolkit::column($items, 'targetId');
        $courses = $this->getCourseService()->findCoursesByIds($courseIds);
        $courses = ArrayToolkit::index($courses, 'id');

        $assignmentCourses = array();
        foreach ($items as $key => $item) {
            $assignmentCourses[$key] = array(
                'id' => $item['targetId'],
                'title' => $courses[$item['targetId']]['title'],
            );
        }

        return $this->render(
            '@App/training-assignment/item-manage.html.twig',
            array(
                'assignmentCourses' => $assignmentCourses,
                'assignment' => $assignment,
                'assignmentCourseIds' => implode(',', ArrayToolkit::column($assignmentCourses, 'id')),
            )
        );
    }

    public function matchCourseAction(Request $request)
    {
        $data = array();
        $queryString = $request->query->get('q');

        $originOrgIds = array('1');
        $orgCodes = $this->getCurrentUser()->getOrgCodes();
        $orgs = $this->getOrgService()->findOrgsByPrefixOrgCodes($orgCodes);
        $orgIds = ArrayToolkit::column($orgs, 'id');
        $orgIds = array_merge($originOrgIds, $orgIds);

        $conditions = array(
            'status' => 'published',
            'parentId' => 0,
            'titleLike' => '%'.$queryString.'%',
            'orgIds' => $orgIds,
        );

        $coursesSets = $this->getCourseSetService()->searchCourseSets($conditions, array(), 0, 10);

        foreach ($coursesSets as $course) {
            $data[] = array('id' => $course['defaultCourseId'], 'name' => $course['title']);
        }

        return $this->createJsonResponse($data);
    }

    private function getAssignmentMemberNum($assignmentId, $orgIds)
    {
        $assignmentMembers = $this->getTrainingAssignmentMemberService()->searchTrainingAssignmentMembers(
            array('assignmentId' => $assignmentId),
                array(),
                0,
                PHP_INT_MAX
        );

        if (empty($assignmentMembers)) {
            return 0;
        }

        $userIds = ArrayToolkit::column($assignmentMembers, 'userId');
        $conditions = array(
            'userIds' => $userIds,
            'orgIds' => $orgIds,
        );

        return $this->getUserService()->countUsers($conditions);
    }

    private function getAssignmentCourseNum($assignmentId, $orgIds)
    {
        $courseItems = $this->getTrainingAssignmentItemService()->findAssignmentItemsByAssignmentIdAndTargetType($assignmentId, 'course');

        if (empty($courseItems)) {
            return 0;
        }

        $courseIds = ArrayToolkit::column($courseItems, 'targetId');

        $conditions = array(
            'defaultCourseIds' => $courseIds,
            'orgIds' => $orgIds,
        );

        return $this->getCourseSetService()->countCourseSets($conditions);
    }

    protected function prepareManageListSearchConditions($conditions)
    {
        if (!isset($conditions['orgIds']) || empty($conditions['orgIds'])) {
            $currentUser = $this->getCurrentUser();
            $orgCodes = $currentUser['orgCodes'];
            $orgIds = ArrayToolkit::column($this->getOrgService()->findOrgsByPrefixOrgCodes($orgCodes), 'id');
        } else {
            $orgIds = explode(',', $conditions['orgIds']);
        }

        $conditions['orgIds'] = $orgIds;

        if (!empty($conditions['startDateTime'])) {
            $conditions['startTime_GE'] = strtotime($conditions['startDateTime'].' 23:59:59');
            unset($conditions['startDateTime']);
        }

        if (!empty($conditions['endDateTime'])) {
            $conditions['endTime_LE'] = strtotime($conditions['endDateTime'].' 23:59:59');
            unset($conditions['endDateTime']);
        }

        if (!empty($conditions['status']) && 'all' === $conditions['status']) {
            unset($conditions['status']);
        }

        return $conditions;
    }

    protected function createAssignmentStrategy()
    {
        return $this->getBiz()->offsetGet('assignment_strategy_context')->createStrategy();
    }

    /**
     * @return \CorporateTrainingBundle\Biz\TrainingAssignment\Service\Impl\TrainingAssignmentServiceImpl
     */
    protected function getTrainingAssignmentService()
    {
        return $this->createService('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentService');
    }

    /**
     * @return \CorporateTrainingBundle\Biz\TrainingAssignment\Service\Impl\TrainingAssignmentItemServiceImpl
     */
    protected function getTrainingAssignmentItemService()
    {
        return $this->createService('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentItemService');
    }

    /**
     * @return \CorporateTrainingBundle\Biz\TrainingAssignment\Service\Impl\TrainingAssignmentMemberServiceImpl
     */
    protected function getTrainingAssignmentMemberService()
    {
        return $this->createService('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentMemberService');
    }

    /**
     * @return \Biz\Course\Service\Impl\CourseServiceImpl
     */
    protected function getCourseService()
    {
        return $this->createService('Course:CourseService');
    }

    /**
     * @return \Biz\Content\Service\Impl\FileServiceImpl
     */
    protected function getFileService()
    {
        return $this->createService('Content:FileService');
    }

    /**
     * @return \Biz\Org\Service\Impl\OrgServiceImpl
     */
    protected function getOrgService()
    {
        return $this->createService('Org:OrgService');
    }

    /**
     * @return \Biz\Course\Service\Impl\CourseSetServiceImpl
     */
    protected function getCourseSetService()
    {
        return $this->createService('Course:CourseSetService');
    }
}
