<?php

namespace CorporateTrainingBundle\Extensions\DataTag;

use AppBundle\Common\ArrayToolkit;
use AppBundle\Extensions\DataTag\DataTag;
use AppBundle\Extensions\DataTag\BaseDataTag;
use Biz\Org\Service\OrgService;
use CorporateTrainingBundle\Biz\DataStatistics\Service\UserDailyLearnRecordService;
use CorporateTrainingBundle\Biz\User\Service\UserService;
use CorporateTrainingBundle\Common\DateToolkit;

class StudyRankDataTag extends BaseDataTag implements DataTag
{
    public function getData(array $arguments)
    {
        $type = empty($arguments['type']) ? 'week' : $arguments['type'];
        list($startDateTime, $endDateTime) = DateToolkit::generateStartDateAndEndDate($type);
        $orgs = $this->getOrgService()->searchOrgs(array(), array(), 0, PHP_INT_MAX);
        $orgIds = ArrayToolkit::column($orgs, 'id');

        $userIds = $this->buildOrgUserIds($arguments);
        $conditions = array(
            'userIds' => $userIds,
            'startDateTime' => strtotime($startDateTime),
            'endDateTime' => strtotime($endDateTime),
        );

        $orgUsers = $this->getUserService()->statisticsOrgUserNumGroupByOrgId();
        $orgUsers = ArrayToolkit::index($orgUsers, 'orgId');

        return array(
            'personStudyRanks' => $this->getUserDailyLearnRecordService()->statisticsPersonLearnTimeRankingList($conditions),
            'orgStudyRanks' => $this->getUserDailyLearnRecordService()->statisticsOrgLearnTimeRankingList($conditions, 0, 5),
            'orgUsers' => $orgUsers,
            'orgIds' => implode(',', $orgIds),
        );
    }

    protected function buildOrgUserIds($arguments)
    {
        $conditions = array();

        if (!empty($arguments['orgIds'])) {
            $conditions['orgIds'] = $arguments['orgIds'];
        }

        $userOrgs = $this->getUserOrgService()->searchUserOrgs(
            $conditions,
            array('id' => 'DESC'),
            0,
            PHP_INT_MAX
        );

        return ArrayToolkit::column($userOrgs, 'userId');
    }

    protected function getTaskResultService()
    {
        return $this->getServiceKernel()->getBiz()->service('Task:TaskResultService');
    }

    /**
     * @return UserDailyLearnRecordService
     */
    protected function getUserDailyLearnRecordService()
    {
        return $this->getServiceKernel()->getBiz()->service('CorporateTrainingBundle:DataStatistics:UserDailyLearnRecordService');
    }

    /**
     * @return UserService
     */
    protected function getUserService()
    {
        return $this->getServiceKernel()->getBiz()->service('CorporateTrainingBundle:User:UserService');
    }

    /**
     * @return OrgService
     */
    protected function getOrgService()
    {
        return $this->getServiceKernel()->getBiz()->service('Org:OrgService');
    }

    protected function getUserOrgService()
    {
        return $this->getServiceKernel()->getBiz()->service('CorporateTrainingBundle:User:UserOrgService');
    }
}
