<?php

namespace CorporateTrainingBundle\Listener;

use AppBundle\Common\ArrayToolkit;
use AppBundle\Common\Exception\AccessDeniedException;
use Symfony\Component\HttpKernel\HttpKernelInterface;
use Symfony\Component\HttpKernel\Event\FilterControllerEvent;
use Topxia\Service\Common\ServiceKernel;

class OrgAccessControlListener
{
    const ROOT_ORG_CODE = '1.';

    public function __construct($container)
    {
        $this->container = $container;
    }

    public function onKernelController(FilterControllerEvent $event)
    {
        if ($event->getRequestType() != HttpKernelInterface::MASTER_REQUEST) {
            return;
        }

        $request = $event->getRequest();
        $queryParams = $request->query->all();
        $formData = $request->request->all();

        if (isset($queryParams['orgCode'])) {
            if (!$this->isOrgCodeAccessible($queryParams['orgCode'])) {
                throw new AccessDeniedException('您无权查看和操作该组织机构');
            }
        }

        if (isset($queryParams['orgId'])) {
            if (!$this->isOrgIdAccessible($queryParams['orgId'])) {
                throw new AccessDeniedException('您无权查看和操作该组织机构');
            }
        }

        if (!empty($queryParams['orgCodes'])) {
            $orgCodes = explode(',', $queryParams['orgCodes']);
            foreach ($orgCodes as $orgCode) {
                if (!$this->isOrgCodeAccessible($orgCode)) {
                    throw new AccessDeniedException('您无权查看和操作该组织机构');
                }
            }
        }

        if (!empty($queryParams['orgIds'])) {
            $orgIds = explode(',', $queryParams['orgIds']);
            foreach ($orgIds as $orgId) {
                if (!$this->isOrgIdAccessible($orgId)) {
                    throw new AccessDeniedException('您无权查看和操作该组织机构');
                }
            }
        }

        if (isset($formData['orgCode'])) {
            if (!$this->isOrgCodeAccessible($formData['orgCode'])) {
                throw new AccessDeniedException('您无权查看和操作该组织机构');
            }
        }

        if (isset($formData['orgId'])) {
            if (!$this->isOrgIdAccessible($formData['orgId'])) {
                throw new AccessDeniedException('您无权查看和操作该组织机构');
            }
        }

        if (isset($formData['orgIds'])) {
            $orgIds = $formData['orgIds'];

            if (!is_array($formData['orgIds'])) {
                $orgIds = explode(',', $orgIds);
            }
            foreach ($orgIds as $orgId) {
                if (!$this->isOrgIdAccessible($orgId)) {
                    throw new AccessDeniedException('您无权查看和操作该组织机构');
                }
            }
        }

        if (isset($formData['orgCodes'])) {
            $orgCodes = $formData['orgCodes'];

            if (!is_array($formData['orgCodes'])) {
                $orgCodes = explode(',', $orgCodes);
            }
            foreach ($orgCodes as $orgCode) {
                if (!$this->isOrgCodeAccessible($orgCode)) {
                    throw new AccessDeniedException('您无权查看和操作该组织机构');
                }
            }
        }
    }

    protected function isOrgCodeAccessible($orgCode)
    {
        $currentUser = ServiceKernel::instance()->getCurrentUser();
        $userOrgCodes = $currentUser['orgCodes'];
        if (in_array($this::ROOT_ORG_CODE, $userOrgCodes)) {
            return true;
        }
        $userAccessibleOrgs = $this->getOrgService()->findOrgsByPrefixOrgCodes($userOrgCodes);

        $userAccessibleOrgCodes = ArrayToolkit::column($userAccessibleOrgs, 'orgCode');

        return in_array($orgCode, $userAccessibleOrgCodes);
    }

    protected function isOrgIdAccessible($orgId)
    {
        $currentUser = ServiceKernel::instance()->getCurrentUser();
        $userOrgCodes = $currentUser['orgCodes'];
        if (in_array($this::ROOT_ORG_CODE, $userOrgCodes)) {
            return true;
        }
        $userAccessibleOrgs = $this->getOrgService()->findOrgsByPrefixOrgCodes($userOrgCodes);

        $userAccessibleOrgIds = ArrayToolkit::column($userAccessibleOrgs, 'id');

        return in_array($orgId, $userAccessibleOrgIds);
    }

    protected function getOrgService()
    {
        return ServiceKernel::instance()->createService('Org:OrgService');
    }
}
