import 'common/tabs-lavalamp/index';
import notify from 'common/notify';

import tabBlock from '../common/tab-block';
import SelectContainer from '../common/select-container';
import GroupSelector from './group-selector';
import DepartmentSelector from './department-selector';
import PostSelector from './post-selector';
import MemberSelector from './member-selector';
import CheckTreeviewInput from '../common/CheckTreeview-input';
import asyncArray from '../common/async-array';

$(".nav.nav-tabs").unbind();

let container =  new SelectContainer('.js-group-selector');
let groupSelector =  new GroupSelector('.js-search-by-group');
let postSelector = new PostSelector('.js-search-by-post');
let departmentSelector = new DepartmentSelector('.js-search-by-department');
let memberSelector = new MemberSelector('.js-search-by-member');

container.setOpts({
  removeCb: (obj) => {
    if (obj.type == memberSelector.itemType) {
      memberSelector.removeItem(obj.item);
    } else if (obj.type == departmentSelector.itemType) {
      departmentSelector.removeItem(obj.item);
    } else if (obj.type == postSelector.itemType) {
      postSelector.removeItem(obj.item);
    } else if (obj.type == groupSelector.itemType) {
      groupSelector.removeItem(obj.item);
    }
  }
});

postSelector.setOpts({
  addCb: (obj) => {
    container.addItem(obj);
  },
  removeCb: (obj) => {
    container.removeItem(obj);
  }
});

groupSelector.setOpts({
  addCb: (obj) => {
    container.addItem(obj);
  },
  removeCb: (obj) => {
    container.removeItem(obj);
  }
});

departmentSelector.setOpts({
  addCb: (obj) => {
    container.addItem(obj);
  },
  removeCb: (obj) => {
    container.removeItem(obj);
  }
});


memberSelector.setOpts({
  addCb: (obj) => {
    container.addItem(obj);
  },
  removeCb: (obj) => {
    container.removeItem(obj);
  }
});

new tabBlock({ element: '.js-member-assignment', mode: 'hover'});

class Page {
    constructor() {
        this.init();
    }

    init() {
        this.initTreeViewInput();
        this.initSelect();
        // this.initDatetimePicker();
        this.initvalidator();
    }

    initSelect() {
        $('[data-role="tree-select"], [name="categoryId"]').select2({
            treeview: true,
            dropdownAutoWidth: true,
            treeviewInitState: 'collapsed',
            placeholderOption: 'first',
            allowClear: true,
        });
    }

    initDatetimePicker() {
        const defaultOpts = {
            language: "zh",
            autoclose: true,
            format: 'yyyy-mm-dd',
            minView: 'month'
        };
        $('[name=startDateTime]').datetimepicker(defaultOpts);

        $('[name=endDateTime]').datetimepicker(defaultOpts);

        $('[name=startDateTime]').on('changeDate',function(){
          $('[name=endDateTime]').datetimepicker('setStartDate',$('[name=startDateTime]').val().substring(0,16));
        });

        $('[name=endDateTime]').on('changeDate',function(){
          $('[name=startDateTime]').datetimepicker('setEndDate',$('[name=endDateTime]').val().substring(0,16));
        });
    }

  initvalidator() {
    const $form = $('.department-search-form');

      var $postContainer = $('#postModalInput');
      $postContainer.select2({
          ajax: {
              url: $postContainer.data('url'),
              dataType: 'json',
              quietMillis: 100,
              data: function (term, page) {
                  return {
                      q: term,
                      page_limit: 10
                  };
              },
              results: function (data) {
                  var results = [];
                  $.each(data, function (index, item) {

                      results.push({
                          id: item.id,
                          name: item.name
                      });
                  });

                  return {
                      results: results
                  };

              }
          },
          initSelection: function(element, callback) {
              var id = $(element).val();
              if (id !== "") {
                  var name = $(element).data('name');
                  callback({id:id, name:name});
              }
          },
          formatSelection: function (item) {
              return item.name;
          },
          formatResult: function (item) {
              return item.name;
          },
          allowClear: true,
          width: 'off',
          placeholder: Translator.trans('所有岗位')
      });

  }

  initTreeViewInput() {
    let $treeview = $('.js-treeview-select-wrap');
    for (let i = 0; i < $treeview.length; i++) {
      new CheckTreeviewInput({ $elem: $treeview.eq(i) })
    }
  }
}

new Page();
$('[data-toggle="popover"]').popover();

$('.js-add-btn').on('click', function() {
  $('.js-add-btn').button('loading');
  $.post($(this).data('convert-attribute-url'), {
    userAttribute: container.tojson(),
  }).done((data) => {
    if (data.userIds && data.userIds.length) {
        let increment = 200;
        let len = data.userIds.length;
        let pos = 0;
        let end = 0;  
        let temp = [];
        while (pos <= len - 1) {
          end = Math.min(pos + increment , len) - 1;
          temp.push(data.userIds.slice(pos, end + 1));
          pos = end + 1;
        }

        let asyncArr = new asyncArray();

        for (let i = 0 ; i < temp.length; i++) {
          asyncArr.push( (next) => {
            $.post($(this).data('batch-add-url'), {
              userIds: temp[i]
            }).done((data) => {
              if(asyncArr.isEmpty()) {
                notify('success',Translator.trans('成员添加成功'));
                $('.js-add-btn').button('reset');

                window.location.reload();
              }

              if (data.status) {
                next();
              } else {
                notify('danger',Translator.trans('成员添加失败'));
              }
            });
          });
        }
    } else {
        notify('warning',Translator.trans('已选对象中没有成员'));
        $('.js-add-btn').button('reset');
    }
  });
});

