export default class MemberSelector {
  constructor(selector) {
    this.$elem = $(selector);
    this.itemType = 'user';
    this.init();
    this.map = {};
  }

  setOpts({ addCb = function() {}, removeCb = function() {} }) {
    this.addCb = addCb;
    this.removeCb = removeCb;
  }

  getItemObj($elem) {
    return  {
      key: `icon-0/${$elem.data('id')}`,
      item: $elem.get(0),
      id: $elem.data('id'),
      type: this.itemType,
      iconClassName: 'es-icon-person',
      name: $elem.data('name')
    };
  }

  removeItem(dom) {
    let $dom = $(dom); 
    $dom.prop('checked', false);
    if (this.map[$dom.data('id')]) {
      delete this.map[$dom.data('id')];
    }
  }

  addItem(dom) {
    let $dom = $(dom); 
    $dom.prop('checked', true);

    if (!this.map[$dom.data('id')]) {
      this.map[$dom.data('id')] = true;
    }
  }

  addItems() {
    this.$elem.find('.js-checkbox').each((index, item) => {
      if (!$(item).prop('checked')) {
        this.addItem(item);
        let obj = this.getItemObj($(item));
        this.addCb && this.addCb(obj);
      }
    });
  }

  removeItems() {
    this.$elem.find('.js-checkbox').each((index, item) => {
      if ($(item).prop('checked')) {
        this.removeItem(item);
        let obj = this.getItemObj($(item));
        this.removeCb && this.removeCb(obj);
      }
    });
  }

  toggleItem(e) {
    let $elem = $(e.currentTarget);

    if ($elem.prop('checked')) {
      if (!this.map[$elem.data('id')]) {
        this.map[$elem.data('id')] = true;
      }

      let obj = this.getItemObj($elem);
      this.addCb && this.addCb(obj);
    } else {
      if (this.map[$elem.data('id')]) {
        delete this.map[$elem.data('id')];
      }

      this.removeCb && this.removeCb(this.getItemObj($elem));
    }
  }

  init() {
    this.initEvent();
  }

  initEvent() {
    this.$elem.on('click', '.js-checkbox', (e) => {
      this.toggleItem(e);
    });

    this.$elem.on('click', '.js-select-all', (e) => {
      let $target = $(e.target);
      if($target.prop('checked')) {
        this.addItems();
      } else {
        this.removeItems();
      }
    });
  
    this.$elem.on('click', '.js-submit', (e) => {
      $.post($(e.target).data('url'), $('#user-select-form').serialize())
        .done((html) => {
          this.$elem.find('.js-search-by-member-wrap').html(html);
          this.$elem.find('.js-checkbox').each((index, item) => {
            if (this.map[$(item).data('id')]) {
              $(item).prop('checked', true);
            }
          });
        });
    });

    this.$elem.on('click','.pagination a', (e) => {
      $.post($(e.currentTarget).prop('href'), $('#user-select-form').serialize())
        .done((html) => {
          this.$elem.find('.js-search-by-member-wrap').html(html);
          this.$elem.find('.js-checkbox').each((index, item) => {
            if (this.map[$(item).data('id')]) {
              $(item).prop('checked', true);
            }
          });
          $('[data-toggle="popover"]').popover();
        });
      e.stopPropagation();
      e.preventDefault();
    });

    $('[data-toggle="popover"]').popover();
  }
}