export default class DepartmentSelector {
  constructor(selector) {
    this.$elem = $(selector);
    this.itemType = 'post';
    this.init();
  }

  setOpts({ addCb = function() {}, removeCb = function() {} }) {
    this.addCb = addCb;
    this.removeCb = removeCb;
  }

  getItemObj($elem) {
    return  {
      key: `icon-2/${$elem.data('id')}`,
      item: $elem.get(0),
      id: $elem.data('id'),
      type: this.itemType,
      iconClassName: 'es-icon-post',
      name: $elem.data('name')
    };
  }

  addItem(dom) {
    let $input = $(dom).find('input');
    $input.prop('checked', true);
  }

  removeItem(dom) {
    let $input = $(dom).find('input');
    $input.prop('checked', false);
  }

  addGroup(e) {
    let $elem = $(e.target).parents('.js-department-select-item');
    $elem.find('.checkbox').each((index, item) => {
      if (!$(item).find('input').prop('checked')) {
        this.addItem(item);
        let obj = this.getItemObj($(item));
        this.addCb && this.addCb(obj);
      }
    });
  }

  toggleItem(e) {
    let $elem = $(e.currentTarget);
    let $input = $(e.target);

    if ($input.prop('checked')) {
      let obj = this.getItemObj($elem);
      this.addCb && this.addCb(obj);
    } else {
      this.removeCb && this.removeCb(this.getItemObj($elem));
    }
  }

  init() {
    this.initEvent();
  }

  initSelect2() {
    let that = this;
    let $postContainer = $("#postModalSelectInput");
    let dataSource = [];
    $(".post-select-item").each((index, item) => {
      dataSource.push({id: $(item).data('id') + '', text: $(item).data('name')});
    });
    let $select = $postContainer.select2({
      data: dataSource,
      formatSelection: function (item) {
        return item.text;
      },
      formatResult: function (item) {
        return item.text;
      },
      formatNoMatches: function() {
        return "没有找到匹配项";
      },
      maximumSelectionSize: 20,
      allowClear: true,
      width: 'off',
      placeholder: Translator.trans('请输入岗位'),
      multiple: true
    });
    $select.on('change', (e) => {
      that.$elem.find('.checkbox').each((index, item) => {
        let $item = $(item);
        if ($item.data('name') == e.added.text) {
          let obj = that.getItemObj($item);
          let isChecked = $(obj.item).find('input').prop('checked');

          if (!isChecked) {
            that.addItem(obj.item);
            that.addCb && that.addCb(obj);
          }
        }
      });
      $select.select2("val", '');
    });
  }

  initEvent() {
    this.$elem.on('click', '.checkbox', (e) => {
      this.toggleItem(e);
    });

    this.initSelect2();

    this.$elem.on('click', '.js-select-all', (e) => {
      this.addGroup(e);
    });
  }
}