import CheckTreeview from './CheckTreeview';


export default class CheckTreeviewInput {

  static getDefaultOpts() {
    return {
      treeviewText: '.js-treeview-text',
      treeviewSelect: '.js-treeview-select',
      treeviewSelectMenu: '.js-treeview-select-menu',
      treeviewIpt: '.js-treeview-ipt',
      treeviewData: '.js-treeview-data'
    };
  }

  constructor(opts) {
    Object.assign(this, CheckTreeviewInput.getDefaultOpts(), opts);
    this.init();
  }

  init() {
    this.initTreeview();
    this.initEvent();

  }

  initTreeview() {
    let nodeArr = JSON.parse(this.getData());
    let checkStr = this.$elem.find(this.treeviewIpt).val();
    let checkArr = checkStr.split(',');
    let checkMap = {};
    let tmpArr = [].concat(nodeArr);

    for (let i = 0; i < checkArr.length; i++) {
      checkMap[checkArr[i]] = true;
    }

    while (tmpArr.length > 0) {
      let node = tmpArr.pop();

      node.state = {};
      if (checkMap[node.id]) {
        node.state.checked = true;
      }
      
      if (node.selectable != undefined && !node.selectable) {
        node.state.disabled = true;
        node.state.checked = false;
        node.state.expanded = true;
      }

      node.state.selected = false;

      if(node.nodes) {
        tmpArr = tmpArr.concat(node.nodes);
      }
    }

    this.checkTreeview = new CheckTreeview(this.$elem.find(this.treeviewSelectMenu), { data: nodeArr });
    const node = this.checkTreeview.getCheckNodes();
    let name = node.reduce( function(tot, item) {
      return tot + (tot && ',') + item.name + ' ';
    },'');  

    this.$elem.find(this.treeviewText).val(name);
  }

  initEvent() {
    this.$elem.on('focus', this.treeviewText, (e) => {
      $(e.currentTarget).parents(this.treeviewSelect).find(this.treeviewSelectMenu).addClass('is-active');
    });

    this.$elem.find(this.treeviewSelect).on('click', (e) => {
      const node = this.checkTreeview.getCheckNodes();

      let name = '';

      let len = Math.min(node.length, 10);

      for (let i = 0 ; i < len; i++) {
          name = name + (name && ',') + node[i].name + ' ';
      }

      if (len != node.length) {
        name = name + '...';
      }

      let id = node.reduce( function(tot, item) {
          return tot + (tot && ',') + item.id;
      },'');

      $(e.currentTarget).find(this.treeviewText).val(name);
      $(e.currentTarget).find(this.treeviewIpt).val(id);
      e.stopPropagation();
    });
  }

  getData() {
      return this.$elem.find(this.treeviewData).val();
  }
}