export default class Base {
    constructor() {
        this.init();
    }

    init() {
        const $form = $('#offlineCourse-form');
        const validator = $form.validate({
            rules: {
                title: {
                    byte_maxlength: 60,
                    chinese_alphanumeric: true,
                    required: {
                        depends () {
                            $(this).val($.trim($(this).val()));
                            return true;
                        }
                    },
                },
                categoryId: {
                    required: {
                        depends () {
                            $(this).val($.trim($(this).val()));
                            return true;
                        },
                    },
                },
                startDate: {
                    startDate_check:true,
                    required: {
                        depends () {
                            $(this).val($.trim($(this).val()));
                            return true;
                        },
                    },
                },
                endDate: {
                    endDate_check:true,
                    required: {
                        depends () {
                            $(this).val($.trim($(this).val()));
                            return true;
                        },
                    },
                },
                enrollmentEndDate: {
                    enrollmentEndDate_before_check:true,
                    enrollmentEndDate_after_check:true,
                    required: {
                        depends () {
                            $(this).val($.trim($(this).val()));
                            return true;
                        },
                    },
                },
                address: {
                    byte_maxlength: 60,
                    chinese_alphanumeric: true,
                    required: {
                        depends () {
                            $(this).val($.trim($(this).val()));
                            return true;
                        }
                    },
                },
                maxStudentNum: {
                    max:10000,
                    unsigned_integer:true,
                    required: {
                        depends () {
                            $(this).val($.trim($(this).val()));
                            return true;
                        }
                    },
                },
            },
            messages: {
                title: {
                    required: Translator.trans('请输入标题'),
                    trim: Translator.trans('请输入标题'),
                },
                categoryId: {
                    required: Translator.trans('请选择分类'),
                    trim: Translator.trans('请选择分类'),
                },
                address: {
                    required: Translator.trans('请输入活动地点'),
                    trim: Translator.trans('请输入活动地点'),
                },
                maxStudentNum: {
                    unsigned_integer:Translator.trans('请输入0-10000的整数'),
                    max: Translator.trans('请输入0-10000的整数'),
                    required: Translator.trans('请输入人数上限'),
                    trim: Translator.trans('请输入人数上限'),
                },
                startDate: {
                    required: Translator.trans('请输入开始时间'),
                    trim: Translator.trans('请输入开始时间'),
                },
                enrollmentEndDate: {
                    required: Translator.trans('请输入截止时间'),
                    trim: Translator.trans('请输入截止时间'),
                },
                endDate: {
                    required: Translator.trans('请输入结束时间'),
                    trim: Translator.trans('请输入结束时间'),
                }
            }
        });
        this.initDatePicker('#startDate');
        this.initDatePicker('#endDate');
        this.initDatePicker('#enrollmentEndDate');
        $('#requireAudit').change(function(){
            let requireAudit = $('[name="requireAudit"]');
            if ($('#requireAudit').is(':checked')) {
                requireAudit.val("1");
            } else {
                requireAudit.val("0");
            }
        });
        $('#offlineCourse-base-submit').click((event) => {

            if (validator.form()) {
                $(event.currentTarget).button('loading');
                $form.submit();
            }
        });

      let editor = CKEDITOR.replace('summary', {
        toolbar: 'Thread',
        filebrowserImageUploadUrl: $('#summary').data('imageUploadUrl'),
        allowedContent: true,
        height: 300
      });

      editor.on('change', () => {
        $('#summary').val(editor.getData());
      });

      editor.on('blur', () => {
        $('#summary').val(editor.getData());
      });
    }

    initDatePicker($id) {
        let $picker = $($id);
        $picker.datetimepicker({
            format: 'yyyy-mm-dd hh:ii',
            language: "zh",
            minView: 0, //month
            autoclose: true,
            endDate: new Date(Date.now() + 86400 * 365 * 10 * 1000)
        });
        $picker.datetimepicker('setStartDate', new Date());
    }
    initExpiryMode() {
        let $startDate = $('[name="startDate"]');
        let $endDate = $('[name="endDate"]');
        let $enrollmentEndDate = $('[name="enrollmentEndDate"]');

        this.elementRemoveRules($startDate);
        this.elementRemoveRules($endDate);
        this.elementRemoveRules($enrollmentEndDate);
    }

}

jQuery.validator.addMethod("startDate_check", function() {
    var startDate = $('[name="startDate"]').val();
    var timestamp = Date.parse(new Date());
    var date = new Date(startDate.replace(/-/g, '/'));
    return (timestamp <= date.getTime());
}, "开始时间大于或等于当前时间");

jQuery.validator.addMethod("endDate_check", function() {
    var startDate = $('[name="startDate"]').val();
    var endDate = $('[name="endDate"]').val();
    return (startDate <= endDate);
}, "结束时间大于或等于开始时间");

jQuery.validator.addMethod("enrollmentEndDate_before_check", function () {
    var enrollmentEndDate = $('[name="enrollmentEndDate"]').val();
    var endDate = $('[name="endDate"]').val();
    return (enrollmentEndDate <= endDate);
}, "截止时间小于或等于结束时间");

jQuery.validator.addMethod("enrollmentEndDate_after_check", function() {
    var enrollmentEndDate = $('[name="enrollmentEndDate"]').val();
    var timestamp = Date.parse(new Date());
    var date = new Date(enrollmentEndDate.replace(/-/g, '/'));
    return (timestamp <= date.getTime());
}, "截止时间大于或等于当前时间");



