import ajaxPageTask from './ajax-page-task';
import chapterAnimate from '../../common/chapter-animate';
import tabBlock from '../../common/tab-block';
import ajaxLink from '../../common/ajax-link';
import notify from 'common/notify';

class Page {
  constructor() {
    this.init();
  }
  init() {
    echo.init();
    
    this.initIntro();
    this.initAjax($('.js-course-item'));
    this.initToolTip();
    this.initEasyPieChart($('.js-chart'));
  }

  initAjax($element) {

    $element.each(function() {
      new ajaxPageTask({ element: $(this)}).init();
      new chapterAnimate({element: $(this)});
    });
  }

  initToolTip() {
    $('[data-toggle="tooltip"]').tooltip({
      container: 'body'
    });
  }

  initEasyPieChart($element) {
    let colorPrimary = $('.js-pie-chart-hover').css('color');
    $element.easyPieChart({
      animate: 2000,
      size: 60,
      barColor: colorPrimary,
      trackColor: '#e1e1e1',
      scaleLength: 0,
      lineWidth: 5,
      onStart: function() {
        $('.js-pie-chart-hover').removeClass('hidden');
      }
    });
  }

  initIntro() {
    let hasReadGuide = $('#step1').data('hasReadGuide');
    if(hasReadGuide) {
      return;
    }

    let lastStepElem;
    $('#nav').find('a').each((i, element) => {
      if ($(element).text() === "课程广场 ") {
        lastStepElem = $(element)[0];
      }
    })

    let intro = introJs();
    let stepContent = [
      {
        intro: '<p class="study-center-tooltip-title">欢迎你！</p><p style="font-size: 30px;">:)</p><p>在这里与大家一起学习、分享、成长</p>'
      },
      {
        element: '#step1',
        intro: '<p class="study-center-tooltip-title">我的任务</p><p  style="font-size: 16px;">这里是你的领导向你推送的相应学习任务</p>'
      },
      {
        element: '#step11',
        intro: '<p class="study-center-tooltip-title">我的活动</p><p  style="font-size: 16px;">这里展示你参加的相关活动</p>'
      },
      {
        element: '#step2',
        intro: '<p class="study-center-tooltip-title">我的课程</p><p  style="font-size: 16px;">这里可以方便的找到你加入过的全部课程</p>'
      },
      {
        element: '#step3',
        intro: '<p class="study-center-tooltip-title">学习档案</p><p  style="font-size: 16px;">这里能了解到你全部的学习历史、成绩等记录</p>'
      },
      {
        element: '#step4',
        intro: '<p class="study-center-tooltip-title">部门课程</p><p  style="font-size: 16px;">这里能找到专属于你所在部门的课程和专题</p>'
      }
    ];

    if (lastStepElem) {
      let finalStep = {
        element: lastStepElem,
        intro: '<p class="study-center-tooltip-title">课程广场</p><p  style="font-size: 16px;">这里可以找到全部公共课程，有空常来逛逛</p>'
      }
      stepContent.push(finalStep);
    }

    intro.setOptions({
      showBullets: false,
      nextLabel: '下一步 <i class="es-icon es-icon-arrowforward"></i>',
      skipLabel: '<i class="es-icon es-icon-close01"></i>',
      doneLabel: '开始学习',
      hidePrev: true,
      hideNext: true,
      tooltipClass: 'study-center-tooltip-intro',
      showStepNumbers: false,
      exitOnOverlayClick: false,
      steps: stepContent,
    });

    let completeGuide = function() {
      $.post($('#step1').data('completeGuideUrl'));
    };

    intro.onchange(function(elem) {
      if ( (!lastStepElem && elem == $('#step4')[0]) || elem == lastStepElem) {
        $('.introjs-button.introjs-skipbutton').addClass('is-last');
      } else {
        $('.introjs-button.introjs-skipbutton').removeClass('is-last');
      };
    }).onexit(function(elem) {
      completeGuide();
    }).oncomplete(function(elem) {
      completeGuide();
    }).start();
  }
}

new Page();

const $searchWrap = $('.js-search-wrap');
new ajaxLink({element: $searchWrap});

