import notify from 'common/notify';
export default class Create {
    constructor($element) {
        this.$element = $element;
        this.init();
        // this.initDatetimePicker();
    }

    init() {
        this.validator = this.$element.validate({
            currentDom: '#offline-course-create-btn',
            rules: {
                name: {
                    byte_maxlength: 40,
                    chinese_alphanumeric: true,
                    required: true,
                    trim: true,
                },
                targetType: {
                    required: {
                        depends () {
                            $(this).val($.trim($(this).val()));
                            return true;
                        }
                    }
                },
                targetId: {
                    required: {
                        depends () {
                            $(this).val($.trim($(this).val()));
                            return true;
                        }
                    }
                },
                startDateTime: {
                },
                endDateTime: {
                    endDate_check:true,
                },
            },
            messages: {
                name: {
                    required: Translator.trans('请输入项目名称'),
                    trim: Translator.trans('请输入项目名称'),
                },
                targetType: {
                    required: Translator.trans('请选择内容类型'),
                    trim: Translator.trans('请选择内容类型'),
                },
                targetId: {
                    required: Translator.trans('请选择内容'),
                    trim: Translator.trans('请选择内容'),
                }
            }
        });

        this.initDatePicker('#startDateTime');
        this.initDatePicker('#endDateTime');

        var $postsContainer = $('#changeModalTargetId');
        var targetType = $('#targetType').val();
        var url = $postsContainer.data('url');
        if (targetType == 'course') {
            url = $('[name=targetId]').data('course');
        }
        if (targetType == 'classroom') {
            url = $('[name=targetId]').data('classroom');
        }
        $postsContainer.select2({
            ajax: {
                url: url,
                dataType: 'json',
                quietMillis: 100,
                data: function (term, page) {
                    return {
                        q: term,
                        page_limit: 10
                    };
                },
                results: function (data) {
                    var results = [];
                    $.each(data, function (index, item) {

                        results.push({
                            id: item.id,
                            name: item.name + '，ID:' + item.id
                        });
                    });

                    return {
                        results: results
                    };

                }
            },
            initSelection (element, callback) {
                var id = $(element).val();
                var targetType = $('#targetType').val();
                var url = '';
                if (id !== "") {
                    if (targetType == 'course') {
                        url = $("#js-course-search").data('url')+'?id=' + id;
                    }
                    if (targetType == 'classroom') {
                        url = $("#js-classroom-search").data('url')+'?id=' + id;
                    }
                    if (url != '') {
                        $.ajax(url, {
                            dataType: "json"
                        }).success(function(resp) {
                            var data = ({
                                id: resp.id,
                                name: resp.name + '，ID:' + resp.id,
                            });
                            callback(data);
                        });
                    }
                }
            },
            formatSelection: function (item) {
                return item.name;
            },
            formatResult: function (item) {
                return item.name;
            },
            placeholder: Translator.trans('输入标题关键字或活动ID。')
        });

        $('.radios').on('click', "input[name=targetType]", function() {
            var selectedValue = $(this).attr('value');
            var $postsContainer = $('#changeModalTargetId');
            var url = $postsContainer.data('url');
            if (selectedValue == 'course') {
                url = $('[name=targetId]').data('course');
            } else if (selectedValue == 'classroom') {
                url = $('[name=targetId]').data('classroom');
            }

            $postsContainer.select2({
                ajax: {
                    url: url,
                    dataType: 'json',
                    quietMillis: 100,
                    data: function (term, page) {
                        return {
                            q: term,
                            page_limit: 10
                        };
                    },
                    results: function (data) {
                        var results = [];
                        $.each(data, function (index, item) {

                            results.push({
                                id: item.id,
                                name: item.name + '，ID:' + item.id
                            });
                        });

                        return {
                            results: results
                        };

                    }
                },
                initSelection (element, callback) {
                },
                formatSelection: function (item) {
                    return item.name;
                },
                formatResult: function (item) {
                    return item.name;
                },
                formatSearching: function() {
                    return Translator.trans('搜索中...');
                },
                placeholder: Translator.trans('输入标题关键字或活动ID。')
            });
        });

        $('#assignment-create-btn').on('click', (e) => {
            let $target = $(e.target);
            if (this.validator.form()) {
                $('#assignment-create-btn').button('loading').addClass('disabled');
                $.post(this.$element.attr('action'), this.$element.serialize(), function(html) {
                    notify('success', `项目成功创建!`);
                    window.location.href = '/training_assignment/'+html.id+'/member/manage/list';
                }).error(function(){
                    notify('danger', `项目创建失败!`);
                });
                return false;
            }
        });
    }

    initDatePicker($id) {
        let $picker = $($id);
        $picker.datetimepicker({
            format: 'yyyy-mm-dd',
            language: "zh",
            minView: 'month',
            autoclose: true,
            endDate: new Date(Date.now() + 86400 * 365 * 10 * 1000)
        });
        $picker.datetimepicker('setStartDate', new Date());
    }

}

jQuery.validator.addMethod("endDate_check", function() {
    var startDate = $('[name="startDateTime"]').val();
    var endDate = $('[name="endDateTime"]').val();
    return (startDate <= endDate);
}, "结束时间大于或等于开始时间");
