import CheckTreeviewInput from '../../common/CheckTreeview-input';
import notify  from 'common/notify';

jQuery.validator.addMethod("endDate_check", function () {
    var startDate = $('[name="startDateTime"]').val();
    var endDate = $('[name="endDateTime"]').val();
    return (startDate <= endDate);
}, "大于或等于开始时间");

class Page {
    constructor() {
        this.init();
    }

    init() {
        this.initTreeViewInput();
        this.initSelect();
        this.initDatetimePicker();
        this.initvalidator();
        this.initOperate();
    }

    initSelect() {
        $('[data-role="tree-select"], [name="categoryId"]').select2({
            treeview: true,
            dropdownAutoWidth: true,
            treeviewInitState: 'collapsed',
            placeholderOption: 'first',
            allowClear: true,
        });
    }

    initDatetimePicker() {
        const defaultOpts = {
            language: "zh",
            autoclose: true,
            format: 'yyyy-mm-dd',
            minView: 'month'
        };
        $('[name=startDateTime]').datetimepicker(defaultOpts);

        $('[name=endDateTime]').datetimepicker(defaultOpts);

        $('[name=startDateTime]').on('changeDate',function(){
            $('[name=endDateTime]').datetimepicker('setStartDate',$('[name=startDateTime]').val().substring(0,16));
        });

        $('[name=endDateTime]').on('changeDate',function(){
            $('[name=startDateTime]').datetimepicker('setEndDate',$('[name=endDateTime]').val().substring(0,16));
        });
    }

    initvalidator() {
        var $postContainer = $('#postId');
        $postContainer.select2({
            ajax: {
                url: $postContainer.data('url'),
                dataType: 'json',
                quietMillis: 100,
                data: function (term, page) {
                    return {
                        q: term,
                        page_limit: 10
                    };
                },
                results: function (data) {
                    var results = [];
                    $.each(data, function (index, item) {

                        results.push({
                            id: item.id,
                            name: item.name
                        });
                    });

                    return {
                        results: results
                    };

                }
            },
            initSelection: function(element, callback) {
                var id = $(element).val();
                if (id !== "") {
                    var name = $(element).data('name');
                    callback({id:id, name:name});
                }
            },
            formatSelection: function (item) {
                return item.name;
            },
            formatResult: function (item) {
                return item.name;
            },
            allowClear: true,
            width: 'off',
            placeholder: Translator.trans('所有岗位')
        });

    }

    initTreeViewInput() {
        let $treeview = $('.js-treeview-select-wrap');
        for (let i = 0; i < $treeview.length; i++) {
            new CheckTreeviewInput({ $elem: $treeview.eq(i) })
        }
    }

    initOperate() {
        $('.assignment-operate').click(function () {

            let notifyTitle = $(this).data('notifyTitle');
            if (!confirm(`确定要${notifyTitle}`)) {
                return;
            }

            $.get($(this).data('url'), function(result){
                if (result) {
                    notify('success', `${notifyTitle}成功!`);
                    window.location.reload();
                } else {
                    notify('danger', `${notifyTitle}失败!`);
                }
            }).error(function() {
                notify('danger', `${notifyTitle}失败!`);
            });
        });
    }
}

new Page();