<?php

namespace CorporateTrainingBundle\Twig\Extension;

use Codeages\Biz\Framework\Context\Biz;

class WebExtension extends \Twig_Extension
{
    protected $container;

    protected $biz;

    public function __construct($container, Biz $biz)
    {
        $this->container = $container;
        $this->biz = $biz;
    }

    public function getFilters()
    {
        return array(
            new \Twig_SimpleFilter('timeFormatBySecond', array($this, 'timeFormatReturnBySecondFilter')),
            new \Twig_SimpleFilter('time_format', array($this, 'timeFormatReturnHour')),
            new \Twig_SimpleFilter('time_format_hour', array($this, 'timeFormatHour')),
            new \Twig_SimpleFilter('time_judgment', array($this, 'timeJudgmentFilter')),
            new \Twig_SimpleFilter('weekday', array($this, 'timeFormatReturnWeekday')),
            new \Twig_SimpleFilter('highlight_keyword', array($this, 'searchKeywordFilter')),
        );
    }

    public function getFunctions()
    {
        return array(
            new \Twig_SimpleFunction('all_post_choices', array($this, 'allPostChoices')),
            new \Twig_SimpleFunction('build_org_tree', array($this, 'buildOrgTreeByCodes')),
            new \Twig_SimpleFunction('avg_learn_time', array($this, 'avgLearnTime')),
            new \Twig_SimpleFunction('is_ding_talk', array($this, 'isDingTalk')),
            new \Twig_SimpleFunction('can_manage_course', array($this, 'canManageCourse')),
            new \Twig_SimpleFunction('is_user_assignment_course', array($this, 'isUserAssignmentCourse')),
        );
    }

    public function timeFormatReturnWeekday($time)
    {
        if (is_numeric($time)) {
            $weekday = array('星期日', '星期一', '星期二', '星期三', '星期四', '星期五', '星期六');

            return $weekday[date('w', $time)];
        }

        return false;
    }

    public function timeJudgmentFilter($time)
    {
        if (empty($time)) {
            return;
        }

        $hour = date('H', $time);
        if ($hour >= 0 && $hour < 12) {
            return '上午好，';
        }

        if ($hour >= 12 && $hour < 18) {
            return '下午好，';
        }

        if ($hour >= 18 && $hour < 24) {
            return '晚上好，';
        }
    }

    public function timeFormatReturnHour($learnTime)
    {
        if (empty($learnTime)) {
            return 0;
        }

        return substr(sprintf('%.2f', $learnTime / 3600), 0, -1);
    }

    public function timeFormatHour($learnTime)
    {
        return substr(sprintf('%.2f', $learnTime / 3600), 0, -1).'小时';
    }

    public function avgLearnTime($totalLearnTime, $userCount)
    {
        if (empty($totalLearnTime) || empty($userCount)) {
            return 0;
        }

        return floor($totalLearnTime / $userCount);
    }

    public function canManageCourse($courseId)
    {
        return $this->getCourseService()->canManageCourse($courseId);
    }

    public function searchKeywordFilter($content, $keyword)
    {
        if (empty($keyword)) {
            return $content;
        }

        $text = str_replace($keyword, "<span style='color:#ff0000'>".$keyword.'</span>', $content);
        echo $text;
    }

    public function allPostChoices()
    {
        $allPosts = $this->getPostService()->getAllPosts();
        $choices = array();

        foreach ($allPosts as $post) {
            $choices[$post['id']] = $post['name'];
        }

        return $choices;
    }

    /*
    * 查询返回秒钟显示格式处理
    */
    public function timeFormatReturnBySecondFilter($learnTime)
    {
        $hour = floor($learnTime / 3600);
        $minute = floor(($learnTime % 3600) / 60);
        if ($hour == 0 && $minute != 0) {
            $time = $minute.'分钟';
        } elseif ($minute == 0 && $hour == 0) {
            $time = '0';
        } elseif ($minute == 0 && $hour != 0) {
            $time = $hour.'小时';
        } else {
            $time = $hour.'小时'.$minute.'分钟';
        }

        return $time;
    }

    public function buildOrgTreeByCodes($orgCodes)
    {
        $tree = $this->getOrgService()->buildVisibleOrgTreeByOrgCodes($orgCodes);

        return json_encode($tree);
    }

    public function isDingTalk()
    {
        return strpos($this->container->get('request')->headers->get('User-Agent'), 'DingTalk') !== false;
    }

    public function isUserAssignmentCourse($courseId, $userId)
    {
        $user = $this->getUserService()->getUser($userId);
        if (empty($user)) {
            return false;
        }

        $isUserPostCourse = $this->getUserPostCourseService()->isCourseBelongToUserPostCourse($courseId, $user);

        $isTrainingAssignmentCourse = $this->getTrainingAssignmentMemberService()->isBelongToUserAssignment($user['id'], $courseId, 'course');

        return $isTrainingAssignmentCourse || $isUserPostCourse;
    }

    protected function getUserService()
    {
        return $this->createService('User:UserService');
    }

    protected function getPostService()
    {
        return $this->createService('CorporateTrainingBundle:Post:PostService');
    }

    protected function getPostCourseService()
    {
        return $this->createService('PostCourse:PostCourseService');
    }

    protected function getCourseService()
    {
        return $this->createService('Course:CourseService');
    }

    protected function getOrgService()
    {
        return $this->createService('Org:OrgService');
    }

    protected function getTrainingAssignmentMemberService()
    {
        return $this->createService('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentMemberService');
    }

    protected function getUserPostCourseService()
    {
        return $this->createService('CorporateTrainingBundle:PostCourse:UserPostCourseService');
    }

    protected function createService($alias)
    {
        return $this->biz->service($alias);
    }
}
